#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: FtuFtt.py,v 1.1 2004/09/06 14:38:37 cguirao Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  05/02/2001  Created
#

"""
The FtuFtt class is used to handle FTTs programmatically.

It is thus possible from within Python to build up and manipulate FTTs.
This makes it possible to add dynamic behavior to the FTTs which are
normally relative static definitions of the translations to carry out.
"""


import os, sys, string, exceptions, types

from PccLog import *
import PccPaf, PccUtString, PccKey, PccUtUtils, PccUtTime
import FtuFttActionBase, FtuFttAction, FtuFttDicRec
import FtuFttFilePath, FtuFttFileRename, FtuFttFileSplit



class FtuFtt:
    """
    The class handles FTT  (FITS Translation Table) files which are used
    by the FITS Translation Tool for doing processing of the FITS headers
    of FITS files.

    It is posible to load and save FTT files, and to manipulate their
    contents.

    For further information about the issue of FTTs, consult the URL:
    http://archive.eso.org/ftu.
    """


    def __init__(self):
        """
        Constructor method initializing the class variables.
        """
        self.setIncludeRefList([])
        self.setDicRefList([])
        self.setDicRecList([])
        self.__filePathAction    = FtuFttFilePath.FtuFttFilePath()
        self.__fileRenameAction  = FtuFttFileRename.FtuFttFileRename()
        self.__fileSplitAction   = FtuFttFileSplit.FtuFttFileSplit()
        self.setHdrDump("")
        self.setActionList([])


    def __handlePccKeyList(self,
                           list,
                           key,
                           parameter,
                           classMethod):
        """
        Private method to handle the conversion from a list of strings
        (or partially consisting of strings), to contain only PccKey
        objects.

        list:              Source list to be checked and possible converted.
        
        key:               The keyword for the PccKey object, e.g. "INCLUDE"
                           or "DICTIONARY".
        
        parameter:         Name of the input parameter of the method calling
                           this method, referring to the list.
        
        classMethod:       Name of the class + method from where this
                           method is called.

        Returns:           Converted/checked list of PccKey objects.
        """
        tmpList = []
        for el in list:
            if (isinstance(el, types.StringTypes)):
                tmpKey = PccKey.PccKey(list, el)
            elif (isinstance(el, PccKey.PccKey)):
                tmpKey = el
            else:
                raise exceptions.Exception, \
                      "Wrong type of element found in \"" + parameter + "\" "+\
                      "list, given to method \"" + classMethod + "\"."
            tmpList.append(tmpKey)
        return tmpList
    

    def setIncludeRefList(self,
                          includeRefList):
        """
        Set the list referring to the included FTTs. If there is already
        referred to a list, the contents of the old list is 'forgotten'.

        includeRefList:   Reference to list containing PccKey objects where
                          the key is set to "INCLUDE" and the key value to
                          the FTT to include. It is also possible to give
                          in a list of strings with the names of the files
                          to include. This will then be converted to PccKey
                          objects internally.

        Returns:          Reference to object itself.
        """
        self.__includeRefs = self.\
                             __handlePccKeyList(includeRefList,
                                                "INCLUDE",
                                                "includeRefList",
                                                "FtuFtt.setIncludeRefList()")
        return self


    def getIncludeRefList(self):
        """
        Return reference to the internal list with included FTTs. See also
        setIncludeRefList().

        Returns:       Reference to list with included FTTs.
        """
        return self.__includeRefs


    def getNoOfIncludeRefs(self):
        """
        Return the number of included FTTs.

        Returns:      Number of included FTTs.
        """
        return len(self.__includeRefs)


    def getIncludeRef(self,
                      no):
        """
        Return a specific reference to an included FTT.

        no:         Number (index) of the specific FTT (first has number 0).

        Returns:    PccKey object with the information about the included
                    FTT.
        """
        return self.__includeRefs[no]


    def addIncludeRef(self,
                      includeRef):
        """
        Add a reference to an included FTT. This can be added either
        as a string or as a PccKey object.

        includeRef:   Referece to PccKey object or string containing the
                      information about the included FTT.

        Returns:      Reference to object itself.
        """
        if (isinstance(includeRef, types.StringType)):
            incRefKey = PccKey.PccKey("INCLUDE", includeRef)
        elif (isinstance(includeRef, PccKey.PccKey)):
            incRefKey = includeRef
        else:
            PccUtUtils.raiseTypeException("includeRef",
                                          "FtuFtt.addIncludeRef()",
                                          PccKey.PccKey)
        self.__includeRefs.append(incRefKey)
        return self
        

    def setDicRefList(self,
                      dicRefList):
        """
        Set the list referring to the included DIDs. If there is already
        referred to a list, the contents of the old list is 'forgotten'.

        dicRefList:       Reference to list containing PccKey objects where
                          the key is set to "DICTIONARY" and the key value to
                          the FTT to include. It is also possible to give
                          in a list of strings with the names of the DIDs
                          to include. This will then be converted to PccKey
                          objects internally.

        Returns:          Reference to object itself.
        
        """
        self.__dicRefs = self.__handlePccKeyList(dicRefList,
                                                 "DICTIONARY",
                                                 "dicRefList",
                                                 "FtuFtt.setDicRefList()")
        return self
    
    
    def getDicRefList(self):
        """
        Return reference to the internal list with included DIDs. See also
        setDicRefList().

        Returns:       Reference to list with included DIDs.
        """
        return self.__dicRefs


    def getNoOfDicRefs(self):
        """
        Return the number of included DIDs.

        Returns:      Number of included DIDs.
        """
        return len(self.getDicRefList())


    def getDicRef(self,
                  no):
        """
        Return a specific reference to an included DID.

        no:         Number (index) of the specific DID (first has number 0).

        Returns:    PccKey object with the information about the included DID.
        """
        return self.getDicRefList()[no]


    def addDicRef(self,
                  dicRef):
        """
        Add a reference to an included DID. This can be added either
        as a string or as a PccKey object.

        dicRef:       Referece to PccKey object or string containing the
                      information about the included DID.

        Returns:      Reference to object itself.
        """
        if (isinstance(dicRef, types.StringType)):
            dicRefKey = PccKey.PccKey("DICTIONARY", dicRef)
        elif (isinstance(dicRef, PccKey.PccKey)):
            dicRefKey = dicRef
        else:
            PccUtUtils.raiseTypeException("dicRef", "FtuFtt.addDicRef()",
                                          PccKey.PccKey)
        self.__dicRefs.append(dicRefKey)
        return self


    def setDicRecList(self,
                      dicRecList):
        """
        Set the list referring to the DID records defined in the FTT.
        The "dicRecList" is a list containing FtuFttDicRec objects.

        dicRecList:  List containing the FtuFttDicRec objects.

        Returns:     Reference to object itself.
        """
        for el in dicRecList:
            PccUtUtils.checkType("dicRecList", el, "FtuFtt.setDicRecList()",
                                 FtuFttDicRec.FtuFttDicRec)
        if dicRecList:
            self.__dicRecList = dicRecList
        else:
            self.__dicRecList = []
        return self


    def getDicRecList(self):
        """
        Return reference to the list containing the FtuFttDicRec objects.

        Returns:     Reference to list with the FtuFttDicRec objects.
        """
        return self.__dicRecList


    def addDicRec(self,
                  dicRec):
        """
        Add a FtuFttDicRec object to the class.

        dicRec:    Reference to the FtuFttDicRec object.

        Returns:   Reference to the object itself.
        """
        PccUtUtils.checkType("dicRec", dicRec, "FtuFtt.addDicRec()",
                             FtuFttDicRec.FtuFttDicRec)
        self.__dicRecList.append(dicRec)
        return self
        
    
    def getFilePathAction(self):
        """
        Return reference to the File Path Action object (FtuFttFilePath
        object).

        Returns:    Reference to the internal FtuFttFilePath Action object.
        """
        return self.__filePathAction
    

    def setHdrDump(self,
                   hdrDump):
        """
        Set the Header Dump Action. This can either be as a string
        ("NEWLINE", "NO_NEWLINE"), or as a PccKey object with key set
        to "HDR.DUMP" and one of the values listed previously.

        hdrDump:     PccKey object or string with the information about the
                     Header Dump Action.
        
        Returns:     Reference to object itself.
        """
        if (isinstance(hdrDump, types.StringType)):
            hdrDumpKey = PccKey.PccKey("HDR.DUMP", hdrDump)
        elif (isinstance(hdrDump, PccKey.PccKey)):
            hdrDumpKey = hdrDump
        else:
            raise exceptions.Exception, \
                  "Wrong type of parameter \"hdrDump\" for method " +\
                  "\"FtuFtt.setHdrDump()\"."
        self.__hdrDump = hdrDumpKey
        return self
        

    def getHdrDump(self):
        """
        Return reference to the Header Dump PccKey object.

        Returns:   Reference to Header Dump PccKey object.
        """
        return self.__hdrDump


    def getFileRenameAction(self):
        """
        Return reference to the File Rename Action object (FtuFttFileRename
        object).

        Returns:    Reference to the internal FtuFttFileRename Action object.
        """
        return self.__fileRenameAction

    
    def getFileSplitAction(self):
        """
        Return reference to the File Split Action object (FtuFttFileSplit
        object).

        Returns:    Reference to the internal FtuFttFileSplit Action object.
        """
        return self.__fileSplitAction


    def setActionList(self,
                      actionList):
        """
        Make the internal list of FtuFttAction objects refer to a new,
        externally generated list.

        actionList:    Refers to list of FtuFttAction objects.

        Returns:       Reference to object itself.
        """
        for el in actionList:
            PccUtUtils.checkType("actionList", el, "FtuFtt.setActionList()",
                                 FtuFttAction.FtuFttAction)
        if actionList:
            self.__actionList = actionList
        else:
            self.__actionList = []
        return self


    def getActionList(self):
        """
        Return reference to internal Action List.

        Returns:    Reference to internal Action List.
        """
        return self.__actionList


    def load(self,
             fttFileName):
        """
        Loads an FTT PAF file into this instance of the FttFtu class.

        fttFileName:  Complete name of the FTT file to load.

        Returns:      Reference to object itself.
        """
        tmpFtt = PccPaf.PccPaf(1)
        tmpFtt.load(fttFileName)

        keyCount = 0
        while (keyCount < tmpFtt.getNoOfKeys()):
            keyObj = tmpFtt.getKeyObjNo(keyCount)
            keyNm = keyObj.getKey()
            
            if (keyNm == "INCLUDE"):
                self.getIncludeRefList().append(keyObj)
                keyCount = keyCount + 1
            elif (keyNm == "DICTIONARY"):
                self.getDicRefList().append(keyObj)
                keyCount = keyCount + 1
            elif (keyNm == "DIC.KEY"):
                didRec = FtuFttDicRec.FtuFttDicRec()
                keyCount = didRec.setFromFttPaf(keyCount, tmpFtt)
                self.getDicRecList().append(didRec)
            elif (keyNm == "HDR.DUMP"):
                self.setHdrDump(keyObj.getValue())
                keyCount = keyCount + 1
            elif (keyNm == "FILE.PATH.BASE"):
                keyCount = self.getFilePathAction().\
                           setFilePathFromFtt(keyCount, tmpFtt)
            elif (keyNm == "FILE.RENAME.PREFIX"):
                keyCount = self.getFileRenameAction().\
                           setFileRenameFromFtt(keyCount, tmpFtt)
            elif (keyNm == "FILE.SPLIT.PREFIX"):
                keyCount = self.getFileSplitAction().\
                           setFileSplitFromFtt(keyCount, tmpFtt)
            elif ((keyNm == "ADD.KEY") or \
                  (keyNm == "CHANGE.KEY") or \
                  (keyNm == "RENAME.SRCKEY") or \
                  (keyNm == "MOVE.KEY") or \
                  (keyNm == "COPY.SRCKEY") or \
                  (keyNm == "REMOVE.KEYLIST")):
                # Retrieve an 'Action Block' from the FTT.
                action = FtuFttAction.FtuFttAction()
                keyCount = action.setActionFromFtt(keyCount, tmpFtt)
                self.getActionList().append(action)
            else:
                raise exceptions.Exception, \
                      "Illegal keyword: " + keyNm + " found " +\
                      "in FTT: " + fttFileName + "."
        return self


    def getNoOfActions(self):
        """
        Return the number of Translation Actions contained in the object.

        Returns:   Number of Translation Actions (integer).
        """
        return len(self.getActionList())


    def getActionsSameKey(self,
                          action,
                          key):
        """
        Return a list with PccKey objects for the same type of Translation
        Actions manipulating a specific key (maybe in different headers).

        action:    Translation Action Type.
        
        key:        Name of the key to look for.
        """
        retList = []
        for act in self.getActionList():
            if ((act.getAction() == action) and (act.getKey() == key)):
                retList.append(act)
        return retList
    

    def getActionNo(self,
                    no):
        """
        Return a specific Translation Action according to its number.

        no:         Number of Translation Action in object. First key is
                    number 0.
                    
        Returns:    A PccKey object for the requested keyword.
        """
        return self.getActionList()[no]
    

    def save(self,
             targetFttName):
        """
        Save the FTT keywords contained in the object into the 
        referenced FTT (PAF) file.

        targetFttName: Filename of FTT to generate (complete path).

        Returns:       Reference to the object itself.
        """
        fd = open(targetFttName, "w")
        fd.write(self.dumpBuf())
        fd.close()
        return self


    def addAction(self,
                  fttActionObj):
        """
        Add a Translation Action in the object.

        fttActionObj: Reference to the Translation Action object
                      (FtuFttAction).

        Returns:      Reference to the objct itself.
        """
        self.getActionList().append(fttActionObj)
        return self


    def execute(self,
                fileList,
                fttName = "",
                logFile = "",
                logLevel = 0,
                tolerant = 0,
                force = 0,
                test = 0,
                noStat = 0,
                noChecksum = 0,
                noHistory = 0,
                noComment = 1,
                overwrite = 0,
                targetDir = "",
                targetFile = "",
                skipFileOnError = 0,
                timeOut = -1):
        """
        Execute the FTU with the FTT contained in this object.

        fileList:         List of files to process. Should be given in the
                          format: "<file> <file> <file> ...".

        fttName:          Name of the FTT which is generated by the class
                          and used as input to the FTU. If set to ""
                          a name will be generated.
        
        logFile:          Name of a log file if desriable that produces
                          log output into a file. Note, the Log Level should
                          be set with the parameter "logLevel".
        
        logLevel:         The intensity with which FTU should produce
                          log output (0..5, 5 is the most intensive).
        
        tolerant:         Run the FTU in Tolerant Mode ignoring certain
                          errors (0|1).
        
        force:            Force the translation of a FITS file, even though
                          it has already been translated by the FTU with
                          the same FTT (0|1).
                          
        test:             Generate test target files, i.e. FITS files with
                          "_test" appended to the name (0|1).
        
        noStat:           Don't generate statistics for the images in the
                          FITS file(s) (0|1).
        
        noChecksum:       Don't generate and write the CHECKSUM in the
                          FITS files (0|1).
        
        noHistory:        Don't generate HISTORY entries for modifications
                          introduced in the FITS headers (0|1).
        
        noComment:        Don't write the FTU COMMENT indicating that this
                          FITS file was translated by the FTU with the given
                          FTT (0|1). 
        
        overwrite:        Overwrite target files even if they exist.
                          Normally if not specified, if a target file exists,
                          the first free index (4 digit number) is found
                          and appended to the target filenames (0|1). 
        
        targetDir:        Name of Target Directory. This overwrites a
                          possible definition of this in the FTT.
        
        targetFile:       Base name of target file. This overwrites a possible
                          definition of this in the FTT.
        
        skipFileOnError:  Indicates to skip a file if an error occurs (0|1).
        
        timeOut:          Timeout to apply when translating the file. A
                          timeout of "-1" indicates that no timeout should
                          be used.  

        Returns:          Void.
        """

        # Dump FTT into temporary FTT file.
        if (fttName == ""):
            fttFileName = "/tmp/FtuFtt_" + \
                          PccUtTime.TimeStamp().getTimeStamp() + ".ftt"
        else:
            fttFileName = fttName
            if (string.find(fttFileName, ".ftt") == -1):
                fttFileName = fttFileName + ".ftt"
        self.save(fttFileName)
        os.system("chmod a+rw " + fttFileName)

        # Build up FTU command line.
        cmd = "fitsTranslate"
        if (logFile != ""):        cmd = cmd + " -logFile " + logFile
        if (logLevel != 0):        cmd = cmd + " -logLevel " + str(logLevel)
        if (tolerant != 0):        cmd = cmd + " -tolerant"
        if (force != 0):           cmd = cmd + " -force" 
        if (test != 0):            cmd = cmd + " -test"
        if (noStat != 0):          cmd = cmd + " -noStat"
        if (noChecksum != 0):      cmd = cmd + " -noChecksum"
        if (noHistory != 0):       cmd = cmd + " -noHistory"
        if (noComment != 0):       cmd = cmd + " -noComment"
        if (overwrite != 0):       cmd = cmd + " -overwrite"
        if (targetDir != ""):      cmd = cmd + " -targetDir " + targetDir
        if (targetFile != ""):     cmd = cmd + " -targetFile " + targetFile
        if (skipFileOnError != 0): cmd = cmd + " -skipFileOnError"
        cmd = cmd + " -ftt " + fttFileName + " " + fileList
        
        # Execute the FTU.
        info(3,"Executing FTU with command: " + cmd)
        res = PccUtUtils.execCmd(cmd, timeOut)
        if (res[2] != ""):
            raise exceptions.Exception, \
                  "Problem executing the FTU with command: " + cmd +\
                  " Error message: " + res[2] + "."


    def dumpBuf(self):
        """
        Dump the contents of the object into a string buffer.

        Returns:      Generated buffer with the keywords of the FTT.
        """
        retBuf = ""

        # Dump possible INCLUDE statements.
        for incRef in self.getIncludeRefList():
            if (retBuf != ""): retBuf = retBuf + "\n"                
            retBuf = retBuf + '%-27s "%s"' % ("INCLUDE", incRef.getValue())

        # Dump possible Dictionary Reference Definitions.
        if (len(self.getDicRefList()) > 0): retBuf = retBuf + "\n"
        for dicRef in self.getDicRefList():
            if (retBuf != ""): retBuf = retBuf + "\n"
            retBuf = retBuf + '%-27s "%s"' % ("DICTIONARY", dicRef.getValue())

        # Dump Dictionary Record Definitions.
        for didRec in self.getDicRecList():
            if (retBuf != ""): retBuf = retBuf + "\n\n"
            retBuf = retBuf + didRec.dumpBuf()

        # Dump a possible Header Dump Definition.
        if (self.getHdrDump().getValue() != ""):
            if (retBuf != ""): retBuf = retBuf + "\n\n"
            retBuf = retBuf + '%-27s "%s"' % ("HDR.DUMP",
                                              self.getHdrDump().getValue())
            
        # Dump a possible File Path Definition.
        if (self.getFilePathAction().isDefined() == 1):
            retBuf = retBuf + "\n\n" + self.getFilePathAction().dumpBuf()
            
        # Dump a possible File Rename Definition.
        if (self.getFileRenameAction().isDefined() == 1):
            retBuf = retBuf + "\n\n" + self.getFileRenameAction().dumpBuf()

        # Dump a possible File Split Definition.
        if (self.getFileSplitAction().isDefined() == 1):
            retBuf = retBuf + "\n\n" + self.getFileSplitAction().dumpBuf()
        
        # Dump Actions.
        for action in self.__actionList:
            retBuf = retBuf + "\n\n" + action.dumpBuf()
        retBuf = retBuf + "\n\n# --- oOo ---"
        return retBuf


# Main/test program.
if __name__ == '__main__':
    """
    Small test main function which can load in an FTT and dumps it to
    stdout.
    """
    if (len(sys.argv) != 2):
        print "\n\nCorrect usage:\n"
        print "% FtuFtt <FTT file>\n\n"
        sys.exit(1)
    ftt = FtuFtt()
    ftt.load(sys.argv[1])
    print ftt.dumpBuf()


#
# ___oOo___
