#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: FtuFttAction.py,v 1.1 2004/09/06 14:38:37 cguirao Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  06/02/2001  Created
#

"""
Class to handle Translation Actions.

This class is used to handle the Keyword Translations Actions which
are as follows:

    o Add
    
    o Change
    
    o Rename
    
    o Move
    
    o Copy
    
    o Remove
    
Note that the class is inherited from the FtuFttActionBase base class
and methods provided by this may be usefull at the applications level.
"""

import exceptions, types
import PccPaf, PccUtUtils, PccUtString
from   FtuFttDefs import *
import FtuFttValueSrc, FtuFttUt, FtuFttActionBase


class FtuFttAction(FtuFttActionBase.FtuFttActionBase):
    """
    Class to contain the keywords of one Translation Action.

    General comments to usage of class:

        o All keywords must be given in the Short-FITS format.
        
        o First FITS header has the number 0.

        o For the Translation Actions the following 'constant' names
          defined are:
                       
            o ftuACTION_UNDEF
          
            o ftuACTION_ADD
          
            o ftuACTION_CHANGE
          
            o ftuACTION_RENAME
          
            o ftuACTION_MOVE
          
            o ftuACTION_COPY
          
            o ftuACTION_REMOVE

          These are defined in the module "FtuFttDefs".
    """


    def __init__(self,
                 action       = ftuACTION_UNDEF,
                 srcKey       = "",
                 srcHdr       = -1,
                 trgKey       = "",
                 trgHdr       = -1,
                 valueSrcList = None,
                 fct          = ""):
        """
        Constructor method to initialize the object.

        It it possible to set the various fields of the Translation Action
        contained in the object.

        action:        Type of Translation Action. The actions are
                       defined in the module FtuFttAction.
 
        srcKey:        Source keyword for the translation.
        
        srcHdr:        Source header for the translation.
        
        trgKey:        Target keyword name.
         
        trgHdr:        Target header for the translation. 
        
        valueSrcList:  Python List of FtuFttValueSrc objects.
        
        fct:           The Translation Action Conversion Function.
        """
        FtuFttActionBase.FtuFttActionBase.__init__(self)
        self.setAction(action)
        self.setSrcKey(srcKey)
        self.setSrcHdr(srcHdr)
        self.setTrgKey(trgKey)
        self.setTrgHdr(trgHdr)
        self.setValueSrcList(valueSrcList)
        self.setFct(fct)
 
        
    def setAddAction(self,
                     key,
                     hdr = 0,
                     valueSrcList = None,
                     fct = ""):
        """
        Set up the object for an Add Translation Action.
        
        key:          Keyword which to add.
        
        hdr:          FITS header in which the keyword should be added.
        
        valueSrcList: Python List of FtuFttValueSrc objects.
        
        fct:          Translation Action Conversion Function to invoke
                      when doing the translation.

        Returns:      Reference to object itself.
        """
        self.setAction(ftuACTION_ADD)
        self.setSrcKey(key)
        self.setHdr(hdr)
        if valueSrcList:
            self.setValueSrcList(valueSrcList)
        else:
            self.setValueSrcList([])
        self.setFct(fct)
        return self


    def setChangeAction(self,
                        key,
                        hdr = 0,
                        valueSrcList = None,
                        fct = ""):
        """
        Set up the object for a Change Translation Action.

        key:          Keyword which to add.
        
        hdr:          FITS header in which the keyword should be added.
        
        valueSrcList: Python List of FtuFttValueSrc objects.
        
        fct:          Translation Action Conversion Function to invoke
                      when doing the translation.

        Returns:      Reference to object itself.        
        """
        self.setAction(ftuACTION_CHANGE)
        self.setSrcKey(key)
        self.setSrcHdr(hdr)
        self.setValueSrcList(valueSrcList)
        self.setFct(fct)
        return self


    def setRenameAction(self,
                        srcKey,
                        trgKey,
                        hdr = 0,
                        fct = ""):
        """
        Set up the object for a Rename Translation Action.

        srcKey:      Keyword to be renamed.
        
        trgKey:      New name for the keyword to be renamed.
        
        hdr:         Header in which the keyword is contained.
        
        fct:         Function used to convert the keyword before
                     writing it back to the FITS header.

        Returns:     Reference to object itself.
        """
        self.setAction(ftuACTION_RENAME)
        self.setSrcKey(srcKey)
        self.setSrcHdr(hdr)
        self.setTrgKey(trgKey)
        self.setTrgHdr(hdr)
        self.setFct(fct)
        return self
        
    
    def setMoveAction(self,
                      key,
                      srcHdr,
                      trgHdr,
                      fct = ""):
        """
        Set up the object for a Move Translation Action.

        key:         Keyword to be moved.
         
        srcHdr:      Source header from which the keyword should be taken.
        
        trgHdr:      Target header to which the keyword should be moved.
        
        fct:         Function to convert the keyword before writing it
                     back to the FITS header.

        Returns:     Reference to object itself.
        """
        self.setAction(ftuACTION_MOVE)
        self.setSrcKey(key)
        self.setSrcHdr(srcHdr)
        self.setTrgKey(key)
        self.setTrgHdr(trgHdr)
        self.setFct(fct)
        return self


    def setCopyAction(self,
                      srcKey,
                      srcHdr,
                      trgKey,
                      trgHdr,
                      fct = ""):
        """
        Set up the object for a Copy Translation Action.
        
        srcKey:      Keyword to be copied.
        
        srcHdr:      Header from which to take the keyword.
        
        trgKey:      Name of target keyword.
        
        trgHdr:      Header in which the copied keyword should be stored.
        
        fct:         Function to convert the keyword before writing it in
                     the target header.

        Returns:     Reference to object itself.
        """
        self.setAction(ftuACTION_COPY)
        self.setSrcKey(srcKey)
        self.setSrcHdr(srcHdr)
        self.setTrgKey(trgKey)
        self.setTrgHdr(trgHdr)
        self.setFct(fct)
        return self
   

    def setRemoveAction(self,
                        keyList,
                        hdr = 0):
        """
        Set up the object for a Copy Translation Action.

        keyList:     Comma separated list of keyword names for keys that
                     should be removed.
        
        hdr:         Header from which to remove the keyword(s).

        Returns:     Reference to object itself.
        """
        self.setAction(ftuACTION_REMOVE)
        self.setSrcKey(keyList)
        self.setSrcHdr(hdr)
        return self
        
                  
    def __setAddChangeActionFromFtt(self,
                                    action,
                                    keyCount,
                                    fttPafObj):
        """
        Extract an Add or Change Translation Action from an FTT.

        action:         Action to extract: FtuFttAction.ftuACTION_ADD or
                        FtuFttAction.ftuACTION_CHANGE.
        
        keyCount:       The present key count.
        
        fttPafObj:      Reference to the FTT Object.

        Returns:        Key counter in the FTT object.
        """
        self.setAction(action)
            
        # Handle <action>.KEY.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             action + ".KEY")
        self.setKey(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle <action>.HDR.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             action + ".HDR")
        self.setHdr(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle value sources (optional).
        if (keyCount < fttPafObj.getNoOfKeys()):
            keyObj = fttPafObj.getKeyObjNo(keyCount)
        while ((keyCount < fttPafObj.getNoOfKeys()) and \
               ((keyObj.getKey() == action + ".VALUE.KEY") or
                (keyObj.getKey() == action + ".VALUE.CONST") or
                (keyObj.getKey() == action + ".VALUE.FCT"))):
            valueSrc = FtuFttValueSrc.FtuFttValueSrc()
            valueSrc.setFromPafLine(keyObj.getKeywordLine(27))
            self.addValueSrc(valueSrc)
            keyCount = keyCount + 1
            if (keyCount < fttPafObj.getNoOfKeys()):
                keyObj = fttPafObj.getKeyObjNo(keyCount)

        # Handle function (optional).
        keyCount = self._setFctFromFtt(action, keyCount, fttPafObj)
              
        return keyCount


    def __setRenameActionFromFtt(self,
                                 keyCount,
                                 fttPafObj):
        """
        Extract a Rename Translation Action from an FTT.
        
        keyCount:       The present key count.
        
        fttPafObj:      Reference to the FTT Object.

        Returns:        Key counter in the FTT object.
        """
        self.setAction(ftuACTION_RENAME)
            
        # Handle RENAME.SRCKEY.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "RENAME.SRCKEY")
        self.setSrcKey(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle RENAME.TRGKEY.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "RENAME.TRGKEY")
        self.setTrgKey(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle RENAME.HDR.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "RENAME.HDR")
        self.setHdr(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle function (optional).
        keyCount = self._setFctFromFtt(ftuACTION_RENAME, keyCount, fttPafObj)
              
        return keyCount


    def __setMoveActionFromFtt(self,
                               keyCount,
                               fttPafObj):
        """
        Extract a Move Translation Action from an FTT.
        
        keyCount:       The present key count.
        
        fttPafObj:      Reference to the FTT Object.

        Returns:        Key counter in the FTT object.
        """
        self.setAction(ftuACTION_MOVE)
            
        # Handle MOVE.KEY.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "MOVE.KEY")
        self.setKey(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle MOVE.SRCHDR.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "MOVE.SRCHDR")
        self.setSrcHdr(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle MOVE.TRGHDR.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "MOVE.TRGHDR")
        self.setTrgHdr(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle function (optional).
        keyCount = self._setFctFromFtt(ftuACTION_MOVE, keyCount, fttPafObj)
              
        return keyCount

    
    def __setCopyActionFromFtt(self,
                               keyCount,
                               fttPafObj):
        """
        Extract a Copy Translation Action from an FTT.
        
        keyCount:       The present key count.
        
        fttPafObj:      Reference to the FTT Object.

        Returns:        Key counter in the FTT object.
        """
        self.setAction(ftuACTION_COPY)
            
        # Handle COPY.SRCKEY.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "COPY.SRCKEY")
        self.setSrcKey(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle COPY.SRCHDR.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "COPY.SRCHDR")
        self.setSrcHdr(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle COPY.TRGKEY.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "COPY.TRGKEY")
        self.setTrgKey(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle COPY.TRGHDR.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "COPY.TRGHDR")
        self.setTrgHdr(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle function (optional).
        keyCount = self._setFctFromFtt(ftuACTION_COPY, keyCount, fttPafObj)
              
        return keyCount


    def __setRemoveActionFromFtt(self,
                                 keyCount,
                                 fttPafObj):
        """
        Extract a Remove Translation Action from an FTT.
        
        keyCount:       The present key count.
        
        fttPafObj:      Reference to the FTT Object.

        Returns:        Key counter in the FTT object.
        """
        self.setAction(ftuACTION_REMOVE)
            
        # Handle REMOVE.KEYLIST.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "REMOVE.KEYLIST")
        self.setKey(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle REMOVE.HDR.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "REMOVE.HDR")
        self.setSrcHdr(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1
              
        return keyCount
        

    def setActionFromFtt(self,
                         keyCount,
                         fttPafObj):
        """
        Extract a Translation Action from an FTT Object. It is supposed that
        the "keyCount" parameters points to the first keyword in the
        block which the Translation Action constitutes.

        keyCount:       The present key count.
        
        fttPafObj:      Reference to the FTT Object.

        Returns:        Key counter in the FTT object.
        """
        PccUtUtils.checkType("keyCount", keyCount,
                             "FtuFttAction.getActionFromFtt()", types.IntType)
        PccUtUtils.checkType("fttPafObj", fttPafObj,
                             "FtuFttAction.getActionFromFtt()", PccPaf.PccPaf)
        keyObj = fttPafObj.getKeyObjNo(keyCount)
        if (keyObj.getKey() == "ADD.KEY"):
            keyCount = self.__setAddChangeActionFromFtt(ftuACTION_ADD,
                                                        keyCount, fttPafObj)
        elif (keyObj.getKey() == "CHANGE.KEY"):
            keyCount = self.__setAddChangeActionFromFtt(ftuACTION_CHANGE,
                                                        keyCount, fttPafObj)
        elif (keyObj.getKey() == "RENAME.SRCKEY"):
            keyCount = self.__setRenameActionFromFtt(keyCount, fttPafObj)
        elif (keyObj.getKey() == "MOVE.KEY"):
            keyCount = self.__setMoveActionFromFtt(keyCount, fttPafObj)
        elif (keyObj.getKey() == "COPY.SRCKEY"):
            keyCount = self.__setCopyActionFromFtt(keyCount, fttPafObj)
        elif (keyObj.getKey() == "REMOVE.KEYLIST"):
            keyCount = self.__setRemoveActionFromFtt(keyCount, fttPafObj)
        else:
            # raise exception
            raise exceptions.Exception, \
                  "Found unexpected key " + keyObj.getKey() + " in PAF file "+\
                  fttPafObj.getFileName() + "."
        
        return keyCount
    

    def setKey(self,
               key):
        """
        Set the keyword of the Translation Action.

        key:            Name of keyword.

        Returns:        Reference to object itself.
        """
        self.__srcKey = key
        return self
        

    def getKey(self):
        """
        Return keyword set for the Translation Action.

        Returns:        Keyword.
        """
        return self.__srcKey


    def setHdr(self,
               hdr):
        """
        Set header number for the Translation Action.

        hdr:            Header number.

        Returns:        Reference to object itself.
        """
        self.__srcHdr = hdr
        return self


    def getHdr(self):
        """
        Return header number for Translation Action.

        Returns:       Header number.
        """
        return self.__srcHdr


    def setSrcKey(self,
                  srcKey):
        """
        Set the Source Key for the Translation Action.

        srcKey:        Source Key.

        Returns:       Reference to the object itself.
        """
        self.__srcKey = PccUtString.trimString(srcKey, " \"")
        return self
        

    def getSrcKey(self):
        """
        Return Source Key for the Translation Action.
        
        Returns:      Source Key.
        """
        return self.__srcKey


    def setTrgKey(self,
                  trgKey):
        """
        Set Target Key for the Translation Action.

        trgKey:       Target Key.

        Returns:      Reference to object itself.
        """
        self.__trgKey = PccUtString.trimString(trgKey, " \"")
        return self
        

    def getTrgKey(self):
        """
        Return the Target Key for the Translation Action.

        Returns:      Target Key.
        """
        return self.__trgKey


    def setSrcHdr(self,
                  srcHdr):
        """
        Set Source Header for the Translation Action.

        srcHdr:       Source Header.

        Returns:      Reference to object itself.
        """
        self.__srcHdr = int(PccUtString.trimString(str(srcHdr), " \""))
        return self


    def getSrcHdr(self):
        """
        Return Source Header for the Translation Action.

        Returns:      Source Header.
        """
        return self.__srcHdr


    def setTrgHdr(self,
                  trgHdr):
        """
        Set Target Header for the Translation Action.

        trgHdr:       Target Header.

        Returns:      Reference to object itself.
        """
        self.__trgHdr = int(PccUtString.trimString(str(trgHdr), " \""))
        return self


    def getTrgHdr(self):
        """
        Return Target Header for Translation Action.

        Returns:      Target Header.
        """
        return self.__trgHdr


    def __dumpAddChangeAction(self,
                              action):
        """
        Dump the contents of an Add or Change Translation Action into
        a string object and return this.

        action:         Translation Action: FtuFttAction.ftuACTION_ADD or
                        FtuFttAction.ftuACTION_CHANGE.
                        
        Returns:        String buffer with the PAF print out of the
                        Translation Action.
        """
        # <action>.KEY           <key>
        # <action>.HDR           <hdr>
        # [<action>.VALUE.KEY    <key>]
        # [<action>.VALUE.CONST  <const>]
        # [<action>.VALUE.FCT    <fct>]
        # [<action>.FCT          <fct>]
        retBuf = '%-27s "%s"' % (action + ".KEY", self.getSrcKey())
        retBuf = retBuf + '\n%-27s %d' % (action + ".HDR",
                                          int(self.getSrcHdr()))
        for valueSrc in self.getValueSrcList():
            retBuf = retBuf + "\n" + valueSrc.dumpBuf(action)
        if (self.getFct() != ""):
            retBuf = retBuf + '\n%-27s "%s"' %\
                     (action + ".FCT", self.getFct())
        return retBuf


    def __dumpRenameAction(self):
        """
        Dump the contents of a Rename Translation Action into
        a string object and return this.

        Returns:        String buffer with the PAF print out of the
                        Translation Action.        
        """
        # RENAME.SRCKEY          <key>
        # RENAME.TRGKEY          <key>
        # RENAME.HDR             <hdr>
        # [RENAME.FCT            <fct>]
        retBuf = '%-27s "%s"' %\
                 ("RENAME.SRCKEY", self.getSrcKey())
        retBuf = retBuf + '\n%-27s "%s"' %\
                 ("RENAME.TRGKEY", self.getTrgKey())
        retBuf = retBuf + '\n%-27s %d' % ("RENAME.HDR", int(self.getSrcHdr()))
        if (self.getFct() != ""):
            retBuf = retBuf + '\n%-27s "%s"' % ("RENAME.FCT", self.getFct())
        return retBuf
        

    def __dumpMoveAction(self):
        """
        Dump the contents of a Move Translation Action into
        a string object and return this.

        Returns:        String buffer with the PAF print out of the
                        Translation Action.        
        """
        # MOVE.KEY               <key>
        # MOVE.SRCHDR            <hdr>
        # MOVE.TRGHDR            <hdr>
        # [MOVE.FCT              <fct>]
        retBuf = '%-27s "%s"' % ("MOVE.KEY", self.getSrcKey())
        retBuf = retBuf + '\n%-27s %d' % ("MOVE.SRCHDR", int(self.getSrcHdr()))
        retBuf = retBuf + '\n%-27s %d' % ("MOVE.TRGHDR", int(self.getTrgHdr()))
        if (self.getFct() != ""):
            retBuf = retBuf + '\n%-27s "%s"' % ("MOVE.FCT", self.getFct())
        return retBuf


    def __dumpRemoveAction(self):
        """
        Dump the contents of a Copy Translation Action into
        a string object and return this.

        Returns:        String buffer with the PAF print out of the
                        Translation Action.  
        """
        # REMOVE.KEYLIST         <key list>
        # REMOVE.HDR             <hdr>
        retBuf = '%-27s "%s"' %\
                 ("REMOVE.KEYLIST", self.getSrcKey())
        retBuf = retBuf + '\n%-27s %d' % ("REMOVE.HDR", int(self.getSrcHdr()))
        return retBuf
        

    def __dumpCopyAction(self):
        """
        Dump the contents of a Copy Translation Action into
        a string object and return this.

        Returns:        String buffer with the PAF print out of the
                        Translation Action.        
        """
        # COPY.SRCKEY            <key>
        # COPY.SRCHDR            <hdr>
        # COPY.TRGKEY            <key>
        # COPY.TRGHDR            <hdr>
        # [COPY.FCT              <fct>]
        retBuf = '%-27s "%s"' % ("COPY.SRCKEY", self.getSrcKey())
        retBuf = retBuf + '\n%-27s %d' % ("COPY.SRCHDR", int(self.getSrcHdr()))
        retBuf = retBuf + '\n%-27s "%s"' % ("COPY.TRGKEY", self.getTrgKey())
        retBuf = retBuf + '\n%-27s %d' % ("COPY.TRGHDR", int(self.getTrgHdr()))
        if (self.getFct() != ""):
            retBuf = retBuf + '\n%-27s "%s"' % ("COPY.FCT", self.getFct())
        return retBuf
        

    def dumpBuf(self):
        """
        Dump the Translation Action in a string buffer in PAF format.

        Returns:         String buffer with the PAF representation of the
                         Translation Action.
        """
        if (self.getAction() == ftuACTION_ADD):
            return self.__dumpAddChangeAction(ftuACTION_ADD)
        elif (self.getAction() == ftuACTION_CHANGE):
            return self.__dumpAddChangeAction(ftuACTION_CHANGE)
        elif (self.getAction() == ftuACTION_RENAME):
            return self.__dumpRenameAction()
        elif (self.getAction() == ftuACTION_MOVE):
            return self.__dumpMoveAction()
        elif (self.getAction() == ftuACTION_COPY):
            return self.__dumpCopyAction()
        else:
            # ftuACTION_REMOVE
            return self.__dumpRemoveAction()
        

#
# ___oOo___
