#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: FtuFttDicRec.py,v 1.1 2004/09/06 14:38:37 cguirao Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  16/02/2001  Created
#

"""
Class to handle DID Records.
"""


import types
import PccUtString, PccUtUtils, PccPaf
import FtuFttUt
from   FtuFttDefs import *


class FtuFttDicRec:
    """
    Class to handle the information in connection with a DID Record.
    """


    def __init__(self,
                 key     = "",
                 type    = "",
                 format  = "",
                 comment = ""):
        """
        Constructor method initializing the object class variables.

        key                Name of keyword.
        
        type               Type of keyword (ftuTYPE_STRING, ftuTYPE_LOGICAL
                           ftuTYPE_INTEGER, ftuTYPE_DOUBLE).
        
        format             Format of value (see DICB definition:
                           http://www.eso.org/dicb).
         
        comment            Comment for keyword.
        """
        self.setKey(key)
        self.setType(type)
        self.setFormat(format)
        self.setComment(comment)


    def setKey(self,
               key):
        """
        Set the name of the keyword.

        key            Name of the keyword.

        Returns:       Reference to the object itself.
        """
        tmpKey = PccUtString.trimString(key, " \"")
        PccUtUtils.checkType("key", tmpKey, "FtuFttDicRec.setKey()",
                             types.StringType)
        self.__key = tmpKey
        return self


    def getKey(self):
        """
        Return name of keyword for object.

        Returns:   Name of key.
        """
        return self.__key


    def setType(self,
                type):
        """
        Set the type of the keyword. See constructor method.

        type        Type of keyword.

        Returns:    Reference to object itself.
        """
        tmpType = PccUtString.trimString(type, " \"")
        PccUtUtils.checkType("type", tmpType, "FtuFttDicRec.setType()",
                             types.StringType)
        PccUtUtils.checkInList(tmpType, [ftuTYPE_UNDEF, ftuTYPE_STRING,
                                         ftuTYPE_LOGICAL,
                                         ftuTYPE_INTEGER, ftuTYPE_DOUBLE],
                               "type", "FtuFttDicRec.setType()")
        self.__type = tmpType
        return self


    def getType(self):
        """
        Return type of keyword.

        Returns:  Type.
        """
        return self.__type


    def setFormat(self,
                  format):
        """
        Set the format of the keyword value. Should be given in the
        C style.

        format    Format string, e.g. "%s", "%d", "%.5E".

        Returns:  Reference to object itself.
        """
        tmpFormat = PccUtString.trimString(format, " \"")
        PccUtUtils.checkType("format", tmpFormat, "FtuFttDicRec.setFormat()",
                             types.StringType)
        # IMPL.: Check proper formatting.
        self.__format = tmpFormat
        return self


    def getFormat(self):
        """
        Return the value format for the keyword.

        Returns:  Value format.
        """
        return self.__format


    def setComment(self,
                   comment):
        """
        Set the comment field for the keyword.

        comment   Comment field.

        Returns:  Reference to object itself.
        """
        tmpComment = PccUtString.trimString(comment, " \"")
        PccUtUtils.checkType("comment", tmpComment,
                             "FtuFttDicRec.setComment()", types.StringType)
        self.__comment = tmpComment
        return self


    def getComment(self):
        """
        Return comment for keyword.

        Returns:   Comment for keyword.
        """
        return self.__comment


    def setFromFttPaf(self,
                      keyCount,
                      fttPafObj):
        """
        Extract the keywords for one FTT Dictionary Record from an FTT
        loaded into a PccPaf object, and return the new keyword counter
        (index) into the FTT PAF object keyword list.

        keyCount     Keyword index referring to a keyword number in the
                     FTT PccPaf object.
        
        fttPafObj    Reference to the PccPaf object into which the FTT is
                     loaded.

        Returns:     New keyword index.
        """

        # Expect:
        #
        # DIC.KEY
        # DIC.TYPE
        # DIC.FORMAT
        # DIC.COMMENT
        PccUtUtils.checkType("keyCount", keyCount,
                             "FtuFttDicRec.setFromFttPaf()", types.IntType)
        PccUtUtils.checkType("fttPafObj", fttPafObj,
                             "FtuFttDicRec.setFromFttPaf()", PccPaf.PccPaf)
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "DIC.KEY", fttPafObj, keyCount)
        self.setKey(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "DIC.TYPE", fttPafObj, keyCount)
        self.setType(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "DIC.FORMAT", fttPafObj, keyCount)
        self.setFormat(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(),
                             "DIC.COMMENT", fttPafObj, keyCount)
        self.setComment(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        return keyCount


    def dumpBuf(self):
        """
        Dump contents of object into a string buffer in PAF format.

        Returns:   Reference to string buffer containing the contents of
                   for the DID Record.
        """
        retBuf = '%-27s "%s"' % ("DIC.KEY", self.getKey())
        retBuf = retBuf + '\n%-27s "%s"' % ("DIC.TYPE", self.getType())
        retBuf = retBuf + '\n%-27s "%s"' % ("DIC.FORMAT", self.getFormat())
        retBuf = retBuf + '\n%-27s "%s"' % ("DIC.COMMENT", self.getComment())
        return retBuf
        

    def dump(self):
        """
        Dump contents of object on stdout in PAF format.

        Returns:   Reference to object itself.
        """
        print self.dumpBuf()
        return self
        

#
# ___oOo___
