#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: FtuFttValueSrc.py,v 1.1 2004/09/06 14:38:37 cguirao Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  06/02/2001  Created
#

"""
Class to handle the properties of an FTT Value Source.
"""

import string, types
import PccKey, PccUtString, PccUtUtils, PccPaf
from   FtuFttDefs import *

class FtuFttValueSrc:
    """
    Class to handle the information for one Value Source.
    """

    
    def __init__(self,
                 action = "",
                 type   = "",
                 src    = "",
                 hdr    = -1):
        """
        Constructor method initializing the object class variables.
        
        action        Action for the object. See documentation for
                      FtuFttAction class for constants.
        
        type          Type of the Value Source.
        
        src           Reference of the Value Source. In case of a keyword
                      the key name, in case of a constant, the constant
                      value, in case of a function, the function name.

        hdr           Header related to a Keyword Value Source. "-1" indicates
                      that the value should be taken from the default
                      header (current header).
        """
        self.setAction(action)
        self.setType(type)
        self.setSrc(src)
        self.setHdr(hdr)


    def setFromPaf(self,
                   keyCount,
                   fttPafObj):
        """
        Set the information of the object from the keywords found in
        the PccPaf object containing an FTT, starting from the keyword
        indicated by "keyCount".
        
        keyCount
        
        fttPafObj

        Returns:     Updated keyword counter (index).
        """
        PccUtUtils.checkType("keyCount", keyCount,
                             "FtuFttValueSrc.setFromPaf()", types.IntType)
        PccUtUtils.checkType("fttPafObj", fttPafObj,
                             "FtuFttValueSrc.setFromPaf()", PccPaf.PccPaf)

        key = fttPafObj.getKeyObjNo(keyCount).getKey()
        keyEls = string.split(key, ".")
        if (len(keyEls) == 3):
            action = keyEls[0]
            valueSrcPrefix = keyEls[0] + "." + keyEls[1] 
        elif (len(keyEls) == 4):
            action = keyEls[0] + "." + keyEls[1]
            valueSrcPrefix = keyEls[0] + "." + keyEls[1] + "." + keyEls[2]
        else:
            raise exceptions.Exception, \
                  "Illegal key found in PAF: " + key + "."
        self.setAction(action)
    
        # Check that the next keyword is the expected one.
        if ((valueSrcPrefix + "." + "KEY" != key) and \
            (valueSrcPrefix + "." + "CONST" != key) and \
            (valueSrcPrefix + "." + "FCT" != key)):
            raise exceptions.Exception, \
                  "Didn't find expected key. Found " + key + " instead."
        self.setFromPafLine(fttPafObj.getKeyObjNo(keyCount).getKeywordLine(27))
        keyCount = keyCount + 1

        # Check if there is a reference to a specific header.
        if (keyCount < fttPafObj.getNoOfKeys()):
            if (fttPafObj.getKeyObjNo(keyCount).getKey() ==
                valueSrcPrefix + "." + "KEY.HDR"):
                self.setHdr(fttPafObj.getKeyObjNo(keyCount).getValue())
                keyCount = keyCount + 1

        return keyCount
        

    def setFromPafLine(self,
                       pafLine):
        """
        Set the object class variables from a PAF keyword line.

        pafLine        PAF keyword line.

        Returns:       Reference to the object itself.
        """ 
        keyValCom = PccKey.splitKeywordLine(pafLine)
        keyComp = string.split(keyValCom[0], ".")

        # Two basic different types of keywords, e.g.:
        # 1) FILE.PATH.CONCAT.KEY, FILE.RENAME.CONCAT.CONST
        # 2) ADD.VALUE.KEY, CHANGE.VALUE.KEY
        if (keyComp[0] == "FILE"):
            self.setAction(keyComp[0] + "." + keyComp[1])
            #self.setType(keyComp[2] + "." + keyComp[3])
            self.setType(keyComp[3])
        else:
            self.setAction(keyComp[0])
            self.setType(keyComp[2])
        self.setSrc(keyValCom[1])
        return self


    def setValueSrc(self,
                    type,
                    src,
                    action):
        """
        Set the properties of the Value Source.
        
        type        Type of Value Source.
        
        src         Source Reference.
        
        action      Action in which context the Value Source is defined
                      
        Returns:    Reference to the object itself.
        """
        self.setType(type)
        self.setSrc(src)
        self.setAction(action)
        return self


    def setAction(self,
                  action):
        """
        Set the Translation Action of the Value Source Object.

        action     Translation Action. See documentation for class
                   FtuFttAction for definition of actions.

        Returns:   Reference to object itself.
        """
        tmpAction = PccUtString.trimString(action, " \"")
        PccUtUtils.checkType("action", tmpAction, "FtuFttValueSrc.setAction()",
                             types.StringType)
        PccUtUtils.checkInList(tmpAction, [ftuACTION_UNDEF,
                                           ftuACTION_HDR_DUMP,
                                           ftuACTION_FILE_PATH,
                                           ftuACTION_FILE_RENAME,
                                           ftuACTION_FILE_SPLIT,
                                           ftuACTION_ADD,
                                           ftuACTION_CHANGE,
                                           ftuACTION_RENAME,
                                           ftuACTION_MOVE,
                                           ftuACTION_COPY,
                                           ftuACTION_REMOVE])
        self.__action = tmpAction
        return self


    def getAction(self):
        """
        Return the Translation Action for the object.

        Returns:   Translation Action.
        """
        return self.__action


    def setType(self,
                type):
        """
        Set type of the Value Source.

        type       Type of Value Source. Check documentation for this
                   class for defined types.

        Returns:   Reference to object itself.
        """
        tmpType = PccUtString.trimString(type, " \"") 
        PccUtUtils.checkType("type", tmpType, "FtuFttValueSrc.setType()",
                             types.StringType)
        PccUtUtils.checkInList(tmpType, [ftuVAL_SRC_UNDEF, ftuVAL_SRC_KEY,
                                         ftuVAL_SRC_CONST, ftuVAL_SRC_FCT])
        self.__type = tmpType
        return self


    def getType(self):
        """
        Return type of Value Source.

        Returns:    Type of Value Source.
        """
        return self.__type


    def setSrc(self,
               src):
        """
        Set the Value Source reference. I.e., from where to take the value
        (keyword name, constant value, function name).

        src        Value Source reference.

        Returns:   Reference to object itself.
        """
        tmpSrc = PccUtString.trimString(src, " \"")
        PccUtUtils.checkType("src", src, "FtuFttValueSrc.setSrc()",
                             types.StringType)
        self.__src = tmpSrc
        return self
                

    def getSrc(self):
        """
        Return the Value Source reference of the object.

        Returns:   Value Source.
        """
        return self.__src


    def setHdr(self,
               hdr):
        """
        Set the header reference for a Key Value Source.

        hdr        Header number.

        Returns:   Reference to object itself.
        """
        self.__hdr = int(PccUtString.trimString(str(hdr), " \""))
        return self
                

    def getHdr(self):
        """
        Return the header reference for a Key Value Source.

        Returns:   Header number. "-1" is returned is not used.
        """
        return self.__hdr


    def dumpBuf(self,
                action = ""):
        """
        Dump contents of object into a PAF keyword line.

        Returns:   String object with the Value Source PAF keyword line.
        """
        if (action == ""):
            actionLoc = self.getAction()
        else:
            actionLoc = action
        if (string.find(actionLoc, "FILE.") == 0):
            srcLoc = ".CONCAT." + self.getType()
        else:
            srcLoc = ".VALUE." + self.getType()
        retBuf = '%-27s "%s"' % (actionLoc + srcLoc, self.getSrc())
        if (self.getHdr() != -1):
            if (string.find(actionLoc, "FILE.") == 0):
                srcLoc = ".CONCAT.KEY.HDR"
            else:
                srcLoc = ".VALUE.KEY.HDR"
            retBuf = retBuf + '\n%-27s %d' % (actionLoc + srcLoc,self.getHdr())
        
        return retBuf
        

#
# ___oOo___
