// -*-c++-*-
#ifndef _TimeStamp_h_
#define _TimeStamp_h_

/*
 * E.S.O. - VLT project/ESO Archive
 * $Id: TimeStamp.h,v 1.2 2004/11/24 15:22:42 nferguso Exp $
 *
 * TimeStamp.h - class representing a timestamp value in either MJD
 *               (modified julian day) or string format:
 *               "YYYY-MM-DDThh:mm:ss[.mss]".
 *
 * who             when         what
 * --------------  -----------  ------------------------------------------- 
 * Allan Brighton  11 Jul 1997  Created. 
 * Neil Ferguson   08 Apr 2004  Methods added to replace RogueWave RWDate & 
 *                              RWTime functionality used by the DFS. 
 */

#include <sys/types.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>

#include <DfsCString.h>

/// The TimeStamp class provides date- and time-related utility functions.
// A more detailed class documentation block is provided at the end of 
// this file (to keep the large table sections out of the way). 
class TimeStamp {
private:
    double mjd_;		///< Modified Julian date. 
    char buf_[80];		///< Temporary string buffer. 
    int status_;		///< Status after construction. 

    /// @brief Converts a modified Julian date to a C-style @code struct tm 
    /// @endcode time. 
    /** @param mjd The MJD to be converted. 
     * @param tm The @code struct tm @endcode which will receive the result. 
     * @param secs The @a tm result can only hold an integral number of 
     * seconds. Any digits after the decimal point will be returned in 
     * @a secs instead. 
     *
     * This method only supports dates within the UNIX @c time_t range. 
     */
    static int mjdToTm(double mjd, struct tm& tm, double& secs);
 
    /// @brief Converts a C-style @code struct tm @endcode time to a modified 
    /// Julian date. 
    /** @param tm The @code struct tm @endcode to be converted. 
     * @param secs The @a tm input can only supply an integral number of 
     * seconds. Any digits after the decimal point should be supplied in 
     * @a secs instead. 
     * @param mjd The resulting MJD. 
     */
    static int tmToMjd(const struct tm& tm, double secs, double& mjd);

public:
    /// Modified Julian date for 00:00:00 on 01.01.1970. 
    enum {MJD_1970 = 40587};

/** @name Constructors 
 *
 * Some of these methods only support restricted date ranges. The detailed 
 * documentation for each method notes any restrictions that apply. 
 */
//@{
    /// Initialize with the current time. 
    TimeStamp();

    /// Initialize with a date (and optional time) expressed as a string. 
    /** Valid string formats are: 
     * - YYYY-MM-DD 
     * - YYYY-MM-DDThh:mm:ss 
     * - YYYY-MM-DDThh:mm:ss.mss 
     * 
     * Leading strings starting with non-numeric chars and ending with "." 
     * are ignored. For example, in "ONTT.1997-07-21T20:03:01.234", the 
     * "ONTT." part will be discarded before the date and time are parsed. 
     * 
     * The date string @a s must be within the range 1901-12-13T22:00:00.000 
     * to 9999-12-31T23:59:59.999. 
     */
    TimeStamp(const char* s);

    /// Initialize with the Julian date @a jd (modified or unmodified). 
    /** If @a isMJD is @c true, @a jd will be treated as a modified Julian 
     * date. Otherwise, it will be treated as an unmodified Julian date. 
     */
    TimeStamp(double jd, bool isMJD = true) : 
        mjd_(isMJD ? jd : jd - 2400000.5), 
        status_(0) 
    {}
//@}

/** @name Member Access */
//@{
    /// Returns the MJD held by this TimeStamp. 
    operator double() const {return mjd_;}

    /// Returns the status value held by this TimeStamp. 
    int status() const {return status_;}

    /// Returns the MJD held by this TimeStamp. 
    double mjd() const {return mjd_;}

    /// @brief Returns this TimeStamp's contents as a C-style @code struct tm 
    /// @endcode plus a fractional number of seconds. 
    /** This method only supports dates within the UNIX @c time_t range. */ 
    int getTmTime(struct tm& tm, double& secs) const; 
//@}

/** @name String Access 
 * All of these methods return a pointer which refers to a buffer within the 
 * TimeStamp object. As a result, the data will be overwritten when another 
 * method that uses the buffer is called. Obviously, the pointer must not 
 * be freed by the caller. 
 *
 * These methods only support dates within the UNIX @c time_t range. 
 */
//@{
    /// @brief Returns this TimeStamp's contents as a C string with format 
    /// YYYY-MM-DDT:hh:mm:ss.mss. 
    operator const char*() const {return getTimeStamp();}

    /// @brief Returns this TimeStamp's contents as a C string with format 
    /// YYYY-MM-DDT:hh:mm:ss.mss. 
    const char* getTimeStamp() const;

    /// Returns this TimeStamp's date as a C string with format YYYY-MM-DD. 
    const char* getDate() const;

    /// @brief Returns this TimeStamp's date for the night (noon to noon) 
    /// as a C string with format YYYY-MM-DD. 
    const char* getNight() const;

    /// Returns this TimeStamp's time as a C string with format hh:mm:ss.mss. 
    const char* getTime() const;
//@}

/** @name RogueWave RWDate/RWTime Replacements 
 *
 * Some of these methods only support restricted date ranges. The detailed 
 * documentation for each method notes any restrictions that apply. 
 */
//@{
    /// Construct for the given date. Time is zero hours on that date. 
    /** The year cannot be negative. */ 
    TimeStamp(unsigned int day, unsigned int month, unsigned int year);

    /// Construct for the given date, but with the specified time. 
    TimeStamp(const TimeStamp& date, 
              unsigned int hour, 
              unsigned int minute, 
              unsigned int second = 0);

    /// Decrements this TimeStamp by one day, but returns the original value. 
    TimeStamp operator--(int dummy); 

    /// @brief Returns the current time as a string, formatted according to 
    /// the given @c strftime() formatting string. 
    /** This method only supports dates within the UNIX @c time_t range. */ 
    DfsCString asString(const char* format) const;

    /// Compares this TimeStamp with the @a compareTime. 
    int compareTo(const TimeStamp* compareTime) const;

    /// Returns the day of the year (1-366) for the stored date. 
    /** This method only supports dates within the UNIX @c time_t range. */ 
    unsigned int day() const;

    /// Returns the hour of the time stored in this TimeStamp. 
    /** This method only supports dates within the UNIX @c time_t range. */ 
    unsigned int hour() const; 
    
    /// Returns the (unmodified) Julian day number of the stored date. 
    double julian() const; 

    /// Returns the year of the stored date. 
    /** This method only supports dates within the UNIX @c time_t range. */ 
    unsigned int year() const;
//@}
};

/// Returns @c true if the given TimeStamps are not equal. 
/// 
/// @relates TimeStamp
extern DfsBoolean operator!=(const TimeStamp& leftOperand, 
                             const TimeStamp& rightOperand); 

/// STL ostream output operator for TimeStamp objects. 
/** Output is in RWTime operator<< format. This implies a two-digit year 
 * and a resolution of one second. 
 *
 * This method only supports dates within the UNIX @c time_t range. 
 *
 * @relates TimeStamp 
 */
extern std::ostream& operator<<(std::ostream& stream, 
                                const TimeStamp& source); 

/** @class TimeStamp 
 * 
 * This version of the TimeStamp class has been adapted from the original 
 * class written by Allan Brighton. It supports the removal of the RWTools 
 * library from the DFS by providing new methods that replace the RWDate 
 * and RWTime functionality previously required by DFS code. An important 
 * consequence of this is that TimeStamp objects simultaneously double as 
 * both calendar dates and clock times. 
 *
 * <B>NOTE:</B> TimeStamp is not part of DfsStdLib - it belongs to the 
 * utilCat library - but it does depend on DfsStdLib functionality. The 
 * TimeStamp.C and TimeStamp.h files are intended for use as drop-in 
 * replacements for the corresponding files under $DFSROOT/util/utilCat. 
 * 
 * Only existing RWDate and RWTime usages are supported by this class, and 
 * several of those usages were rationalised to reduce the number of new 
 * constructors and methods that needed to be written and supported. As a 
 * result, existing RWTools-based code will need to be converted according 
 * to the guidelines given in Tables 1 and 2. 
 *
 * <table frame="border" rules="all" width=100%>
 *   <caption align=top>Table 1: Converting from RWDate to TimeStamp</caption>
 *   <tr>
 *     <th>RWDate Call</th>
 *     <th>TimeStamp Call</th>
 *   </tr>
 *   <tt>
 *   <tr>
 *     <td>RWDate()</td>
 *     <td>TimeStamp()</td>
 *   </tr>
 *   <tr>
 *     <td>RWDate(const RWDate&)</td>
 *     <td>TimeStamp(const TimeStamp&)</td>
 *   </tr>
 *   <tr>
 *     <td>RWDate(unsigned, unsigned, unsigned)</td>
 *     <td>TimeStamp(unsigned, unsigned, unsigned)</td>
 *   </tr>
 *   <tr>
 *     <td>RWDate(const RWTime&, const RWZone& zone)</td>
 *     <td>TimeStamp(const TimeStamp&)</td>
 *   </tr>
 *   <tr>
 *     <td>RWDate(unsigned long)</td>
 *     <td>TimeStamp(double, bool)</td>
 *   </tr>
 *   <tr>
 *     <td>~RWDate()</td>
 *     <td>~TimeStamp()</td>
 *   </tr>
 *   <tr>
 *     <td>RWDate& operator=(const RWDate&)</td>
 *     <td>TimeStamp& operator=(const TimeStamp&)</td>
 *   </tr>
 *   <tr>
 *     <td>RWDate operator--(int)</td>
 *     <td>TimeStamp operator--(int)</td>
 *   </tr>
 *   <tr>
 *     <td>RWCString asString(char, const RWLocale&) const</td>
 *     <td>DfsCString asString(const char*) const</td>
 *   </tr>
 *   <tr>
 *     <td>RWCString asString(const char*, const RWLocale&) const</td>
 *     <td>DfsCString asString(const char*) const</td>
 *   </tr>
 *   <tr>
 *     <td>unsigned day() const</td>
 *     <td>unsigned day() const</td>
 *   </tr>
 *   <tr>
 *     <td>unsigned long julian() const</td>
 *     <td>double julian() const</td>
 *   </tr>
 *   <tr>
 *     <td>unsigned year() const</td>
 *     <td>unsigned year() const</td>
 *   </tr>
 *   <tr>
 *     <td>RWBoolean operator!=(const RWDate&, const RWDate&)</td>
 *     <td>DfsBoolean operator!=(const TimeStamp&, const TimeStamp&)</td>
 *   </tr>
 *   </tt>
 * </table>
 *
 * <table frame="border" rules="all" width=100%>
 *   <caption align=top>Table 2: Converting from RWTime to TimeStamp</caption>
 *   <tr>
 *     <th>RWTime Call</th>
 *     <th>TimeStamp Call</th>
 *   </tr>
 *   <tt>
 *   <tr>
 *     <td>RWTime()</td>
 *     <td>TimeStamp()</td>
 *   </tr>
 *   <tr>
 *     <td>RWTime(const RWTime&)</td>
 *     <td>TimeStamp(const TimeStamp&)</td>
 *   </tr>
 *   <tr>
 *     <td>RWTime(unsigned long)</td>
 *     <td><i>Obsolete</i></td>
 *   </tr>
 *   <tr>
 *     <td>RWTime(unsigned, unsigned, unsigned, const RWZone&)</td>
 *     <td>TimeStamp(const TimeStamp&, unsigned, unsigned, unsigned)</td>
 *   </tr>
 *   <tr>
 *     <td>RWTime(const RWDate&, unsigned, unsigned, unsigned, const RWZone&)</td>
 *     <td>TimeStamp(const TimeStamp&, unsigned, unsigned, unsigned)</td>
 *   </tr>
 *   <tr>
 *     <td>~RWTime()</td>
 *     <td>~TimeStamp()</td>
 *   </tr>
 *   <tr>
 *     <td>RWTime& operator=(const RWTime&)</td>
 *     <td>TimeStamp& operator=(const TimeStamp&)</td>
 *   </tr>
 *   <tr>
 *     <td>RWCString asString(char, const RWZone&, const RWLocale&) const</td>
 *     <td>DfsCString asString(const char*) const</td>
 *   </tr>
 *   <tr>
 *     <td>RWCString asString(char*, const RWZone&, const RWLocale&) const</td>
 *     <td>DfsCString asString(const char*) const</td>
 *   </tr>
 *   <tr>
 *     <td>int compareTo(const RWTime*) const</td>
 *     <td>int compareTo(const TimeStamp*) const</td>
 *   </tr>
 *   <tr>
 *     <td>unsigned hour(const RWZone&) const</td>
 *     <td>unsigned hour() const</td>
 *   </tr>
 *   <tr>
 *     <td>unsigned long seconds() const</td>
 *     <td><i>Obsolete</i></td>
 *   </tr>
 *   <tr>
 *     <td>static RWTime now()</td>
 *     <td>TimeStamp()</td>
 *   </tr>
 *   <tr>
 *     <td>ostream& operator<<(ostream&, const RWTime&)</td>
 *     <td>ostream& operator<<(ostream&, const TimeStamp&)</td>
 *   </tr>
 *   </tt>
 * </table>
 */
#endif /* _TimeStamp_h_ */
