#include <sstream>
#include "FitsIO.h"

/*
 * E.S.O. - VLT project 
 *
 * "@(#) $Id: FitsIO.C,v 1.3 2007/02/21 17:11:21 szampier Exp $" 
 *
 * This class is a replacement of the "original" FITS manipulation class.
 * See previous CVS revisions for original code.
 * Only the interfaces used by OLAS are kept.
 *
 * who                 when           what
 * ----------------    -----------    -------
 * Stefano Zampieri    15 Sep 2005    Created
 */

// common error message
const char* noFitsErrMsg = "No FITS file is currently open";

const char* noHdrErrMsg = "Can't get keyword: no FITS header.";

// local buffer used to return keyword and table values
static char buf_[1024];

/*
 * private constructor
 */
FitsIO::FitsIO(fitsfile *fitsio) :
  fitsio_(fitsio)
{
}

/*
 * Destructor: close the cfitsio object
 */
FitsIO::~FitsIO()
{
  if (fitsio_) {
    int status = 0;
    if (fits_close_file(fitsio_, &status) != 0)
      cfitsio_error();
    fitsio_ = NULL;
  }
}


/*
 * report a CFITSIO error (with user message)
 */
int FitsIO::cfitsio_error(const char* msg)
{
    char cfitsioMsg[FLEN_ERRMSG]; 

    if(fits_read_errmsg(cfitsioMsg))
    {
        fmt_error("%s: %s", msg, cfitsioMsg);
    }
    else
    {
        fmt_error("%s: [no message from CFITSIO]", msg); 
    }

    fits_clear_errmsg();
    
    return 1;
}


/*
 * report a CFITSIO error (no user message)
 */
int FitsIO::cfitsio_error()
{
  return cfitsio_error("");
}

/*
 * Read a FITS file and return an initialized FitsIO object for it,
 * or NULL if there are errors. 
 *
 * The Mem class is used to speed up loading the file. The optional mem_options
 * control whether the memory is mapped read-only or read/write (see class Mem).
 */
FitsIO* FitsIO::read(const char* filename, enum Mem::FileOptions options)
{
  Mem header(filename, options);
  if (header.status() != 0) 
    return NULL;
  return initialize(header);
}


/*
 * This static method returns an allocated FitsIO object given a Mem object
 * containing the data for the file. (header points to the data for the entire
 * file...).
 */
FitsIO* FitsIO::initialize(Mem& header)
{
  int status = 0;
  fitsfile *fitsio;
  
  int iomode;
  if (header.options() == Mem::FILE_DEFAULTS)
    iomode = READONLY;
  else
    iomode = READWRITE;
  
  fits_open_file(&fitsio, header.filename(), iomode, &status);

  if (status)
    return NULL;
  
  return new FitsIO(fitsio);
}


/*
 * get value for the given FITS keyword and return 0 if OK (found)
 */
int FitsIO::get(const char* keyword, double& val) const {
    if (! fitsio_)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio_, TDOUBLE, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(const char* keyword, float& val) const {
    if (! fitsio_)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio_, TFLOAT, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(const char* keyword, int& val) const {
    if (! fitsio_)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio_, TINT, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(const char* keyword, long& val) const {
    if (! fitsio_)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio_, TLONG, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(const char* keyword, unsigned char& val) const {
    if (! fitsio_)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio_, TBYTE, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(const char* keyword, short& val) const {
    if (! fitsio_)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio_, TSHORT, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(const char* keyword, unsigned short& val) const {
    if (! fitsio_)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio_, TUSHORT, (char*)keyword, &val, NULL, &status);
}


/*
 * find and return the value for the given FITS keyword, or NULL if not found
 * The returned value, if any, points to static data and should be saved by the
 * caller.
 * NOTE: this method is not thread-safe
 */
char* FitsIO::get(const char* keyword) const {
    if (! fitsio_) {
	error(noHdrErrMsg);
	return NULL;
    }
    int status = 0;
    return fits_read_key(fitsio_, TSTRING, (char*)keyword, buf_, NULL, &status) != 0 ? NULL : buf_;
}


/*
 * This is the same as get(const char*), but you supply the buffer to hold 
 * the result, which is then an empty string, if not found.
 */
char* FitsIO::get(const char* keyword, char* buf, int bufsz) const
{
    char* s = get(keyword);
    if (s)
	strncpy(buf, s, bufsz);
    else 
	buf[0] = '\0';
    return buf;
}


/*
 * these are static versions of the above
 */
int FitsIO::get(fitsfile* fitsio, const char* keyword, double& val) {
    if (! fitsio)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio, TDOUBLE, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(fitsfile* fitsio, const char* keyword, float& val) {
    if (! fitsio)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio, TFLOAT, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(fitsfile* fitsio, const char* keyword, int& val) {
    if (! fitsio)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio, TINT, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(fitsfile* fitsio, const char* keyword, long& val) {
    if (! fitsio)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio, TLONG, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(fitsfile* fitsio, const char* keyword, unsigned char& val) {
    if (! fitsio)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio, TBYTE, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(fitsfile* fitsio, const char* keyword, unsigned short& val) {
    if (! fitsio)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio, TUSHORT, (char*)keyword, &val, NULL, &status);
}


int FitsIO::get(fitsfile* fitsio, const char* keyword, short& val) {
    if (! fitsio)
	return error(noHdrErrMsg);
    int status = 0;
    return fits_read_key(fitsio, TSHORT, (char*)keyword, &val, NULL, &status);
}


/*
 * Find and return the value for the given FITS keyword, or NULL if not found.
 * The returned value, if any, points to static data and should be saved by the
 * caller.
 * NOTE: this method is not thread-safe
 */
char* FitsIO::get(fitsfile* fitsio, const char* keyword) {
    if (! fitsio) {
	error(noHdrErrMsg);
	return NULL;
    }
    int status = 0;
    return fits_read_key(fitsio, TSTRING, (char*)keyword, buf_, NULL, &status) != 0 ? NULL : buf_;
}


/* 
 * Insert the given FITS keyword and value and return 0 if OK.
 * If there is not enough space in the header, the file size is
 * automatically increased.
 */
int FitsIO::put(const char* keyword, double val, const char* comment) 
{
    int status = 0;
    if (fits_update_key(fitsio_, TDOUBLE, (char*)keyword, &val, (char*)comment, &status) != 0)
	return cfitsio_error();

    return status;
}


int FitsIO::put(const char* keyword, float val, const char* comment) 
{
    int status = 0;
    if (fits_update_key(fitsio_, TFLOAT, (char*)keyword, &val, (char*)comment, &status) != 0)
	return cfitsio_error();

    return status;
}


int FitsIO::put(const char* keyword, int val, const char* comment) 
{
    int status = 0;
    if (fits_update_key(fitsio_, TINT, (char*)keyword, &val, (char*)comment, &status) != 0)
	return cfitsio_error();

    return status;
}


int FitsIO::put(const char* keyword, const char* val, const char* comment) 
{
    int status = 0;
    if (fits_update_key(fitsio_, TSTRING, (char*)keyword, (char*)val, (char*)comment, &status) != 0)
	return cfitsio_error();

    return status;
}

/*
 * update CHECKSUM and DATASUM in all FITS HDUs
 */
int FitsIO::writeChecksum()
{
  int status = 0;
  int oldHduNum = 0;
  int numHdus = 0;
  int hduType;
  
  // save current HDU
  fits_get_hdu_num(fitsio_, &oldHduNum);
  
  // get # of HDUs
  fits_get_num_hdus(fitsio_, &numHdus, &status);
  if (status)
    return cfitsio_error();
  
  for (int i=1; i<=numHdus; i++) {  // loop through HDUs
    // move to next HDU
    fits_movabs_hdu(fitsio_, i, &hduType, &status);
    if (status)
      break;
    
    // write/update checksum and datasum
    fits_write_chksum(fitsio_, &status);
    if (status)
      break;
  }
  if (status)
    return cfitsio_error();
  
  // restore current HDU
  fits_movabs_hdu(fitsio_, oldHduNum, &hduType, &status);
  return status;
}

// get fits file name
int FitsIO::filename(char *fname)
{
  int status = 0;
  return fits_file_name(fitsio_, fname, &status);
}
