/******************************************************************************
* E.S.O. - VLT project
*
* slxAliasToShortFITS.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  18/09/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxAliasToShortFITS - converts an alias into the Short-FITS keyword 
* 
*   SYNOPSIS
*   #include "slx.h"
*
*
*   ccsCOMPL_STAT slxAliasToShortFITS(const slxKEYW_LINE   alias,
*                                     const vltLOGICAL     loadTable,
*                                     const slxFILENAME    tableFileName,
*                                     slxKEYW_LINE         shortFITS,
*                                     ccsERROR             *error)
* 
* 
*   DESCRIPTION
*   The function converts an alias into the complete Short-FITS keyword.
*   If the conversion was not possible, the first character of the "shortFITS"
*   buffer will be a NULL character.
*
*   When it is desired that the function loads a new set of alias definitions,
*   the parameter "loadTable" must be TRUE, and the "tableFileName" must
*   specify the filename.
*
*
*   alias          IN   Points to buffer containing the alias.
*
*   loadTable      IN   Specifies if the function should update the Alias
*                       Conversion Table. If this is the case, the value should
*                       TRUE, otherwise FALSE.
*
*   tableFileName  IN   The name of the Alias Conversion Table to load. If
*                       "loadTable" is FALSE, this parameter is ignored.
*
*   shortFITS      OUT  Points to buffer which will contain the Short-FITS
*                       keyword. If the alias was not defined, the first 
*                       element of this buffer will be a NULL character.
*          
*   error          OUT  Indicates the type of error that possibly occurred.
* 
*   
*
*   RETURN VALUES 
*   SUCCESS  If the operation was successfull.
*   FAILURE  If the operation failed. The following error codes can be 
*            produced by the function:
*
*            slxERR_OPEN_ALIAS_TABLE      Couldn't open the alias conversion
*                                         table.
*
*            slxERR_CRE_ALIAS_HASH_TABLE  Couldn't create the alias conversion
*                                         table.
*
*            slxERR_ALIAS_NOT_DEF         The alias referenced is not defined
*                                         in the table.
*
*
*   CAUTIONS 
*   The function keeps track of a local hash table for the conversions. It is
*   important that it is initialized with the correct alias definitions.
*
*   The function is private to each application, which then has to initialize
*   it itself.
*
*   Initializing the function should not be done while searching for an 
*   alias. I.e. the table will only be created there will not be searched
*   for the alias in the table.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxAliasToShortFITS.c,v 1.145 2005/07/27 09:03:56 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Hash table used for the conversion
 */
static slxALIAS_HASH    *hashTable[slxMAX_NO_OF_ALIASES];

/* Function prototyping
 */
unsigned slxHash(const char *string);
slxALIAS_HASH *slxLookUp(const char *string);
slxALIAS_HASH *slxInstall(const char *alias, const char *shortFITS);

/* Function definition
 */
ccsCOMPL_STAT slxAliasToShortFITS(const slxKEYW_LINE  alias,
                                  const vltLOGICAL    loadTable,
                                  const slxFILENAME   tableFileName,
                                  slxKEYW_LINE        shortFITS,
                                  ccsERROR            *error)
/* The function is based on the algorithm described in 
 * "The C (ANSI C) Programming Language", Brian W. Kernighan, and
 * Dennis M. Ritchie.
 */
{
    char                    aliasTableFileName[256];
    char                    inBuf[256];
    char                    inShortFITS[slxMAX_FITS_LI_LEN];
    char                    inAlias[slxMAX_FITS_LI_LEN];
    char                    *comPathPtr;
    FILE                    *filePtr;    
    static vltUINT8         calledBefore;
    slxALIAS_HASH           *hashTablePtr;
    slxALIAS_HASH           *hashTableDel;
    vltUINT16               n;


    
    *shortFITS = '\0';

    if (!calledBefore || (loadTable == TRUE))
        {        
        /* Delete a possible existing table and build a new one
         */        
        for (n = 0; n < slxMAX_NO_OF_ALIASES; n++)
            {
            if (hashTable[n] != NULL)
                {
                hashTablePtr = hashTable[n];
                do
                    {
                    hashTableDel = hashTablePtr;
                    hashTablePtr = hashTablePtr->next;
                    free(hashTableDel->shortFITS); 
                    free(hashTableDel->alias); 
                    free(hashTableDel); 
                    }
                while (hashTablePtr != NULL);
                hashTable[n] = NULL;
                }
            }    /* end-for (n = 0; n < slxMAX_NO_OF_ALIASES; n++) */

        /* Build the new hash table. Alias Conversion tables are located in
	 * $INS_ROOT/SYSTEM/Alias
         */
	if (slxResolvePath(tableFileName, &comPathPtr, error) == FAILURE)
	    return FAILURE;
	if ((*comPathPtr != '/') && (*comPathPtr != '.'))
	    {
	    if (slxBuildInsFileName("SYSTEM", "Alias", "", tableFileName,
				    aliasTableFileName, error) == FAILURE)
		return FAILURE;
	    }
	else
	    {
	    slxCOPY_MAX_N_BYTES(aliasTableFileName, comPathPtr, 
				slxFILENAME_LEN, error);
	    }
	if (strstr(aliasTableFileName, slxALIAS_FILE_EXT) == NULL)
	    strcat(aliasTableFileName, slxALIAS_FILE_EXT);

        if ((filePtr = fopen(aliasTableFileName, "r")) == NULL)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_ALIAS_TABLE, 
                      __FILE__, NULL, modUNIX, aliasTableFileName);
            return FAILURE;
            }

        while (fgets(inBuf, (int)256, filePtr) != NULL)
	    {
            if ((inBuf[0] != '#') && (strlen(inBuf) > 5) &&
                (inBuf[0] != ' ') && (inBuf[0] != '\t'))
                {
                if (sscanf(inBuf, "%s %s", inShortFITS, inAlias) == 2)
                    {
                    if (slxInstall(inAlias, inShortFITS) == NULL)
                        {
                        errAdd(error, slxMODULE_NAME, 
                               slxERR_CRE_ALIAS_HASH_TABLE, __FILE__, NULL,
                               aliasTableFileName); 
                        fclose(filePtr);
                        return FAILURE;
                        }
                    }
                }
            }    /* end-while (fgets(inBuf(inBuf, (int)256, ... */
        fclose(filePtr);
        calledBefore = 1;
        }    /* end-if (!calledBefore || (loadTable == TRUE)) */

    /* Look up the alias - if the function did not just create the hash table
     */
    if (loadTable == FALSE)
        {
        if ((hashTablePtr = slxLookUp(alias)) == NULL)
            {
            errAdd(error, slxMODULE_NAME, slxERR_ALIAS_NOT_DEF, __FILE__, 
                   NULL, alias);  
            return FAILURE;
            }
        strcpy(shortFITS, hashTablePtr->shortFITS);
        }

    return SUCCESS;
}

/* Generate hash value for the string
 */
unsigned slxHash(const char *string)
{
    unsigned hashVal;

    for (hashVal = 0; *string != '\0'; string++)
        hashVal = *string + 31 * hashVal;
 
    return (hashVal % slxMAX_NO_OF_ALIASES);
}

/* Searchs for string in the hash table
 */
slxALIAS_HASH *slxLookUp(const char *string)
{
    slxALIAS_HASH  *np;

    for (np = hashTable[slxHash(string)]; np != NULL; np = np->next)
        if (strcmp(string, np->alias) == 0)
            return np;    /* Found */

    return NULL;    /* Not found */
}

/* Install write new name to the hash table
 */
slxALIAS_HASH *slxInstall(const char *alias, const char *shortFITS)
{
    slxALIAS_HASH   *np;
    unsigned        hashVal;

    if ((np = slxLookUp(alias)) == NULL)
        {
        np = (slxALIAS_HASH *)malloc(sizeof(slxALIAS_HASH));
        if (np == NULL || (np->alias = slxStrDup(alias)) == NULL)
            return NULL;
        hashVal = slxHash(alias);
        np->next = hashTable[hashVal];
        hashTable[hashVal] = np;
        }
    else
        free((void *)np->shortFITS);

    if ((np->shortFITS = (char *)slxStrDup(shortFITS)) == NULL)
        return NULL;

    return np;
}


/*
 *****************************************************************************
 */




/*___oOo___*/
