/******************************************************************************
* E.S.O. - VLT project
*
* slxConvertToShort.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  28/07/94  Created 1. version 
*/

/************************************************************************
*   NAME
*   slxConvertToShort - converts a set-up file from normal to Short-FITS 
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxConvertToShort(const slxFILENAME  setUpFileName,
*                                   ccsERROR           *error)
* 
* 
*   DESCRIPTION
*   The subroutine converts an entire set-up file from the Normal FITS format
*   to the Short-FITS format. The set-up file in the Short-FITS format 
*   overwrites the old set-up in the Normal FITS format. 
*
* 
*   setUpFileName  IN  Name of the set-up file. Can be either the complete 
*                      path, or just the filename. In the latter case there 
*                      will be searched after the set-up according to the 
*                      environment variable "INS_SETUPPATH".
*                      If not set, uses the current directory.
*
*   error          IN  CCS error structure indicating the kind of error which
*                      possibly occurred.
*
*
*   ENVIRONMENT
*   INS_SETUPPATH      The environment variable contains a number of 
*                      concatenated paths where set-up files are located.
*                      If not set, uses the current directory.
*
*
*   RETURN VALUES 
*   SUCCESS	If the operation was successful.
*   FAILURE	If something went wrong. In this case the CCS error structure
*               indicates the error. The following types are possible:
*  
*               slxERR_LOCATE_FILE    The set-up file could not be located.
*
*               slxERR_OPEN_FILE      The set-up file could not be opened.
*             
*               slxERR_FILE_FORMAT    One or more lines in the set-up file has
*                                     a wrong format (is not Normal FITS).   
*
*               slxERR_RENAME_FILE    An error occurred while renaming the
*                                     temporary file.
*
*               slxERR_INS_SETUPPATH  Could not get the value of the
*                                     environment variable INS_SETUPPATH.
*
*               The function calls slxLocateSetupFile(), slxDetectFormat() and
*               slxGetPath() and may return errors generated by these 
*               functions.
*
*   CAUTIONS 
*   The operation will overwrite the old set-up with the keywords in the
*   Short-FITS format.
*
*
*   SEE ALSO
*   slxNormalToShortFITS(), slxConvertToNormal(), slxShortToNormalFITS()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxConvertToShort.c,v 1.145 2005/07/27 09:05:14 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>
#include <stdio.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxConvertToShort(const slxFILENAME  setUpFileName, 
				ccsERROR           *error)
{
    char             path[256];
    char             *tempFileName = "sLxNoRmAlToShOrT";
    char             inBuf[128];
    char             shortFITS[128];
    FILE             *filePtrOld;
    FILE             *filePtrNew;
    FILE             *filePtrTest;
    slxFILENAME      tempComplFileName;
    slxFILENAME      completeFileName;
    slxSETUP_FORMAT  type;
    vltUINT8         protRegFound;



    if (slxLocateSetupFile(setUpFileName, completeFileName, error) != SUCCESS)
        return FAILURE;

    /* Check if the format of the file is Normal FITS
     */
    if (slxDetectFormat(completeFileName, &type, error) == FAILURE)
        return FAILURE;
    if (type != slxNORMAL_FITS)
        {
        errAdd(error, slxMODULE_NAME, slxERR_FILE_FORMAT, __FILE__, NULL,
               completeFileName);  
        return FAILURE;
        }

    /* Test if the new file can be accessed (is maybe write protected?)
     */
    if ((filePtrTest = fopen(completeFileName, "a+")) == NULL)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, 
                  NULL, modUNIX, completeFileName);
        return FAILURE;
        }
    fclose(filePtrTest);

    if (slxGetPath(completeFileName, path, error) != SUCCESS)
        return FAILURE;

    strcpy(tempComplFileName, path);
    strcat(tempComplFileName, tempFileName);

    if ((filePtrOld = fopen(completeFileName, "r")) == NULL)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, 
                  NULL, modUNIX, completeFileName);
        return FAILURE;
        }

    if ((filePtrNew = fopen(tempComplFileName, "w")) == NULL)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, 
                  NULL, modUNIX, tempComplFileName);
        return FAILURE;
        }
     
    protRegFound = 0;
    while (fgets(inBuf, 128, filePtrOld) != NULL)
        {
        if (strncmp(inBuf, "HIERARCH ESO SUP PROT", 
                    strlen("HIERARCH ESO SUP PROT")) == 0)
	    {

            /*
             * Have to give "HIERARCH ESO SUP PROT" a special treatment 
             * (no value)
             */

            if (protRegFound)
                strcpy((char *)shortFITS, slxEND_PROT_REG_SHORT);
            else
                strcpy((char *)shortFITS, slxSTART_PROT_REG_SHORT);

            protRegFound = 1;
            }
        else if (slxNormalToShortFITS(inBuf, shortFITS, error) != SUCCESS)
            {
            fclose(filePtrOld); 
            fclose(filePtrNew);
            return FAILURE;
            }

        /*
         * Save only the keyword if it is a hierarchical one, a comment or
         * a blank line (initiated with either ' ', '\t' or '\n')
         */

        if ((*(shortFITS + 3) == '.') || (*shortFITS == '#') || 
            (*shortFITS == ' ') || (*shortFITS == '\t') || 
            (*shortFITS == '\n'))
            {
            if (fputs(shortFITS, filePtrNew) == EOF)
                {
                fclose(filePtrOld); 
                fclose(filePtrNew);
                errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                          NULL, modUNIX, tempComplFileName);
                return FAILURE;
                }
            }
        }    /* end-while (fgets(inBuf, 128, filePtrOld) != NULL) */
    fclose(filePtrOld);
    fclose(filePtrNew);

    if (rename(tempComplFileName, completeFileName) != 0)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_RENAME_FILE, __FILE__, 
                  NULL, modUNIX, tempComplFileName, completeFileName);
        return FAILURE;
        }

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/

