/******************************************************************************
* E.S.O. - VLT project
*
* slxCreateFITS.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  28/07/94  Created 1. version
* - - - - - - -MAR2002 - - - - - - - - -
* epozna    17/07/02  compillation on  gcc 3.0.4
*                     fixed warnings in 'scanf' of integer values
*/

/************************************************************************
*   NAME
*   slxCreateFITS - generates a FITS keyword command line
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxCreateFITS(const char         *category,
*                               const char         *subSystem,
*                               const char         *parameter,
*                               const slxPAR_TYPE  *parameterType,
*                               const char         *stringValue,
*                               const vltLOGICAL   logicalValue,
*                               const vltINT32     integerValue,
*                               const vltUINT8     precision,
*                               const vltLOGICAL   exponential,
*                               const vltDOUBLE    doubleValue,
*                               const char         *comment,
*                               char               *fitsLine,
*                               ccsERROR           *error)
*
*
*   DESCRIPTION
*   NOTE: A new function slxCreateFits2() is now supported providing a more
*   convenient interface, and providing two sub-system keywords.
*   This new function should be used.
*
*   The function generates a FITS keyword line from the input
*   parameters. This is done in accordance with the rules specified in
*   the "Data Interface Requirements" document (ARC-SPE-ESO-0000-0001/1.4),
*   and in "NOST Implementation of the Flexible Image Transport System 
*   - FITS, November 6, 1991".
*   
*   The FITS keyword generated can be either a standard FITS keyword, 
*   i.e. only a parameter, a value and a comment:
*
*   "<par>    = <value>                         / <comment>"
*
*   or it can be a hierarchical FITS keyword line, i.e. with the format: 
*
*   "HIERARCH ESO <cat> [<sub>] <par> = <value> / <comment>"
*  
*   The first two keywords ("HIERARCH ESO") are generated automatically.
*   
*   All strings passed to the function must be terminated with '\0'.
*
*   If a standard FITS keyword should be generated, slxSTD_FITS, should used 
*   as input for the category. If there is no subsystem, slxNULL_STR, should 
*   be the input value for this parameter. 
*
*   The comment should be given only as the actual text -  do not append any
*   "/" or try to format it in any way.
*
*   A comment line, i.e. "COMMENT <comment>" shall be created, the input
*   parameters "category", "subSystem" and "parameter" should be slxSTD_FITS, 
*   slxNULL_STR and slxNULL_STR, and the comment given in in the "comment"
*   parameter.
*
*   The generated FITS line is stored in the buffer pointed to by "fitsLine".
*   This buffer must have the a length of at least 81 characters.
*
*   If the generated FITS line is longer than the maximum FITS line length 
*   (80 characters) the characters beyond the 80th position, are not taken 
*   into account.
*
*
*   category      IN  Pointer to string buffer containing the category of the 
*                     FITS keyword, e.g. "INS", "DET", ... . If it is a 
*                     standard FITS keyword (only the parameter), this string
*                     should be, slxSTD_FITS. A category can maximum be 
*                     3 characters long.
*
*   subSystem     IN  Pointer to string buffer containing the subsystem.
*                     If this is not used, slxNULL_STR must be the input
*                     parameter.
*   
*   parameter     IN  Pointer to string buffer containing the name of the 
*                     parameter. This input parameter is mandatory.
*
*   parameterType IN  The type of the parameter. Can be either slxSTRING,
*                     slxLOGICAL, slxINTEGER or slxDOUBLE.
*
*   stringValue   IN  Pointer to buffer containing the possible string value.
*                     If not used, i.e. "parameterType" is not a string, this
*                     parameter is ignored (can be slxNULL_STR).
*
*   logicalValue  IN  Can be either TRUE or FALSE. If not used, the 
*                     parameter is ignored.
*
*   integerValue  IN  Contains an integer value. If "parameterType" is not
*                     integer, this is ignored.
*
*   precision     IN  Used when the input value is of type double. Indicates
*                     the number of decimals after the point. If the value is
*                     not a double, this parameter is ignored.
*
*   exponential  IN   Indicates for a double type parameter if it should be 
*                     be written as an exponential number or a plain number.
*                     Can be either TRUE for exponential or FALSE. If 
*                     the value is not a double, this parameter is ignored.
*
*   doubleValue  IN   Contains the value when parameter type is double. If the
*                     type is not double, this parameter is ignored.
*
*   comment       IN  Used to give in the comment; should be plain text
*                     terminated with "\0".
*
*   fitsLine      OUT The buffer in which the generated FITS will be stored.
*                     Must be allocated by the calling application. The 
*                     function will fill exactly 80 characters into this 
*                     buffer. THE LENGTH OF THE BUFFER HOWEVER, MUST BE AT
*                     LEAST 81!! If the actual FITS line contents is less than 
*                     that, there is zeropadded with blanks to obtain 80 
*                     characters. If the generated FITS line is longer, the 
*                     line is truncated. There will be written a '\0' 
*                     character in the 81th column.
*
*   error         OUT Indicates the type of a possible error occurring.
*
*
*   RETURN VALUES 
*   SUCCESS    If the operation was successful.
*   FAILURE    If the operation failed. The function only fails if the 
*              parameter type is unknown, otherwise it will always try to
*              generate a FITS line. The following error code can be 
*              returned:
*
*              slxERR_CREATE_FITS      An error occurred while creating FITS
*                                      keyword - input parameters somehow
*                                      wrong.
*
*              slxERR_INTEGER_TOO_LRG
*              slxERR_DOUBLE_TOO_LRG   The value entered is larger than allowed
*                                      (string value too long).
*             
*
*   CAUTIONS 
*   Beware that all string buffers must be allocated by the calling process,
*   and terminated by a null character ('\0').
*
*   Quotes in strings and complex floating point values are not supported in 
*   this version. 
*
*
*   EXAMPLES
*   
*   #include "slx.h"
*
*   ccsERROR     error;
*   char         *stringVal = "";
*   vltLOGICAL   logicalVal;
*   long int     integerVal;
*   vltUINT8     prec;
*   vltLOGICAL   expo;
*   double       doubleVal;
*   char         newFitsLine[slxMAX_FITS_LI_LEN];
*   char         com[slxSTD_FITS_COM_MAX_LEN];
*
*
*   (Get the parameters (would probably read them from OLDB))
*    
*   (This is a double value)
*
*   parType = slxDOUBLE;
*   prec = 4;
*   expo = TRUE;
*   doubleVal = 12.45;
*
*   strcpy(comment, "Detector integration time");
*
*    
*   (Create the FITS line)
*
*   if (slxCreateFITS("DET", 
*                     "WIN1", 
*                     "DIT", 
*                     parType, 
*                     stringVal,
*                     logicalVal,
*                     integerVal,
*                     prec,
*                     expo,
*                     doubleVal,
*                     com,
*                     newFitsLine,
*                     &error) == FAILURE)
*       {
*       ... error processing ..
*       }
*   
*   (The FITS is generated save it, print it ...)
*   
*  
*   printf("New FITS line: %s\n", newFitsLine);
*
*   ...
*
*
*   Output ->
*
*   HIERARCH ESO DET WIN1 DIT   = 1.245E+1         / Detector integration time
*
*    
*   SEE ALSO
*   slxCreateWriteFITS(), slxNormalToShortFITS()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxCreateFITS.c,v 1.145 2005/07/27 09:06:35 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>
#include <limits.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxCreateFITS(const char         *category,
                            const char         *subSystem,
                            const char         *parameter,
                            const slxPAR_TYPE  parameterType,
                            const char         *stringValue,
                            const vltLOGICAL   logicalValue,
                            const vltINT32     integerValue,
                            const vltUINT8     precision,
                            const vltLOGICAL   exponential,
                            const vltDOUBLE    doubleValue,
                            const char         *comment,
                            char               *fitsLine,
                            ccsERROR           *error)
/* The position of various elements in the line can be adjusted by means
 * of constants. There is a constant for adjusting the position of the
 * equal sign, the comment and so forth. Don't try to change the code
 * to change the format, try first to fiddle with the constants.
 *
 * A bug fix for version 1.2 makes this obsolete for Standard FITS keywords.
 */
{
    char      newFitsLine[2 * slxMAX_FITS_LI_LEN];
                                    /* Array in which to build up the FITS */
                                    /* line */
    char      string[slxMAX_FITS_LI_LEN];
                                    /* To contain the new string */
    char      format[64];           /* For creating format for sprintf() */
    char      valueLine[80];        /* Help line to generate FITS line */
    char      state[16];            /* Help character */
    char      numberStr[64];        /* Buffer to contain the integer value */
                                    /* as a string */
    char      *chrPtr;             
    vltUINT8  eqPos;                /* Position of equal sign */
    vltUINT8  hierarchical = 0;     /* Indicates that the keyword is */
                                    /* hierarchical */

    /* gcc: introduced tmp variables to avoid compillation 
     *      warning of 'scanf' integers
     */
    vltINT32 tmppos;
    vltINT32 tmplen;


    memset((char *)fitsLine, ' ', (size_t)slxMAX_FITS_LI_LEN);
    memset((char *)newFitsLine, 0, (size_t)(2 * slxMAX_FITS_LI_LEN));
    memset((char *)string, 0, (size_t)slxMAX_FITS_LI_LEN);

    if (strlen(category) > slxCATEGORY_LEN)
        {
        errAdd(error, slxMODULE_NAME, slxERR_CREATE_FITS, __FILE__, NULL);
        return FAILURE;
        }

    /* Check if a single comment line should be made
     */
    if (strlen(parameter) == 0)
        {
        strcpy((char *)newFitsLine, "COMMENT ");
        strcat((char *)newFitsLine, comment);
        memcpy((char *)fitsLine, (char *)newFitsLine, 
               (size_t)slxMAX_FITS_LI_LEN);

       /* If the newFitsLine was shorter that 80 characters zero pad with 
        * blanks to obtain the desired length.
        */
       if (strlen(fitsLine) < slxMAX_FITS_LI_LEN)
           memset((char *)(fitsLine + strlen(fitsLine)), ' ', 
                  (size_t)(slxMAX_FITS_LI_LEN - strlen(fitsLine)));

       /* Set the last character to '\0'
	*/
       *(fitsLine + slxMAX_FITS_LI_LEN) = '\0';

       /* It might be that there is a newline character in the comment,
        * this is undesirable. Check if there are a '\n' within the 
        * first 80 characters and replace by a ' ' if there is.
        */
       if ((chrPtr = strchr((char *)fitsLine, '\n')) != NULL)
           {
           if ((chrPtr - fitsLine + 1) < slxMAX_FITS_LI_LEN)
               *chrPtr = ' ';
           }

       *(fitsLine + slxMAX_FITS_LI_LEN) = '\0';
       return SUCCESS;
       }

    /* Build standard keyword "PAR = <value>"
     * otherwise build "HIERARCH ESO CAT [SUB] PAR =" if hierarchical keyword
     */    
    else if (strlen(category) == 0)
        {      
        /* Standard FITS: "PAR = <value> / <comment>"
         * Fist, generate the format for sprintf()
         */
        sprintf(format, "%%-%ds= ", (slxSTD_FITS_EQ_POS - 1));
        sprintf(newFitsLine, format, parameter);
        eqPos = slxSTD_FITS_EQ_POS;
	

	/* Generate a value string (including the comment).
	 *
	 * Common for all the values is, that first a format string is 
	 * generated to be able to perform the sprintf(). This is done to
	 * have a flexible solution where the format can be changed via
	 * the header file "slx.h"
	 */
	memset((char *)format, 0, (size_t)64);
	switch(parameterType) 
	    {
	    case slxSTRING:
                { 
		/* Put string and comment: 
		 * "<parameter> = '<string>'      / <comment>"
		 * in the "valueLine"
		 */           
		if (strlen(stringValue) < slxFITS_MIN_STR_LEN)
		    {
		    /*comp gcc: */
		    tmplen=(slxFITS_MIN_STR_LEN - strlen(stringValue));
		    tmppos= (slxSTD_FITS_COM_POS - (eqPos + 12));
		    /*sprintf(format, "'%%s%%-%ldc'%%-%ic/ %%s",*/
		    sprintf(format, "'%%s%%-%dc'%%-%dc/ %%s",
			    tmplen,tmppos );
		    sprintf(valueLine, format, stringValue, ' ', ' ', comment);
		    }
		else if ((strlen(stringValue) > slxHIER_FITS_MAX_STR_LEN) &&
			 hierarchical)
		    {
		    sprintf(valueLine, "'%s' / %s", stringValue, comment);
		    }
		else if ((strlen(stringValue) > slxSTD_FITS_MAX_STR_LEN) &&
			 !hierarchical)
		    {
		    sprintf(valueLine, "'%s' / %s", stringValue, comment);
		    }
		else
		    { 
		    /* gcc:  */
		    tmppos=(slxSTD_FITS_COM_POS - (eqPos + 12) - 
			    (strlen(stringValue) - slxFITS_MIN_STR_LEN)); 
		    /* sprintf(format, "'%%s'%%-%ldc/ %%s",*/
		    sprintf(format, "'%%s'%%-%dc/ %%s", tmppos);
		    sprintf(valueLine, format, stringValue, ' ', comment);
		    }
		break;
		}
	    case slxLOGICAL:
		{
		if (logicalValue == TRUE)
		    strcpy(state, "T ");
		else 
		    strcpy(state, "F ");
		sprintf(format, "%%%d.2s%%%ds/ %%s", 21, 9);
		sprintf(valueLine, format, state, " ", comment);
		break;
		}
	    case slxINTEGER:
		{
		sprintf(numberStr, "%d%c", integerValue, '\x0');
		/* Check if the integer value is too long
		 */
		if (strlen(numberStr) > 20)
		    {
		    errAdd(error, slxMODULE_NAME, slxERR_INTEGER_TOO_LRG, 
			   __FILE__, NULL);
		    return FAILURE;
		    }
		/*gcc:*/ 
		tmplen=strlen(numberStr);
		/* sprintf(format, "%%%d.%lds%%%ds/ %%s",*/
		sprintf(format, "%%%d.%ds%%%ds/ %%s",
			 20,tmplen ,10);
		sprintf(valueLine, format, numberStr, " ", comment);
		break;
		}
	    case slxDOUBLE:
		{
		if (exponential == TRUE)
		    {
		    sprintf(format, "%%.%dE%%c", precision);
		    sprintf(numberStr, format, doubleValue, '\x0');
		    }
		else
		    {
		    sprintf(format, "%%.%df%%c", precision);
		    sprintf(numberStr, format, doubleValue, '\x0');
		    if (precision == 0)
			{
			*(numberStr + strlen(numberStr)) = '.';
			*(numberStr + strlen(numberStr) + 1) = '\x0';
			}
		    }
		/* Check if the double value is too long
		 */
		if (strlen(numberStr) > 20)
		    {
		    errAdd(error, slxMODULE_NAME, slxERR_DOUBLE_TOO_LRG, 
			   __FILE__, NULL);
		    return FAILURE;
		    }
		memset((char *)format, 0, (size_t)64);
		
		/*gcc:*/
		tmplen=strlen(numberStr);
		/*sprintf(format, "%%%d.%lds%%%ds/ %%s",*/
		sprintf(format, "%%%d.%ds%%%ds/ %%s", 
			20,tmplen ,10);
		sprintf(valueLine, format, numberStr, " ", comment);
		break;
		}
	    default:
		{
		/* There's something wrotten in the "slxFITS_VAL" structure
		 */		
		errAdd(error, slxMODULE_NAME, slxERR_CREATE_FITS, __FILE__, 
		       NULL);
		return FAILURE;
		}
	    }    /* end-switch(parameterType) */
        }    /* if standard FITS (not hierarchical) */
    else
        {
        hierarchical = 1;

        /* Hierachical FITS: "HIERARCH ESO CAT [SUB] PAR = <value> / <comment>"
         */ 
        if (strlen(subSystem) > 0)
            {
	    /* gcc: */
	    tmppos=(slxHIER_FITS_EQ_POS - strlen(slxFITS_ESO_PREFIX) -
                    strlen(category) - strlen(subSystem) - 
                    strlen(parameter) - 1);
	    /*sprintf(format, "%%s%%s %%s %%s%%%ld.2s%%c",*/
	    sprintf(format, "%%s%%s %%s %%s%%%d.2s%%c",
		    tmppos);
            sprintf(newFitsLine, format, slxFITS_ESO_PREFIX, category, 
                    subSystem, parameter, "= ", '\x0');
            }
        else
            {
            /* gcc: */
	    tmppos=(slxHIER_FITS_EQ_POS - strlen(slxFITS_ESO_PREFIX) -
                    strlen(category) - strlen(parameter));
	    /*sprintf(format, "%%s%%s %%s%%%ld.2s%%c", */
	    sprintf(format, "%%s%%s %%s%%%d.2s%%c",
                    tmppos);
            sprintf(newFitsLine, format, slxFITS_ESO_PREFIX, category,
                    parameter, "= ", '\x0');
            }
        eqPos = slxHIER_FITS_EQ_POS;

	/* Generate a value string (including the comment).
	 *
	 * Common for all the values is, that first a format string is 
	 * generated to be able to perform the sprintf(). This is done to
	 * have a flexible solution where the format can be changed via
	 * the header file "slx.h"
	 */       
	memset((char *)format, 0, (size_t)64);
	switch(parameterType) 
	    {
	    case slxSTRING:
                { 
		/* Put string and comment: 
		 * "<parameter> = '<string>'      / <comment>"
		 * in the "valueLine"
		 */
		if (strlen(stringValue) < slxFITS_MIN_STR_LEN)
		    {
		    /* gcc:  */
		    tmplen= (slxFITS_MIN_STR_LEN - strlen(stringValue));
		    tmppos= (slxHIER_FITS_COM_POS - (eqPos + 12));
		    /* sprintf(format, "'%%s%%-%ldc'%%-%ic/ %%s",*/
		    sprintf(format, "'%%s%%-%dc'%%-%dc/ %%s",    
			    tmplen, tmppos );
		    sprintf(valueLine, format, stringValue, ' ', ' ', comment);
		    }
		else if ((strlen(stringValue) > slxHIER_FITS_MAX_STR_LEN) &&
			 hierarchical)
		    {
		    sprintf(valueLine, "'%s' / %s", stringValue, comment);
		    }
		else if ((strlen(stringValue) > slxSTD_FITS_MAX_STR_LEN) &&
			 !hierarchical)
		    {
		    sprintf(valueLine, "'%s' / %s", stringValue, comment);
		    }
		else
		    {
		    /* gcc:*/
		    tmppos= (slxHIER_FITS_COM_POS - (eqPos + 12) - 
			     (strlen(stringValue) - slxFITS_MIN_STR_LEN));
		    /*sprintf(format, "'%%s'%%-%ldc/ %%s",*/ 
		    sprintf(format, "'%%s'%%-%dc/ %%s",
			  tmppos );
		    sprintf(valueLine, format, stringValue, ' ', comment);
		    }
		break;
		}
	    case slxLOGICAL:
		{
		if (logicalValue == TRUE)
		    strcpy(state, "T ");
		else 
		    strcpy(state, "F ");
		sprintf(format, "%%%d.2s/ %%s", 
			(slxHIER_FITS_COM_POS - (eqPos + 2)));
		sprintf(valueLine, format, state, comment);
		break;
		}
	    case slxINTEGER:
		{
		sprintf(numberStr, "%d%c%c", integerValue, ' ', '\x0');
		sprintf(format, "%%%d.%ds/ %%s", (slxHIER_FITS_COM_POS - 
			(eqPos + 2)), (int)strlen(numberStr));
		sprintf(valueLine, format, numberStr, comment);
		break;
		}
            case slxDOUBLE:
		{
		if (exponential == TRUE)
		    {
		    sprintf(format, "%%.%dE%%c%%c", precision);
		    sprintf(numberStr, format, doubleValue, ' ', '\x0');
		    }
		else
		    {
		    sprintf(format, "%%.%df%%c%%c", precision);
		    sprintf(numberStr, format, doubleValue, ' ', '\x0');
		    if (precision == 0)
			{
			*(numberStr + strlen(numberStr) - 1) = '.';
			*(numberStr + strlen(numberStr)) = ' ';
			*(numberStr + strlen(numberStr) + 1) = '\x0';
			}
		    }
		memset((char *)format, 0, (size_t)64);
		sprintf(format, "%%%d.%ds/ %%s", (slxHIER_FITS_COM_POS - 
			(eqPos + 2)), (int)strlen(numberStr));
		sprintf(valueLine, format, numberStr, comment);
		break;
		}
	    default:
		{
		/*
		 * There's something wrotten in the "slxFITS_VAL" structure
		 */
		errAdd(error, slxMODULE_NAME, slxERR_CREATE_FITS, __FILE__, 
		       NULL);
		return FAILURE;
		}
	    }    /* end-switch(parameterType) */
        }    /* end-else hierarchical keyword */

    strcat(newFitsLine, valueLine);
 
    /*
     * Copy over the new FITS line in the output array
     * - only slxMAX_FITS_LI_LEN characters are copied possibly more
     * characters are thus truncated.
     */

    memcpy((char *)fitsLine, (char *)newFitsLine, (size_t)slxMAX_FITS_LI_LEN);

    /*
     * Check that in case the line was truncated, that  it is only the
     * comment that have been truncated and not the value as well
     */

    if (strlen(newFitsLine) > strlen(fitsLine))
        {
        if ((chrPtr = strstr(newFitsLine, "/ ")) == NULL)
            {
            errAdd(error, slxMODULE_NAME, slxERR_CREATE_FITS, __FILE__, NULL);
            return FAILURE;
            }
        if ((chrPtr - newFitsLine) > (slxMAX_FITS_LI_LEN + 1))
	    {
            errAdd(error, slxMODULE_NAME, slxERR_CREATE_FITS, __FILE__, NULL);
            return FAILURE;
            }
        }

    /*
     * If the newFitsLine was shorter that 80 characters zero pad with blanks
     * to obtain the desired length.
     */

    if (strlen(fitsLine) < slxMAX_FITS_LI_LEN)
        memset((char *)(fitsLine + strlen(fitsLine)), ' ', 
               (size_t)(slxMAX_FITS_LI_LEN - strlen(fitsLine)));

    /*
     * It might be that there is a newline character in the comment,
     * this is undesirable. Check if there are a '\n' within the 
     * first 80 characters and replace by a ' ' if there is.
     */

    if ((chrPtr = strchr((char *)fitsLine, '\n')) != NULL)
        {
        if ((chrPtr - fitsLine + 1) < slxMAX_FITS_LI_LEN)
            *chrPtr = ' ';
        }



    *(fitsLine + slxMAX_FITS_LI_LEN) = '\0';
    return SUCCESS;
}



/*
 *****************************************************************************
 */


  
/*___oOo___*/


