/******************************************************************************
* E.S.O. - VLT project
*
* slxGetDicRecSeq.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  02/04/96  created
* epozna    11/04/03  VLTSW20020630 slxGetDicRecSeq broken into parts so 
*                     that reusable in oslx (slxSetUnitId,slxParseUnitConv)
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized 
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxGetDicRecSeq - return keywords sequentially from a Dictionary
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxGetDicRecSeq(const vltLOGICAL   init,
* 			          vltLOGICAL         *gotRecord,
* 			          slxDYN_BUF         dicBuf,
* 				  slxDIC_HANDLE      *dicInfo,
* 				  ccsERROR           *error)
*
*   DESCRIPTION
*   The function is used to return records in a Dictionary sequentially.
*
*   If the record for the parameter is found, the values from the dictionary
*   are stored in the SLX Dictionary Handle Structure. The context of this
*   structure is:
*
*   Parameter Name:    Name of the parameter, e.g. "DET DATE"
*   Class:             Class of the parameter.
*   Context:           The context in which it is used.
*   Type:              The type of the parameter. Possible types are "string"
*                      "logical", "integer" and "double".
*   Value Format:      Format in which the parameter should be represented.
*                      The format representation from ANSI-C is used (see
*                      e.g. printf()).
*   Unit:              The SI or derived SI unit for the parameter.
*   Comment Format:    The standard comment (if any) for the parameter.
*   Description:       A (detailed) description of the parameter.
*
*
*   init         IN    Indicate for the function if it should start form the 
*                      first record. When the "init" parameter is TRUE the 
*                      function returns with SUCCESS but doesn't set the SLX 
*                      Dictionary Handle. Before starting to use the function 
*                      on a buffer this flag must always be set to TRUE.
*
*   gotRecord    OUT   Indicates for calling function if a record was found. 
*                      I.e. when there are no more records to return this flag
*                      becomes FALSE.
*
*   dicBuf       OUT   SLX Dynamic Buffer containing the Dictionary loaded in
*                      from the file.   
*
*   dicInfo      OUT   the next dictionary record.
*
*   error        OUT   CCS Error Structure.
*
*
*   FILES
*   ESO-VLT-DIC.<name> Dictionaries for the VLT. There are a number of 
*                      dictionaries defined, e.g. for the instruments
*                      ISAAC, FORS and UVES.
*                  
*   RETURN VALUES 
*   SUCCESS            
*   FAILURE            
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxGetDicRecSeq.c,v 1.145 2005/07/27 09:14:00 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Local Headers
 */
#include "slx.h"

#define _slxDIC_ERR(line, par) { \
errResetStack(&tmpError); \
errAdd(error, slxMODULE_NAME, slxERR_FORMAT_DIC2, __FILE__, NULL, line, par); \
return FAILURE;}

#define _slxUNIT_CONV_ERR { \
    sprintf(hlpBuf, "%s/Unit Conversion illegal", keywordOrg); \
    _slxDIC_ERR(lineCount, hlpBuf); \
}

/* The tables with the units accepted by SLX (from the DICB document)
 * are made global
 */
int           slxUnitTable[] = {slxUNIT_NONE,
				slxUNIT_LENGTH_M,
				slxUNIT_LENGTH_MM,
				slxUNIT_LENGTH_MUM,
				slxUNIT_LENGTH_NM,
				slxUNIT_MASS,
				slxUNIT_TIME,
				slxUNIT_PLANE_ANGLE,
				slxUNIT_SOLID_ANGLE,
				slxUNIT_TEMP,
				slxUNIT_ELEC_CURRENT,
				slxUNIT_FREQ,
				slxUNIT_ENERGY,
				slxUNIT_POWER,
				slxUNIT_ELEC_POT,
				slxUNIT_FORCE,
				slxUNIT_PRESSURE,
				slxUNIT_DEGS_OF_ARC,
				slxUNIT_MINS_OF_ARC,
				slxUNIT_SECS_OF_ARC,
				slxUNIT_MAGNITUDE,
				slxUNIT_FLUX_FREQ,
				slxUNIT_FLUX_WAVE,
				slxUNIT_MJD,
				-1};

char          *slxUnitIdTable[] = {slxUNIT_NONE_ID,
				   slxUNIT_LENGTH_M_ID,
				   slxUNIT_LENGTH_MM_ID,
				   slxUNIT_LENGTH_MUM_ID,
				   slxUNIT_LENGTH_NM_ID,
				   slxUNIT_MASS_ID,
				   slxUNIT_TIME_ID,
				   slxUNIT_PLANE_ANGLE_ID,
				   slxUNIT_SOLID_ANGLE_ID,
				   slxUNIT_TEMP_ID,
				   slxUNIT_ELEC_CURRENT_ID,
				   slxUNIT_FREQ_ID,
				   slxUNIT_ENERGY_ID,
				   slxUNIT_POWER_ID,
				   slxUNIT_ELEC_POT_ID,
				   slxUNIT_FORCE_ID,
				   slxUNIT_PRESSURE_ID,
				   slxUNIT_DEGS_OF_ARC_ID,
				   slxUNIT_MINS_OF_ARC_ID,
				   slxUNIT_SECS_OF_ARC_ID,
				   slxUNIT_MAGNITUDE_ID,
				   slxUNIT_FLUX_FREQ_ID,
				   slxUNIT_FLUX_WAVE_ID,
				   slxUNIT_MJD_ID,
				   NULL};

/* Function definition
 */
ccsCOMPL_STAT slxGetDicRecSeq(const vltLOGICAL   init,
			      vltLOGICAL         *gotRecord,
			      slxDYN_BUF         *dicBuf,
			      slxDIC_HANDLE      *dicInfo,
			      ccsERROR           *error)
{    
    static slxDYN_BUF    tmpBuf;
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL    initialized=ccsFALSE;
    static vltINT32      lineCount;
    static ccsERROR      tmpError;
    char                 *chrPtr;
    char                 inBuf[512];
    slxDIC_RECORD_ENTRY  keyword, keywordOrg;
    slxDIC_RECORD_ENTRY  hlpBuf;
    slxDIC_RECORD_ENTRY  hlpBuf2;
    slxDIC_RECORD_ENTRY  hlpBuf3;
    vltINT32             index;
    vltLOGICAL           run, firstDescrStore;
    
    
    memset(keyword, 0, sizeof(slxDIC_RECORD_ENTRY));
    memset(inBuf, 0, 512);
    if (!initialized)
	{
	errResetStack(&tmpError);
	if (slxInitDynBuf(&tmpBuf, error) == FAILURE)
	    return FAILURE;
	initialized = TRUE;
	}
    
    if (init)
	{
	if (slxGetNextLineBuf(dicBuf, slxINIT_DYN_BUF, inBuf, 
			      error) == FAILURE)
	    return FAILURE;
	*gotRecord = FALSE;
	lineCount = 0;
	return SUCCESS;
	}
    memset(dicInfo, 0, sizeof(slxDIC_HANDLE));
    firstDescrStore = TRUE;
    
    /* Find the next record or the end of the Dictionary
     */
    strcpy(inBuf, "*");
    while ((*inBuf != '\0') &&
	   ((strstr(inBuf, slxDICO_PAR_NAME_PREFIX) == NULL) ||
	    slxSkipLine(inBuf)))
	{
	lineCount++;
	if (slxGetNextLineBuf(dicBuf, slxDO_NOT_INIT, inBuf, 
			      &tmpError) == FAILURE)
	    _slxDIC_ERR(lineCount, "?");
	}
    
    if (*inBuf == '\0')
	{
	*gotRecord = FALSE;
	return SUCCESS;
	}
    
    /* Set the fields of the SLX Dictionary Handle
     */
    
    /* Get the keyword - put it in Short-FITS format
     */
    index = strlen(slxDICO_PAR_NAME_PREFIX);
    memset(keyword, 0, slxDIC_BUF_LEN);
    while ((*(inBuf + index) == ' ') && (*(inBuf + index) != '\0'))
	index++;
    strcpy(keyword, (inBuf + index));
    index = (strlen(keyword) - 1);
    while ((*(keyword + index) == ' ') || (*(keyword + index) == '\t') ||
	   (*(keyword + index) == '\n'))
	*(keyword + index--) = '\0';
    index = 0;
    strcpy(keywordOrg, keyword);
    while (*(keyword + index) != '\0')
	{
	if (*(keyword + index) == ' ')
	    *(keyword + index) = '.';
	index++;
	}
    strcpy(dicInfo->parName, keyword);
    
    /* Get the other features for the keyword.
     * Expects dictionary file in the format:
     *
     * Parameter Name: <name>
     * Class: <class>
     * Context: <context>
     * Type: <type>
     * Value Format: <ANSI C format>
     * Unit: <unit>
     * Comment Format: <comment>
     * Description: <description>
     */
    
    /* Class: <class>
     */
    do 
	{
	lineCount++;
	if (slxGetNextLineBuf(dicBuf, slxDO_NOT_INIT, inBuf, 
			      &tmpError) == FAILURE)
	    _slxDIC_ERR(lineCount, keywordOrg);
	}
    while (slxSkipLine(inBuf));
    memset(hlpBuf2, 0, slxDIC_BUF_LEN);
    if (sscanf(inBuf, "%s %s", hlpBuf, hlpBuf2) < 2)
	_slxDIC_ERR(lineCount, keywordOrg);
    
    strcpy(dicInfo->parClass, hlpBuf2);
    memset(dicInfo->parClassVec, 0, 
	   ((slxNO_KEYW_CLASSES + 1) * sizeof(vltLOGICAL)));
    
    if (strstr(hlpBuf2, slxSETUP_CLASS_STR) != NULL)
	dicInfo->parClassVec[slxSETUP_CLASS] = TRUE;
    if (strstr(hlpBuf2, slxMAINT_HEADER_CLASS_STR) != NULL)
	dicInfo->parClassVec[slxMAINT_HEADER_CLASS] = TRUE;
    if (!dicInfo->parClassVec[slxMAINT_HEADER_CLASS] &&
	(strstr(hlpBuf2, slxHEADER_CLASS_STR) != NULL))
	dicInfo->parClassVec[slxHEADER_CLASS] = TRUE;
    if (strstr(hlpBuf2, slxCONFIG_CLASS_STR) != NULL)
	dicInfo->parClassVec[slxCONFIG_CLASS] = TRUE;
    if (strstr(hlpBuf2, slxPRIVATE_CLASS_STR) != NULL)
	dicInfo->parClassVec[slxPRIVATE_CLASS] = TRUE;
    if (strstr(hlpBuf2, slxTEMPLATE_CLASS_STR) != NULL)
	dicInfo->parClassVec[slxTEMPLATE_CLASS] = TRUE;
    if (strstr(hlpBuf2, slxCONF_LOG_CLASS_STR) != NULL)
	dicInfo->parClassVec[slxCONF_LOG_CLASS] = TRUE;
    if (strstr(hlpBuf2, slxCOND_LOG_CLASS_STR) != NULL)
	dicInfo->parClassVec[slxCOND_LOG_CLASS] = TRUE;
    if (strstr(hlpBuf2, slxOPS_LOG_CLASS_STR) != NULL)
	dicInfo->parClassVec[slxOPS_LOG_CLASS] = TRUE;
    if (strstr(hlpBuf2, slxREDUC_LOG_CLASS_STR) != NULL)
	dicInfo->parClassVec[slxREDUC_LOG_CLASS] = TRUE;
    if (strstr(hlpBuf2, slxALL_CLASS_STR) != NULL)
	memset(dicInfo->parClassVec, 1, sizeof(slxKEYWORD_CLASS_VEC));
    
    /* Context: <context>
     */
    do
	{
	lineCount++;
	if (slxGetNextLineBuf(dicBuf, slxDO_NOT_INIT, inBuf, 
			      &tmpError) == FAILURE)
	    _slxDIC_ERR(lineCount, keywordOrg);
	}
    while (slxSkipLine(inBuf));
    memset(hlpBuf2, 0, slxDIC_BUF_LEN);
    if (sscanf(inBuf, "%s %s", hlpBuf, hlpBuf2) < 2)
	_slxDIC_ERR(lineCount, keywordOrg);
    strcpy(dicInfo->parContext, hlpBuf2);
    
    /* Type: <type>
     */
    do
	{
	lineCount++;
	if (slxGetNextLineBuf(dicBuf, slxDO_NOT_INIT, inBuf, 
			      &tmpError) == FAILURE)
	    _slxDIC_ERR(lineCount, keywordOrg);
	}
    while (slxSkipLine(inBuf));
    memset(hlpBuf2, 0, slxDIC_BUF_LEN);
    if (sscanf(inBuf, "%s %s", hlpBuf, hlpBuf2) < 2)
	_slxDIC_ERR(lineCount, keywordOrg);
    strcpy(dicInfo->dicParType, hlpBuf2);
    if (strstr(dicInfo->dicParType, slxSTRING_TYPE_STR) != NULL)
	dicInfo->dicParamType = slxSTRING;
    else if (strstr(dicInfo->dicParType, slxLOGICAL_TYPE_STR) != NULL)
	dicInfo->dicParamType = slxLOGICAL;
    else if (strstr(dicInfo->dicParType, slxINTEGER_TYPE_STR) != NULL)
	dicInfo->dicParamType = slxINTEGER;
    else if (strstr(dicInfo->dicParType, slxDOUBLE_TYPE_STR) != NULL)
	dicInfo->dicParamType = slxDOUBLE;
    else
	{
	dicInfo->dicParamType = slxUNKNOWN;
	_slxDIC_ERR(lineCount, keywordOrg);
	}
    
    /* Value Format: <ANSI C format>
     */ 
    do
	{
	lineCount++;
	if (slxGetNextLineBuf(dicBuf, slxDO_NOT_INIT, inBuf, 
			      &tmpError) == FAILURE)
	    _slxDIC_ERR(lineCount, keywordOrg);
	}
    while (slxSkipLine(inBuf));
    memset(hlpBuf2, 0, slxDIC_BUF_LEN);
    if (sscanf(inBuf, "%s %s %s", hlpBuf, hlpBuf3, hlpBuf2) < 3)
	_slxDIC_ERR(lineCount, keywordOrg);
    strcpy(dicInfo->valFormat, hlpBuf2);
    
    /* Unit: <unit>
     */ 
    do
	{
	lineCount++;
	if (slxGetNextLineBuf(dicBuf, slxDO_NOT_INIT, inBuf, 
			      &tmpError) == FAILURE)
	    _slxDIC_ERR(lineCount, keywordOrg);
	}
    while (slxSkipLine(inBuf));
    memset(hlpBuf2, 0, slxDIC_BUF_LEN);
    if (sscanf(inBuf, "%s %s", hlpBuf, hlpBuf2) == 2)
	{
	strcpy(dicInfo->parUnit, hlpBuf2);
	
	/* Check that the unit is allowed
	 * //VLTSW20020630 
	 */
	slxSetUnitId(dicInfo,error);
	}
    
    /* Comment Format: <comment>
     */ 
    do
	{
	lineCount++;
	if (slxGetNextLineBuf(dicBuf, slxDO_NOT_INIT, inBuf, error)==FAILURE)
	    return FAILURE; 
	}
    while (slxSkipLine(inBuf));
    if (((chrPtr = strchr(inBuf, ':')) != NULL) &&
	(sscanf(inBuf, "%s %s %s", hlpBuf, hlpBuf3, hlpBuf2) == 3))
	{            
	/* Skip blanks and read until '\n'
	 */
	chrPtr++;
	while ((*chrPtr == ' ') || (*chrPtr == '\t'))
	    chrPtr++;
	
	index = 0;
	while (*chrPtr != '\n')
	    {
	    *(dicInfo->comField + index) = *chrPtr;
	    chrPtr++;
	    index++;
	    }
	
	/* Now check if conversions are requested in the Comment Field. I.e.,
	 * one of the tags "%TIME", "%DAYTIM", "%DEGREE", ..., appears
	 * (VLTSW20020630 separate function) 
	 */
	if(slxParseUnitConv(dicInfo,error)!=SUCCESS)
	    {
	    return FAILURE;
	    }
     
	}    /* end-if (((chrPtr = strchr((char *)inBuf, ':')) ... */
    else
	*(dicInfo->comField) = '\0';
    
    /* Description: <description>
     * Read in line until:
     *         - "Parameter Name:" is encountered.
     *         - Comment is encountered ("#").
     *         - a blank line is returned (no more contents in the buffer.
     */
    if (slxSoftResetDynBuf(&tmpBuf, error) == FAILURE)
	return FAILURE;    
    run = TRUE;
    do 
	{ 
	lineCount++;
	if (slxGetNextLineBuf2(dicBuf, slxDO_NOT_INIT, &chrPtr,
			       &tmpError) == FAILURE)
	    _slxDIC_ERR(lineCount, keywordOrg);
	
	/* Jump the "Description:" keyword
	 */
	if (strstr(chrPtr, slxDICO_DESCR_PREFIX) != NULL)
	    {
	    index = strlen(slxDICO_DESCR_PREFIX);
	    if (strlen(chrPtr) > strlen(slxDICO_DESCR_PREFIX))
		{
		while (*(chrPtr + index) == ' ')
		    index++;
		}
	    }
	else
	    index = 0;
	if (strlen(chrPtr) > 0)
	    {
	    if ((*chrPtr == '\0') || (*chrPtr == '#'))
		run = FALSE;
	    if ((strstr(chrPtr, slxDICO_PAR_NAME_PREFIX) != NULL) ||
		(*chrPtr == '#'))
		{
		/* Rewind the Dynamic Buffer
		 */
		run = FALSE;
		if (slxRewindDynBuf(dicBuf, strlen(chrPtr), error) == FAILURE)
		    return FAILURE;
		}
	    if (run)
		{
		/* Skip initiating blanks and '\t's
		 */
		/*while (((*(chrPtr + index) == ' ') ||
			(*(chrPtr + index) == '\t')) &&
		       (*(chrPtr + index) != '\0'))
		    index++;*/
		/* Skip initiating '\n's the first time a line is stored
		 */
		if (firstDescrStore)
		    {
		    while ((*(chrPtr + index) == '\n') &&
			   (*(chrPtr + index) != '\0'))
			index++;
		    firstDescrStore = FALSE;
		    }		
		if (slxStoreBufDynBuf(&tmpBuf, (chrPtr + index), 
				      (strlen(chrPtr + index)), 
				      error) == FAILURE)
		    return FAILURE;
		}
	    }
	else 
	    run = FALSE;
	}
    while (run);
    
    if (slxDynBufAddEOS(&tmpBuf, error) == FAILURE)
	return FAILURE;
    if (strlen(slxDynBuf(&tmpBuf)) > 0)
	slxYankTrailNewLines(slxDynBuf(&tmpBuf));
    if (strlen(slxDynBuf(&tmpBuf)) > 0)
	{
	/* Make sure there are two and only two '\n's at the end of the 
	 * description
	 */
	if (slxStoreBufDynBuf(&tmpBuf, "\n\n", 2, error) == FAILURE)
	    return FAILURE;
	if (slxDynBufAddEOS(&tmpBuf, error) == FAILURE)
	    return FAILURE;
	/* A dirty trick
	 */
	dicInfo->description = tmpBuf.dynBuf;
	tmpBuf.dynBuf = NULL;
	if (slxInitDynBuf(&tmpBuf, error) == FAILURE)
	    return FAILURE;
	}
    else
	dicInfo->description = NULL;
    
    *gotRecord = TRUE;

    return SUCCESS;
}


/*
 * Part of the above function is copied here.... 
 *
 */
ccsCOMPL_STAT slxSetUnitId(slxDIC_HANDLE      *dicInfo,
			   ccsERROR           *error)
{ 
    vltLOGICAL           found;
    vltINT32             i;
    i = 1;
    found = FALSE;
    while (!found && (slxUnitIdTable[i] != NULL))
      {
	if (strcmp(dicInfo->parUnit, slxUnitIdTable[i]) == 0)
	  found = TRUE;
	else
	  i++;
      }

    /* For the moment we don't generate an error in case of an unknown
     * unit but we'll simply set unit to none
     */
    if (!found)
      {
	dicInfo->parUnitId = slxUNIT_NONE;
	/* IMPL.: Generate error for unknown units?
	 * sprintf(hlpBuf, "%s/Unit = \"%s\" illegal", keywordOrg, 
	 * dicInfo->parUnit);
	 * _slxDIC_ERR(lineCount, hlpBuf);
	 */
      }
    else
      dicInfo->parUnitId = slxUnitTable[i];

    return SUCCESS;
}

ccsCOMPL_STAT slxParseUnitConv(slxDIC_HANDLE      *dicInfo,
			       ccsERROR           *error)
{

    char                 *chrPtr, *tagPtr;
    vltINT32             i,tagLen;
    slxDIC_RECORD_ENTRY  hlpBuf;
    slxDIC_RECORD_ENTRY  keywordOrg;  
    static vltINT32      lineCount;
    static ccsERROR      tmpError;
   

    /* Now check if conversions are requested in the Comment Field. I.e.,
     * one of the tags "%TIME", "%DAYTIM", "%DEGREE", ..., appears
     */
    if ((dicInfo->parUnitId != slxUNIT_NONE) &&
	strchr(dicInfo->comField, '%') != NULL)
	{
	tagPtr = NULL;
	tagLen = 0;
	chrPtr = dicInfo->comField;
	switch (dicInfo->parUnitId)
	    {
	    case slxUNIT_TIME:
		if ((tagPtr = 
		     strstr(chrPtr, slxGLUE_2("%", slxCONV_TIME_ID))) != NULL)
		    {
		    dicInfo->conversion = slxCONV_SECS_TO_TIME;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_TIME_ID));
		    }
		else if ((tagPtr = 
			  strstr(chrPtr, slxGLUE_2("%", slxCONV_DAYTIM_ID))) != NULL)
		    {
		    dicInfo->conversion = slxCONV_SECS_TO_DAYTIM;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_DAYTIM_ID));
		    }
		else
		    _slxUNIT_CONV_ERR;
		break;
	    case slxUNIT_MJD:
		if ((tagPtr = 
		     strstr(chrPtr, slxGLUE_2("%", slxCONV_DAYTIM_ID))) != NULL)
		    {
		    dicInfo->conversion = slxCONV_MJD_TO_DAYTIM;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_DAYTIM_ID));
		    }
		else
		    _slxUNIT_CONV_ERR;
		break;
	    case slxUNIT_DEGS_OF_ARC:
		if ((tagPtr = 
		     strstr(chrPtr, slxGLUE_2("%", slxCONV_DEGREE_ID))) != NULL)
		    {
		    dicInfo->conversion = slxCONV_DEG_TO_DEGREE;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_DEGREE_ID));
		    }
		else if ((tagPtr = 
			  strstr(chrPtr, slxGLUE_2("%", slxCONV_HOURANG_ID))) != NULL)
		    {
		    dicInfo->conversion = slxCONV_DEG_TO_HOURANG;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_HOURANG_ID));
		    }
		else
		    _slxUNIT_CONV_ERR;
		break;
	    case slxUNIT_PLANE_ANGLE:
		if ((tagPtr = 
		     strstr(chrPtr, 
			    slxGLUE_2("%", slxCONV_DEGREE_ID))) != NULL)
		    {
		    dicInfo->conversion = slxCONV_RAD_TO_DEGREE;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_DEGREE_ID));
		    }
		else if ((tagPtr = 
			  strstr(chrPtr, 
				 slxGLUE_2("%", slxCONV_HOURANG_ID)))
			 !=NULL)
		    {
		    dicInfo->conversion = slxCONV_RAD_TO_HOURANG;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_HOURANG_ID));
		    }
		else
		    _slxUNIT_CONV_ERR;
		break;
	    case slxUNIT_MINS_OF_ARC:
		if ((tagPtr = 
		     strstr(chrPtr, 
			    slxGLUE_2("%", slxCONV_DEGREE_ID))) != NULL)
		    {
		    dicInfo->conversion = slxCONV_ARCM_TO_DEGREE;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_DEGREE_ID));
		    }
		else if ((tagPtr = 
			  strstr(chrPtr, 
				 slxGLUE_2("%", slxCONV_HOURANG_ID))) != NULL)
		    {
		    dicInfo->conversion = slxCONV_ARCM_TO_HOURANG;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_HOURANG_ID));
		    }
		else
		    _slxUNIT_CONV_ERR;
		break;
	    case slxUNIT_TEMP:
		if ((tagPtr = 
		     strstr(chrPtr, 
			    slxGLUE_2("%", slxCONV_CELSIUS_ID))) != NULL)
		    {
		    dicInfo->conversion = slxCONV_KELVIN_TO_CELSIUS;
		    tagLen = strlen(slxGLUE_2("%", slxCONV_CELSIUS_ID));
		    }
		else
		    _slxUNIT_CONV_ERR;
		break;
	    case slxUNIT_NONE:
		break;
	    default:
		/* Cannot happen
		 */
		break;
	    }
	/* Build up the comment field ready to have the variable field
	 * printed:
	 *
	 * "abcde \"%DAYTIM\" fdhijk" -> "abcde \"%s\" fdhijk"
	 */
	memset(hlpBuf, 0, sizeof(slxDIC_RECORD_ENTRY));
	i = (tagPtr - dicInfo->comField);
	slxCOPYN_MAX_N_BYTES(hlpBuf, dicInfo->comField, i,
			     sizeof(slxDIC_RECORD_ENTRY), error);
	slxSTRCAT_MAX_N_BYTES(hlpBuf, "%s", sizeof(slxDIC_RECORD_ENTRY),
			      error);
	slxSTRCAT_MAX_N_BYTES(hlpBuf, (tagPtr + tagLen), 
			      sizeof(slxDIC_RECORD_ENTRY), error);
	dicInfo->comFieldFormat = slxStrDup(hlpBuf);
	}
    return SUCCESS;
}
  

/*
 *****************************************************************************
 */

/*___oOo___*/
