/******************************************************************************
* E.S.O. - VLT project
*
* slxGetKeywordBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  22/01/94  Created
* jknudstr  31/05/95  Fixed bug when storing a new keyword in a buffer
*                     with a keyword being a subset of this  
* epozna    22/08/01  VLTSW20010283 manpage update
*                                   example added*/

/************************************************************************
*   NAME
*   slxGetKeywordBuf - searches for a keyword in a SLX Dynamic Buffer
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxGetKeywordBuf(slxDYN_BUF        *setupHandle,
*                                  const char        *keyword,
*                                  const vltUINT16   keywordNumber,
*                                  slxKEYW_INFO      *keywordInfo,
*                                  ccsERROR          *error)
*
* 
*   DESCRIPTION
*   The function  searches for a  keyword in the specified Dynamic Buffer.
*   The keyword can be identified by its name or its serial number.
*   If the keyword is found, the information about this keyword
*   is returned in the slxKEYW_INFO structure. 
*
*   When the name of the keyword- i.e. the argument 'keyword'- is given as 
*   a non-zero-length string, the function search for the kewyord with 
*   the given name. In this case the parameter 'keywordNumber' is ignored.
*   
*   When the name of the keyword - i.e. the argument 'keyword'- is given as  
*   zero-length string, the function will search for the keyword with the 
*   given serial number as indicated by argument 'keywordNumber'.
*   
*   Arguments of the function:
*   setupHandle     IN  The SLX Dynamic Buffer containing the setup parameters
*                       currently loaded in memory.
*
*   keyword         IN  The name of the keyword. Must be in the Short-FITS
*                       format.  If given as a zero-length string, the  
*                       search is carried out for the specified keyword 
*                       number.
*
*   keywordNumber   IN  The number of the keyword in the buffer. The setup 
*                       header is considered. The first keyword has number 1.
*
*   keywordInfo     OUT Structure that contains the various information about
*                       the keyword.
*                       This structure contains the following members:
*      
*                       vltUINT8  found     - Indicates if the keyword was 
*                                             found.
*                       vltUINT16 number    - Line number in set-up file;
*                       char   keyword[200] - The complete keyword 
*                                             (Short-FITS)
*                       char   category[4]  - The category of the keyword, 
*                                             or empty if not defined.
*                       char   subSystem[16]- The subsystem of the keyword, 
*                                             or empty if not defined.
*                       char   parameter[16]- The parameter of the keyword.
*                       char   value[200]   - The value as a string. A string
*                                              value keeps the quotes. 
*                       char   comment[200] - The possible comment of the 
*                                             keyword line.
* 
*   error           OUT CCS Error Structure that contains information about
*                       a possible error occurring.
*
*
*   RETURN VALUES 
*   SUCCESS         If the keyword was found and the information read 
*                   successfully.
*   FAILURE         When something goes wrong during the operation, the 
*                   following error codes may be returned by the function:
*
*                   slxERR_FORMAT_KEYWORD    The format of the keyword is
*                                            wrong.
*
*                   slxERR_KEYWORD_NOT_FOUND The specified keyword was not
*                                            found.
*
*                   The internal calls of functions slxGetNextLineBuf(), 
*                   slxSplitShortFITS() may generate error, in which case 
*                   these errors are returned.
*
*   EXAMPLES
*   The usage if slxGetKeywordBuf() is similar to slxGetKeyword().
*   See examples of the function slxGetKeyword in its manual.
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxGetKeywordBuf.c,v 1.145 2005/07/27 09:15:19 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxGetKeywordBuf(slxDYN_BUF       *setupHandle,
			       const char       *keyword,
			       const vltUINT16  keywordNumber,
			       slxKEYW_INFO     *keywordInfo,
			       ccsERROR         *error)
{
    char           *charPtr;
    char           *bufPtr;
    slxKEYW_LINE   wantedLine;
    slxKEYW_LINE   inBuf;
    slxKEYW_LINE   inKeyword;
    vltUINT8       found;
    vltUINT16      keywordCount;
    

    found = 0;
    keywordInfo->number = 0;
    if (strlen(keyword) == 0)
        {
        /* Search for a specific keyword number
         */
	if (slxGetNextLineBuf(setupHandle, slxINIT_DYN_BUF, inBuf,
			      error) == FAILURE)
	    return FAILURE;
        keywordCount = 0;	
	do
	    {
	    if (slxGetNextLineBuf(setupHandle, slxDO_NOT_INIT, inBuf,
				  error) == FAILURE)
		return FAILURE;
	    keywordCount++;
	    }
	while ((inBuf[0] != '\0') && (keywordCount < keywordNumber));

        if ((keywordNumber == keywordCount) && (keywordNumber > 0))
            {
	    strcpy((char *)wantedLine, (char *)inBuf);
            found = 1;
            if ((charPtr = strchr((char *)wantedLine, ' ')) != NULL)
                {
                strncpy((char *)keywordInfo->keyword, wantedLine, 
                        (size_t)(charPtr - wantedLine));
                *(keywordInfo->keyword + (charPtr - wantedLine)) = '\0';
                }
            }
        }
    else
        {
	if (slxGetNextLineBuf(setupHandle, slxINIT_DYN_BUF, inBuf,
			      error) == FAILURE)
	    return FAILURE;

	found = 0;
	do 
	    {
	    if (slxGetNextLineBuf(setupHandle, slxDO_NOT_INIT, inBuf,
				  error) == FAILURE)
		return FAILURE;

	    if (inBuf[0] != '\0')
		{
		(keywordInfo->number)++;
		bufPtr = inBuf;
		if (slxGetNextField(bufPtr, slxKEYW_LINE_LEN, inKeyword,
				    &bufPtr, error) == FAILURE)
		    return FAILURE;
		if (strcmp((char *)keyword, inKeyword) == 0)
		    {
		    found = 1;
		    strcpy((char *)wantedLine, (char *)inBuf);
		    }
		}    /* end-if (inBuf[0] != '\0') */
	    }
        while ((inBuf[0] != '\0') && !found);

        strcpy((char *)keywordInfo->keyword, keyword);
        }    /* end-if-else (strlen(keyword) == 0) */

    if (!found)
        {
        keywordInfo->found = 0;
        errAdd(error, slxMODULE_NAME, slxERR_KEYWORD_NOT_FOUND, __FILE__, 
               NULL, keyword, "Dictionary"); 
        return FAILURE;       
        }

    keywordInfo->found = 1;
    if (slxSplitShortFITS(wantedLine, keywordInfo->category,
                          keywordInfo->subSystem, keywordInfo->parameter,
                          keywordInfo->value, keywordInfo->comment,
                          error) == FAILURE)
         return FAILURE;
        
    return SUCCESS;
}

/*
 *****************************************************************************
 */


/*___oOo___*/
