/******************************************************************************
* E.S.O. - VLT project
*
* slxGetNextLineBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  22/01/94  created
* jknudstr  23/07/96  changed to use the dynBuf.count + changed "initDynBuf"
*                     parameter to type vltLOGICAL.
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized 
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxGetNextLineBuf - get next line from SLX Dynamic Buffer 
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxGetNextLineBuf(slxDYN_BUF           *setupHandle,
*                                   const slxDYN_BUF_OP  initDynBuf,
*                                   slxKEYW_LINE         keywordLine,
*   			            ccsERROR             *error)
* 
*    ccsCOMPL_STAT slxGetNextLineBuf2(slxDYN_BUF            *dynBuf,
*				      const slxDYN_BUF_OP   initDynBuf,
*				      char                  **keywordLine,
*				      ccsERROR              *error)
*
*   DESCRIPTION
*   The function returns the sequentially next setup keyword (including 
*   value and possible comment) from the "setupHandle" buffer.
*
*   Before starting out using the function, it should be called once with
*   the "initDynBuf" set to TRUE; no keyword is returned in this
*   case. 
*   
*   Note that the keyword lines in the Dynamic Buffer must be terminated with
*   CRT ('\n'). Only up to currently 128 bytes are copied to the "keywordLine"
*   buffer. In case the CRT is not found before it returns with an error.
*
*   When the last keyword has been read from the buffer, the length of the
*   the "keywordLine" buffer will be '\0'.
*
*
*   setupHandle  IN  SLX Dynamic Buffer containing setup keyword information.
*
*   initDynBuf   IN  Prepare the SLX Dynamic Buffer for the sequential
*                    request of lines.
*
*   keywordLine  OUT Buffer in which the next keywordline is copied to.
*                    When the last keyword has been read, this buffer is
*                    initiated with '\0'.
*
*   error        OUT CCS error structure indicating a possible error 
*                    occurring.
*
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed. The following error codes can be
*             be generated by the function:
*
*             slxERR_DYN_BUF_LINE_LEN  The length of the next line longer than
*                                      what fits into a buffer of type 
*                                      slxKEYW_LINE (use slxDynBufReadToCrt()).
*
*   CAUTIONS 
*   Remember to initialize the Dynamic Buffer before using the function the
*   1st time, when starting a new readout of keywords from the SLX Dynamic
*   Buffer.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxGetNextLineBuf.c,v 1.145 2005/07/27 09:17:09 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxGetNextLineBuf(slxDYN_BUF            *setupHandle,
				const  slxDYN_BUF_OP  initDynBuf,
				slxKEYW_LINE          keywordLine,
				ccsERROR              *error)
{
    char        *chrPtr1;
    vltINT32    lineLen;


    *keywordLine = '\0';

    if (initDynBuf)
	{
	setupHandle->count = 0;
	return SUCCESS;
	}

    if (setupHandle->count >= setupHandle->bytesStored)
	{
	*keywordLine = '\0';
	return SUCCESS;
	}

    /* Check if the last characters have a format like:
     *
     * "# --- oOo ---"
     *
     * i.e. without '\n' and we there simply copy out the last characters
     */
    if ((chrPtr1 = strchr((setupHandle->dynBuf + setupHandle->count), 
			  '\n')) == NULL)
	lineLen = (setupHandle->bytesStored - setupHandle->count);
    else
	lineLen = ((chrPtr1 - (setupHandle->dynBuf + setupHandle->count)) + 1);

    /* Check for overflow and copy over the characters
     */
    if (lineLen > (slxKEYW_LINE_LEN - 1))
	{
	errAdd(error, slxMODULE_NAME, slxERR_DYN_BUF_LINE_LEN, __FILE__, NULL);
	return FAILURE;
	}
    strncpy(keywordLine, (setupHandle->dynBuf + setupHandle->count), lineLen);
    *(keywordLine + lineLen) = '\0';
    setupHandle->count += lineLen;

    return SUCCESS;
}

ccsCOMPL_STAT slxGetNextLineBuf2(slxDYN_BUF            *dynBuf,
				 const slxDYN_BUF_OP   initDynBuf,
				 char                  **keywordLine,
				 ccsERROR              *error)
{
    static char        emptyBuf[1]; 
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL  initialized=ccsFALSE;
    static slxDYN_BUF  lineBuf;
    char               *chrPtr1;
    vltINT32           lineLen;

    if (!initialized)
	{
	if (slxInitDynBuf(&lineBuf, error) == FAILURE)
	    return FAILURE;
	*emptyBuf = '\0';
	initialized = TRUE;
	}
    *keywordLine = emptyBuf;

    if (initDynBuf)
	{
	dynBuf->count = 0;
	if (slxResetDynBuf(&lineBuf, error) == FAILURE)
	    return FAILURE;
	return SUCCESS;
	}
    if (slxSoftResetDynBuf(&lineBuf, error) == FAILURE)
	return FAILURE;

    if (dynBuf->count == dynBuf->bytesStored)
	{
	*keywordLine = emptyBuf;
	return SUCCESS;
	}

    /* Check if the last characters have a format like:
     *
     * "# --- oOo ---"
     *
     * i.e. without '\n' and we there simply copy out the last characters
     */
    if ((chrPtr1 = strchr((dynBuf->dynBuf + dynBuf->count),'\n')) == NULL)
	lineLen = (dynBuf->bytesStored - dynBuf->count);
    else
	lineLen = ((chrPtr1 - (dynBuf->dynBuf + dynBuf->count)) + 1);

    if (slxStoreBufDynBuf(&lineBuf, (dynBuf->dynBuf + dynBuf->count), 
			  lineLen, error) == FAILURE)
	return FAILURE;
    if (slxStoreBufDynBuf(&lineBuf, " ", 1, error) == FAILURE)
	return FAILURE;
    *(lineBuf.dynBuf + (lineBuf.bytesStored - 1)) = '\0';

    *keywordLine = lineBuf.dynBuf;
    dynBuf->count += lineLen;

    return SUCCESS;
}

/*
 *****************************************************************************
 */


/*___oOo___*/

