/******************************************************************************
* E.S.O. - VLT project
*
* slxLoadFile.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  02/03/95  created
* epozna    26/03/01  VLTSW20000108 ('\0' at the end of fileBuf->dynBuf)
* epozna    17/09/02  small update in slxLoadFile for LINUX, 
*                     and manpage update.
*/

/************************************************************************
*   NAME
*   slxLoadFile - loads a file into an SLX Dynamic Buffer
* 
*   SYNOPSIS
*   #include "slx.h"
*
*  ccsCOMPL_STAT slxLoadFile(slxFILENAME    fileName,
*                            const char     *envVar,
*	 		     slxDYN_BUF     *fileBuf,
*			     ccsERROR       *error)
*
*  ccsCOMPL_STAT slxLoadFitsHeaderBuf(slxFILENAME  fileName,
*			  	      slxDYN_BUF   *headerBuf,
*				      vltLOGICAL   *asciiFile,
*				      ccsERROR     *error)
* 
*   DESCRIPTION
*   slxLoadFile():
*   The function loads a file into an SLX Dynamic Buffer. The old contents
*   is automatically deleted.
*
*
*   fileName     IN/OUT   Name of the setup file to load (/merge) into the 
*                         buffer. The function will write the complete filename
*                         of the file loaded in the buffer at successfull 
*                         return, if not the complete filename was specified.
*                         Therefore it is important that that the filname is given 
*                         as type slxFILENAME (!)
*
*   envVar       IN       Colon separated list of paths in which the file
*                         will be searched.
*
*   fileBuf      IN/OUT   SLX Dynamic Buffer to handle a file in memory. The
*                         application should normally not themselves access
*                         the contents of the structure, but should use the
*                         access routines.
*
*   slxLoadFitsHeaderBuf():
*   Used to load a FITS File or a PAF file into an SLX Dynamic Buffer.
*   The flag asciiFile will indicate if the file was in ASCII format or in 
*   the binary FITS formar (~ no newlines).
*
*
*   FILES
*   Setup files must be in the Short-FITS format.
*
*   ENVIRONMENT
*   INS_SETUPPATH  Specifies a number of paths where set-up files are stored.
*                  If not set, uses the current directory.
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed. The following error codes can be
*             be generated by the function:
*
*             slxERR_OPEN_FILE    Cannot open setup file.
* 
*             slxERR_READ_FILE    Cannot read from setup file.
*
*             Furthermore errors generated by the functions
*             slxLocateSetupFile(), slxStoreLineBuf() can be returned.
*
*   SEE ALSO
*   slxLoadSetup()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxLoadFile.c,v 1.145 2005/07/27 09:17:47 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <errno.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxLoadFile(slxFILENAME   fileName,
			  const char    *envVar,
	 	          slxDYN_BUF    *fileBuf,
  		          ccsERROR      *error)
{
    slxFILENAME      completeName;
    FILE             *filePtr;
    struct stat      statBuf;




    if (slxResetDynBuf(fileBuf, error) == FAILURE)
	return FAILURE;

    /* If it is the complete path we do nothing
     */
    if (*fileName == '/')
      strcpy(completeName, fileName);
    else
      {	
	/* - otherwise we try to locate it using the specified environment
	 * variable
	 */
	if (slxLocateFile(envVar, fileName, completeName, error) == FAILURE)
	  return FAILURE;
      }

 
    if (stat(completeName, &statBuf) == -1)
      {
	errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, "%s",
		  modUNIX, completeName);
	return FAILURE;
      }

 
    /* EPO VLTSW20000108 : make sure the buffer ends with '\0' 
     *      note :slxDynBufAlloc allocates memory ,sets fileBuf->byteAlloc and
     *            (at first allocation) memsets the allocated buf to '\0'  
     */
    if (slxDynBufAlloc(fileBuf, (1+ statBuf.st_size) , error) == FAILURE)
      return FAILURE;

    fileBuf->count = 0;


    if ((filePtr = fopen(completeName, "r")) == NULL)
      {
	errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, "%s",
		  modUNIX, completeName);
	return FAILURE;
      }
    

    /* Read in lines from the file and store them in the Dynamic Buffer
     */ 
    if (fread(fileBuf->dynBuf, sizeof(char), (size_t)statBuf.st_size, 
	      filePtr) != statBuf.st_size)
      {
	fclose(filePtr);
	errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, "%s",
		  modUNIX, completeName);
	return FAILURE;
      }


    fileBuf->bytesStored = statBuf.st_size;

    fclose(filePtr);

    
    if (fileName==NULL) printf("NULL\n");
    /* additional check for LINUX
     * (prevents problem on linux when fileName is given directly 
     *  as string and fileName and completeName is the same.
     *  This case should be considered as bug in the implamentation, 
     *  but let slx be more flexible..)
     */
    if ((*fileName != '/') && (strcmp(fileName,completeName)!=0))
	{
	strncpy(fileName, completeName,(strlen(completeName)-1));
	}

    return SUCCESS;
}

ccsCOMPL_STAT slxLoadFitsHeaderBuf(slxFILENAME  fileName,
				   slxDYN_BUF   *headerBuf,
				   vltLOGICAL   *asciiFile,
				   ccsERROR     *error)
{
    char          tempBuf[slxFITS_HD_BLOCK_LEN + 1];
    char          *pathPtr;
    FILE          *filePtr;
    slxFILENAME   completeName;
    vltINT8       endFound;
    vltINT32      i;
    vltLOGICAL    isAsciiFile;


    if (slxResetDynBuf(headerBuf, error) == FAILURE)
	return FAILURE;

    /* Try first to open the file as it is if given with complete path
     */
    if (slxResolvePath(fileName, &pathPtr, error) == FAILURE)
	return FAILURE;
    slxCOPY_MAX_N_BYTES(completeName, pathPtr, sizeof(slxFILENAME), error);
    if (*completeName == '/')
	{
	if ((filePtr = fopen(completeName, "r")) == NULL)
	    {
	    errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, NULL,
		      modUNIX, completeName);
	    /* Try also to add extension ".fits" if not added
	     */
	    if (strstr(completeName, ".fits") != NULL)
		{
		strcat(completeName, ".fits");
		if ((filePtr = fopen(completeName, "r")) == NULL)
		    {
		    errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, 
			      __FILE__, NULL, modUNIX, completeName);
		    return FAILURE;
		    }
		else
		    errResetStack(error);
		}
	    else
		return FAILURE;
	    }
	else
	    errResetStack(error);
	}
    else
	{
	/* IMPL.: Use slxSearchFitsFile + global search
	 */
	if (slxBuildImageFileName(fileName, completeName, error) == FAILURE)
	    return FAILURE;
	if ((filePtr = fopen(completeName, "r")) == NULL)
	    {
	    errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, NULL,
		      modUNIX, completeName);
	    return FAILURE;
	    }
	}

    memset(tempBuf, 0, sizeof(tempBuf));
    fread(tempBuf, sizeof(char), (size_t)slxFITS_HD_BLOCK_LEN, filePtr);
    if (strchr(tempBuf, '\n') != NULL)
	isAsciiFile = TRUE;
    else
	{
	isAsciiFile = FALSE;
	fseek(filePtr, 0, SEEK_SET);
	}
    if (!isAsciiFile)
	{
	endFound = 0;
	do
	    {
	    if (fread(tempBuf, sizeof(char), (size_t)slxFITS_HD_BLOCK_LEN,
		      filePtr) != slxFITS_HD_BLOCK_LEN)
		{
		errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, 
			  NULL, modUNIX, completeName);
		goto errorCond;
		}
	    /* Check in steps of 80 characters if the "END" keyword is 
	     * encountered
	     */
	    for (i = 0; ((i < slxFITS_HD_BLOCK_LEN) && !endFound); 
		 (i += slxMAX_FITS_LI_LEN))
		{
		if (strncmp((const char *)(tempBuf + i), "END", 
			    (unsigned int)3) == 0)
		    endFound = 1;
		}
	    if (slxStoreBufDynBuf(headerBuf, tempBuf, slxFITS_HD_BLOCK_LEN,
				  error) == FAILURE)
		goto errorCond;
	    }
	while (!endFound);
	fclose(filePtr);
	}
    else
	{
	fclose(filePtr);
	if (slxLoadFile(completeName, "", headerBuf, error) == FAILURE)
	    goto errorCond;
	}

    *asciiFile = isAsciiFile;
    strcpy(fileName, completeName);

    return SUCCESS;

  errorCond:
    slxResetDynBuf(headerBuf, error);
    fclose(filePtr);
    return FAILURE;
}

/*
 *****************************************************************************
 */


/*___oOo___*/


