/******************************************************************************
* E.S.O. - VLT project
*
* slxLocateFile.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  10/05/95  Created
* gchiozzi  30/05/95  If INS_SETUPPATH is not defined, used "./" as default
* jknudstr  30/05/95  Completed handling of local path "./"
*/

/************************************************************************
*   NAME
*   slxLocateFile - searches for file according to directory path env. var.
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxLocateFile(const char         *envVar,
*                               const slxFILENAME  fileName,
*                               slxFILENAME        completeName,
*                               ccsERROR           *error)
*
* 
*   DESCRIPTION
*   The function searches for the specified filename in the path list given
*   by the environment variable "envName". The possible first occurrence of 
*   the file is returned!
*
*   If it does not find the set-up file, the function returns with FAILURE,
*   and the first character of the "completeName" will be a NULL character
*   ('\0'); i.e. the length (strlen()) will be 0.
*
*   It is up to the calling application itself to open/load the file.
*
*   envVar           IN   Colon separated list of paths in which there will
*                         be searched for the file.
*
*   setUpFileName    IN   The name of the set-up file to locate.
*
*   completeName     OUT  The complete name of the specified set-up file
*                         including the path. If the file was not found,
*                         the first character of the buffer will be a NULL
*                         character.
*
*   error            IN   Indicates a possible error occurring.
*
*
*   RETURN VALUES 
*   SUCCESS   If the operation was successfull.
*   FAILURE   If the operation failed. The following error code is defined:
*
*             slxERR_ENV_VAR        Could not get the value of the environment
*                                   variable specified by "envVar".
*
*             slxERR_LOCATE_FILE    Error occurred while locating set-up file.
*
*
*   CAUTIONS
*   It is assumed that filenames can consist of the characters a-z, A-Z,
*   0-9, "_", "-", "." and "#". Wildcards and tilde (~) representations are not
*   supported. The filenames in the specified environment variable the 
*   absolute names including the complete paths.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxLocateFile.c,v 1.145 2005/07/27 09:19:03 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifndef MAKE_LCU
    #include <sys/stat.h>
#else
    #include "vxWorks.h"
    #include "sys/stat.h"
#endif

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxLocateFile(const char         *envVar,
                            const slxFILENAME  fileName,
                            slxFILENAME        completeName,
                            ccsERROR           *error)
{
    char               *nextPathPtr;
    char               *prevPathPtr;
    char               *pathPtr;
    char               currentPath[256];
    char               *completePathPtr;
    struct stat        statBuf;
    vltUINT8           fileFound = 0;
    vltUINT8           morePaths = 1;
    vltUINT8           pathLen = 0;
    vltUINT16          pathVarLen;
    vltUINT16          pathIndex = 0;
    vltUINT16          count;


    *completeName = '\0';

    /* Check if the filename contains environment variables or "~/"
     */
    if (slxResolvePath(fileName, &completePathPtr, error) == FAILURE)
	return FAILURE;

    /* Test if the file can be opened as is
     */
    if (stat(completePathPtr, &statBuf) == 0)
	{
	slxCOPY_MAX_N_BYTES(completeName, completePathPtr, slxFILENAME_LEN,
			    error);
	return SUCCESS;
	}

    /* Test if the file can be opened as it is (starting with "/" or "./"
     */
    /*if ((*completePathPtr == '/') || 
      ((*completePathPtr == '.') && (*(completePathPtr + 1) == '/')))
      {
      if (stat(completePathPtr, &statBuf) == 0)
      {
      slxCOPY_MAX_N_BYTES(completeName, completePathPtr, slxFILENAME_LEN,
      error);
      return SUCCESS;
      }
      }*/

    if ((pathPtr = getenv(envVar)) == NULL)
        {
	errAdd(error, slxMODULE_NAME, slxERR_LOCATE_FILE_GEN, __FILE__, 
               "%s", fileName, envVar);
	return FAILURE;
        }

    nextPathPtr = pathPtr;
    prevPathPtr = pathPtr;
    pathVarLen = strlen(pathPtr);

    while ((!fileFound) && morePaths)
       {
       /* Generate path. We assume that the path list in "envVar" 
        * can have the following structure:
        * [:]/-----/-----[/]:/-----/-----/-----[/]: ... :/-----/-----[/][:]
        */
       if (((pathIndex + 1) == pathVarLen) &&
	   (strcmp(pathPtr, slxINS_PATH_DEFAULT) != 0))
           morePaths = 0;
       else
           {
           if (*prevPathPtr == ':')
               {
               prevPathPtr++;
               pathIndex++;
               }

           if ((nextPathPtr = strstr(prevPathPtr, ":")) == NULL)
               {
               count = 0;
               nextPathPtr = prevPathPtr;
               memset((char *)currentPath, 0, (size_t)256);
               while ((*nextPathPtr != ':') && (pathIndex < pathVarLen))
                   {
                   if (!isalpha((int)*nextPathPtr) &&
                       !isdigit((int)*nextPathPtr) &&
                       (*nextPathPtr != '_') && 
                       (*nextPathPtr != '-') &&
                       (*nextPathPtr != '#') &&
                       (*nextPathPtr != '/') &&
                       (*nextPathPtr != '.'))
 		        {

                        /* There is an illegal character in the string
                         */
                        errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR, 
                               __FILE__, "%s", envVar);
                        *completeName = '\0';
                        return FAILURE;
                        }

                   *(currentPath + count++) = *nextPathPtr;
                   pathIndex++;
                   nextPathPtr++;
                   }
               if (pathIndex == pathVarLen)
                   morePaths = 0;
               }
           else
               {
               pathLen = (nextPathPtr - prevPathPtr);
               pathIndex += pathLen;
               memset((char *)currentPath, 0, (size_t)256);
               strncpy((char *)currentPath, prevPathPtr, (size_t)pathLen);
               }

	   /* Check if the file exists
	    */
	   strcpy(completeName, currentPath);
	   if (*(currentPath + strlen(currentPath) - 1) != '/')
	       strcat(completeName, "/");
	   strcat(completeName, fileName);
	   if (stat(completeName, &statBuf) == 0)
	       fileFound = 1;               

           }
       prevPathPtr = nextPathPtr;
       }   /* while ((!fileFound) && morePaths) */

    if (!fileFound)
        {
        *completeName = '\0';
        errAdd(error, slxMODULE_NAME, slxERR_LOCATE_FILE_GEN, __FILE__, 
               "%s", fileName, envVar);
        return FAILURE;
        }

    return SUCCESS;
}


/*
 *****************************************************************************
 */



/*___oOo___*/



