/******************************************************************************
* E.S.O. - VLT project
*
* slxLocateSetupFile.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  18/09/94  Created
* gchiozzi  29/05/95  If INS_SETUPPATH is not defined, used "./" as default
* jknudstr  30/05/95  Completed handling of local path "./"
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized 
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxLocateSetupFile - locates setup files
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxLocateSetupFile(const slxFILENAME  setupFileName,
*                                    slxFILENAME        completeName,
*                                    ccsERROR           *error)
*
* 
*   DESCRIPTION
*   The function searches for the specified set-up file (or any file)
*   according to the environment variable INS_SETUPPATH. This is done in 
*   accordance with the search criterias described in 
*   INS Common SW/VLT-SPE-ESO-17240-0385.
*
*   If it does not find the set-up file, the function returns with FAILURE,
*   and the first character of the "completeName" will be a NULL character
*   ('\0'); i.e. the length (strlen()) will be 0.
*
*
*   setUpFileName    IN   The name of the set-up file to locate.
*
*   completeName     OUT  The complete name of the specified set-up file
*                         including the path. If the file was not found,
*                         the first character of the buffer will be a NULL
*                         character.
*
*   error            IN   Indicates a possible error occurring.
*
*
*   ENVIRONMENT
*   INS_ROOT        Defines the root directory of the INS directory structure
*                   used by this application.
*
*   INS_USER        Defines the current working point in the INS directory 
*                   structure. Could e.g. be "SYSTEM".
*
*   INS_SETUPPATH   Colon separated list of paths where Setup Files may be 
*                   located.
*
*
*   RETURN VALUES 
*   SUCCESS   If the operation was successfull.
*   FAILURE   If the operation failed. The following error code is defined:
*
*             slxERR_INS_SETUPPATH  Could not get the value of the environment
*                                   variable INS_SETUPPATH.
*
*             slxERR_LOCATE_FILE    Error occurred while locating set-up file.
*
*
*   CAUTIONS
*   It is assumed that filenames can consist of the characters a-z, A-Z,
*   0-9, "_", "-", "." and "#". Wildcards and tilde (~) representations are not
*   supported. The filenames in the INS_SETUPPATH should be the absolute names
*   including the complete paths.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxLocateSetupFile.c,v 1.145 2005/07/27 09:19:45 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifndef MAKE_LCU
    #include <sys/stat.h>
#else
    #include "vxWorks.h"
    #include "sys/stat.h"
#endif

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxLocateSetupFile(const slxFILENAME   setupFileName,
                                 slxFILENAME         completeName,
                                 ccsERROR            *error)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL      initialized=ccsFALSE;
    static slxSTRING_LIST  *fileList;
    vltINT32               numberFound;

    
    if (!initialized)
	{
	if (slxInitStrList(&fileList, "HEADER", error) == FAILURE)
	    return FAILURE;
	initialized = TRUE;
	}
    
    if (slxSearchSetupFile(setupFileName, &numberFound, fileList, TRUE, 
			   FALSE, FALSE, error) == FAILURE)
	return FAILURE;

    if (!numberFound)
        {
        *completeName = '\0';
        errAdd(error, slxMODULE_NAME, slxERR_LOCATE_FILE, __FILE__, 
               "%s", setupFileName);
        return FAILURE;
        }
    else
	{
	slxCOPY_MAX_N_BYTES(completeName, fileList->next->string,
			    slxFILENAME_LEN, error);
	}

    return SUCCESS;
}


/*
 *****************************************************************************
 */



/*___oOo___*/



