/******************************************************************************
* E.S.O. - VLT project
*
* slxLookUpKeywordBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  10/10/94  created
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized 
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxLookUpKeywordBuf - find information about a keyword in dictionary
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxLookUpKeywordBuf(slxDYN_BUF        *dicBuf,
*                                     const char        *keywordName,
*                                     const vltUINT16   recordNumber,
*                                     slxDIC_HANDLE     *dicInfo,
*                                     ccsERROR          *error)
*                                  
*
*   DESCRIPTION
*   The function searches in the specified VLT dictionary after the keyword
*   specified.
*
*   If the record for the parameter is found, the values from the dictionary
*   are stored in the SLX Dictionary Handle Structure. The context of this
*   structure is:
*
*   Parameter Name:    Name of the parameter, e.g. "DET DATE"
*   Class:             Class of the parameter.
*   Context:           The context in which it is used.
*   Type:              The type of the parameter. Possible types are "string"
*                      "logical", "integer" and "double".
*   Value Format:      Format in which the parameter should be represented.
*                      The format representation from ANSI-C is used (see
*                      e.g. printf()).
*   Unit:              The SI or derived SI unit for the parameter.
*   Comment Format:    The standard comment (if any) for the parameter.
*   Description:       A (detailed) description of the parameter.
*
*
*   dicBuf        IN   Pointer to SLX Dynamic Buffer containing a dictionary
*                      loaded into memory. The dictionary should be loaded
*                      with slxLoadFile().
*
*   keywordName   IN   Name of the parameter to search for. The parameter
*                      must be given in in the Short-FITS format. Must
*                      be null terminated.
*
*   recordNumber  IN   Used to request a record according to its position in
*                      the dictionary. If this option is used the parameter
*                      "keywordName" must be set to "". The first record 
*                      starts at 1. If this parameter is not used it is 
*                      ignored. If a record number is specified larger 
*                      than the actual number of records in the dictionary
*                      the Dictionary Handle structure will be empty.
*
*   dicInfo     OUT    Structure that contains the information about the
*                      parameter in case it was found.
*
*                      The buffer to contain the possible description will 
*                      be allocated by the function and will be overwritten
*                      at the next call to the function.
*
*
*   error     OUT      CCS Error Structure. Indicates the type of a possible
*                      error occurring, e.g. the specified dictionary could
*                      not be found.
*
*
*   FILES
*   ESO-VLT-DIC.<name> Dictionaries for the VLT. There are a number of 
*                      dictionaries defined, e.g. for the instruments
*                      ISAAC, FORS and UVES.
*                  
*   RETURN VALUES 
*   SUCCESS            If the operation was successful.
*   FAILURE            If the operation failed. The following error codes
*                      can be returned:
*
*                      slxERR_KEYWORD_NOT_FOUND The keyword (or keyword with
*                                               the number specified) was not
*                                               found.
*
*                      slxERR_SYN_FORMAT_DICO   Error in format of dictionary.
*
*                      slxERR_SYN_HEADER_DICO   Header of dictionary not OK.
*                                               Dictionary may be inconsistent.
*
*                      slxERR_ALLOC_MEM         Error occurred while allocating
*                                               memory (for the description).
*
*                      The function makes a call to slxLocateSetupFile() and
*                      may return errors generated by this function.
*
*   CAUTIONS
*   The function allocates space for the possible description. It is up to the
*   calling application to free this buffer (using slxFreeDicRecord()).
*
*   If the keyword searched for is not found, FAILURE is returned and
*   the error type slxERR_KEYWORD_NOT_FOUND is returned in the error 
*   structure.
*
*   SEE ALSO
*   slxResetDicRecord()
*
*   BUGS
*   If the description of a dictionary record consists of several '\n' 
*   terminated lines, only the first line is loaded, the rest are ignored.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxLookUpKeywordBuf.c,v 1.145 2005/07/27 09:21:07 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxLookUpKeywordBuf(slxDYN_BUF        *dicBuf,
                                  const char        *keywordName,
                                  const vltUINT16   recordNumber,
                                  slxDIC_HANDLE     *dicInfo,
                                  ccsERROR          *error)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL       initialized=ccsFALSE;
    static miscHASH_TABLE   dicHashTable;
    static slxDIC_HANDLE    dicList;
    slxDIC_HANDLE           *dicPtr;
    slxKEYW_LINE            tmpBuf;
    vltINT32                i;

    if (!initialized)
	{
	miscHashInit(dicHashTable);
	initialized = TRUE;
	}
    miscHashDeleteTable(dicHashTable);
    memset(&dicList, 0, sizeof(slxDIC_HANDLE));

    if (slxInstallDicBuf(dicBuf, dicHashTable, &dicList, error) == FAILURE)
	return FAILURE;
    
    if (*keywordName != '\0')
	{
	if (slxLookUpKeywordHashX(dicHashTable, keywordName, &dicInfo, 
				  error) == FAILURE)
	    return FAILURE;
	return SUCCESS;
	}

    /* Look for a certain keyword number
     */
    memset(dicInfo, 0, sizeof(slxDIC_HANDLE));
    dicPtr = dicList.next;
    for (i = 0; (i < recordNumber) && (dicPtr != &dicList); i++)
	dicPtr = dicPtr->next;
    
    if (i == recordNumber)
	{
	sprintf(tmpBuf, "number \"%d\"", recordNumber);
	errAdd(error, slxMODULE_NAME, slxERR_KEYWORD_NOT_FOUND, __FILE__,NULL, 
	       tmpBuf, "Dictionary");
	return FAILURE;
	}

    memcpy(dicInfo, dicPtr, sizeof(slxDIC_HANDLE));
    if (dicPtr->description != NULL)
	{
	if ((dicInfo->description = slxStrDup(dicPtr->description)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, NULL); 
	    return FAILURE;
	    }
	}
        
    return SUCCESS;
}


/*
 *****************************************************************************
 */

/*___oOo___*/
