/******************************************************************************
* E.S.O. - VLT project
*
* slxMergeSetupFiles.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  28/07/94  Created 1. version
* jknudtsr  16/10/95  Introduced new SLX functions tp make it more efficient
*/

/************************************************************************
*   NAME
*   slxMergeSetupFiles - merges a number of setup files into one
* 
*   SYNOPSIS
*   slxMergeSetupFiles <output filename.<type>> <setup ID> <dictionary> \
*                      <setup 1> <setup 2> ... <setup n> [-t]
*
* 
*   DESCRIPTION
*   The subroutine merges the setup files which names are given as input 
*   parameters. If a file with the name of the output setup file specified
*   already exists, the program will return with an error.
*
*
*   <output filename.<type>>  Name of the new setup file. The file must be
*                             unique. I.e. if it exists the program returns
*                             with an error messages. The name must be the 
*                             complete filename (inclusive the path).
*                             The <type> can be one of the following values:
*
*                             ".ref", ".ins", ".det" and ".targ"
* 
*   <setup ID>                The setup ID must describe the setup shortly.
*
*   <setup 1>                 Name of the setup files to merge.
*   ...
*   <setup n>
*
*   -t                        Verbose switch; if specified, will send CCS error
*                             messages also to stderr.
*
*   FILES
*   setupHdTemplate    The file contains a template for the obligatory header
*                      of setup files.
*
*   ENVIRONMENT 
*   INS_SETUPPATH   Contains a colon separated list of paths where the 
*                   setup files can be located.
*
*   RETURN VALUES
*   0   The operation was successfull.
*   1   The operation failed.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxMergeSetupFiles.c,v 1.145 2005/07/27 09:23:58 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Local Headers
 */
#include "slx.h"

/* Main
 */
int main (int argc, char *argv[])
{
    int               i;
    ccsERROR          error;
    ccsPROCNAME       procName;
    slxFILENAME       completeName;
    slxDYN_BUF        setupBuf;
    slxSETUP_TYPE     type;
    slxCHANGE_RECORD  changeRec;
    slxDICTIONARY     dictionary;
    slxFILENAME       fileName;
    vltLOGICAL        verbose = FALSE;


    if (strcmp(argv[argc-1], "-t") == 0)
        {
	verbose = TRUE;
	argc--;			/* only count "real" arguments */
        }    
    
    if (argc < 6)
        {
	fprintf(stderr,"\nCorrect syntax is:\n"
		"\nslxMergeSetupFiles <new setup.<type>> <setup ID> "
		"<dictionary>\\"
		"\n                   <setup file 1> <setup file 2>\\"
		"\n                   ... <setup file n> [-t]\n\n");
	exit (1);
        }

    slxInitDynBuf(&setupBuf, &error);

    if ((strstr(argv[1], slxREF_EXTENSION) == NULL) &&
        (strstr(argv[1], slxINS_EXTENSION) == NULL) &&
        (strstr(argv[1], slxDET_EXTENSION) == NULL) &&
        (strstr(argv[1], slxTARG_EXTENSION) == NULL))
        {
	fprintf(stderr,"\nError in output setup filename; extension must be:\n"
		"\n%s, %s, %s or %s\n\n", slxREF_EXTENSION, slxINS_EXTENSION,
                slxDET_EXTENSION, slxTARG_EXTENSION);
        exit (1);
        }

    /* Check that complete path is given for the new setup file 
     * (assume a '/' as first character)
     */
    if (*argv[1] != '/')
        {
        fprintf(stderr,
	       "\nError, name of new setup file must be complete name!!\n\n");
        exit(1);
        }

    strcpy(procName, "slxMergeSetups");
    if (ccsInit(procName, 0, NULL, NULL, &error) == FAILURE)
	{
	fprintf (stderr,"ccsInit failed.\n");
	exit(1);
	}

    /* Create the header of the setup file - first check if it already exists.
     */
    if (slxLocateSetupFile(argv[1], completeName, &error) == SUCCESS)
        {
        fprintf(stderr, "\nError: \"%s\" exists already!\n\n",
               completeName);
        ccsExit(&error);
        exit(1);
        }

    /* Determine the Setup Type
     */
    if (strstr(argv[1], slxREF_EXTENSION) != NULL)
        type = slxREFERENCE_SETUP;
    else if (strstr(argv[1], slxINS_EXTENSION) != NULL)
        type = slxDETECTOR_SETUP;
    else if (strstr(argv[1], slxDET_EXTENSION) != NULL)
        type = slxINSTRUMENT_SETUP;
    else if (strstr(argv[1], slxTARG_EXTENSION) != NULL)
        type = slxTARGET_SETUP;
    else
        {
        fprintf(stderr, "\nError in extension of new setup file!\n");
        ccsExit(&error);
        exit(1);
        }
    strcpy(dictionary, argv[3]);
    if (slxCreateSetupFileBuf(&setupBuf, dictionary, type, argv[2], "", "",
			      "", "", "", &error) == FAILURE)
	goto errorCond;

    /* Merge the setup files into the Setup Buffer.
     * Unfortunate that we have to load the Dictionary again.
     */
    for (i = 4; i < argc; i++)
	{
	strcpy(fileName, argv[i]);
	if (slxMergeSetupBuf(&setupBuf, dictionary, changeRec, &error, "-f",
			     fileName, NULL) == FAILURE)
	    goto errorCond;
	}

    /* Store the new Setup File on disk
     */
    strcpy(fileName, argv[1]);
    if (slxSaveFile(fileName, "", &setupBuf, &error) == FAILURE)
	goto errorCond;


    slxResetDynBuf(&setupBuf, &error);
    ccsExit(&error);
    exit(0);

  errorCond:
    slxPrintError(&error);
    slxResetDynBuf(&setupBuf, &error);
    ccsExit(&error);
    exit(1);
}


/*
 *****************************************************************************
 */


/*___oOo___*/






