/******************************************************************************
* E.S.O. - VLT project
*
* slxNormalToShortFITS.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  27/07/94  Created 1. version
* epozna    31/08/00  VLTSW20000298 ( COMMENT problem )
*/

/************************************************************************
*   NAME
*   slxNormalToShortFITS - converts from normal FITS to short FITS
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxNormalToShortFITS(const slxKEYW_LINE  normalFITS,
*				       slxKEYW_LINE        shortFITS,
*                                      ccsERROR            *error)
*
*
*   DESCRIPTION
*   The function converts a set-up keyword from the normal FITS format to the
*   short FITS format.
*
*   If the Short-FITS line generated is longer than 80 characters it is 
*   truncated to this value.
*
* 
*   normalFITS	IN	Pointer to buffer containing the keyword string
*			in the normal FITS format.
*			
*   shortFITS	IN	Pointer to buffer in which the keyword string in 
*    			Short-FITS keyword will be written. Must be provided
*                       by the calling application. Make this buffer somewhat  
*                       larger say 100 character or so.
*
*   error       OUT     Indicates the type of an error possibly occurring.
*
*
*   RETURN VALUES 
*   SUCCESS     In case the operation was successful.
*
*   FAILURE     In case the operation went wrong. Could be a problem with the
*               format of the keyword line. The following error code can be
*               returned:
*
*               slxERR_FORMAT_KEYWORD  The format of the keyword is wrong.
*
*
*   CAUTION
*   The function cannot handle operations logs (FITS logs).
*
*
*   SEE ALSO
*   slxShortToNormalFITS(), slxDetectFormat(), slxConvertToShorts()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxNormalToShortFITS.c,v 1.145 2005/07/27 09:26:05 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxNormalToShortFITS(const slxKEYW_LINE  normalFITS,
                                   slxKEYW_LINE        shortFITS,
                                   ccsERROR            *error)
{
    const char  *strPtr;
    char        *strPtr2;
    vltUINT8    inAString = 0;
    vltUINT8    semiColonPlaced = 0;
    vltUINT8    wasLogical = 0;
    vltUINT16   index = 0;
    vltUINT16   index2 = 0;
    vltUINT16   lineLen;
    vltUINT16   count = 0; 


    lineLen = (vltUINT16)strlen(normalFITS);
    if (lineLen > sizeof(slxKEYW_LINE))
        lineLen = sizeof(slxKEYW_LINE);
    memset(shortFITS, 0, sizeof(slxKEYW_LINE));

    /* 1. Check if just one comment line
     * ("COMMENT / <comment>\0")
     *
     * 2. Is it a standard FITS or a hierarchical FITS keyword?   
     */

 

    if (strstr(normalFITS, "COMMENT ") == normalFITS)
        {
        strPtr = strchr(normalFITS, ' ');
        while ((*strPtr == ' ') || (*strPtr == '/'))
            strPtr++;
        strcpy(shortFITS, "# ");
        strcat(shortFITS, strPtr);
        
        /* If the Normal FITS line contains a '\n' and the 
         * length of the Short-FITS line is longer than slxKEYW_LINE
         * -> write a '\n' at the sizeof(slxKEYW_LINE) character of 
	 * the Short-FITS line.
         */
        /*if ((strchr(normalFITS, '\n') != NULL) && 
	  (strlen(shortFITS) > sizeof(slxKEYW_LINE)))
	  *(shortFITS + sizeof(slxKEYW_LINE) - 1) = '\n';
	  
	  *(shortFITS + sizeof(slxKEYW_LINE) - 1) = '\0';
	  */

        return SUCCESS;
        }
    else if (strstr(normalFITS, slxFITS_ESO_PREFIX) == NULL)
        {
        /* It's a standard FITS keyword: "<par>  = <value>  [/ <comment>]"
         * Get the keyword, and make the chrPtr point to the equal sign.
         */
	strPtr = normalFITS;
        count = 0;
        while ((*strPtr != '=') && (count++ < lineLen))
            {
            *(shortFITS + index++) = *strPtr;
            strPtr++;
            }

        if (count == lineLen)
            {
            errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, __FILE__, 
                   "%s", normalFITS);
            return FAILURE;
            }
        } 
    else
	{
        /* It's a hierarchical FITS keyword: 
         * "HIERARCH ESO <cat> [<sub>] <par> = <value> [/ <comment>]"
         * Skip the "HIERARCH ESO" and work through the buffer until
         * the equal sign.
         */

        strPtr2 = strchr(normalFITS, ' ');
        strPtr2++;
        strPtr = strchr(strPtr2, ' ');
        strPtr++;

        count = 0;
        while ((*strPtr != '=') && (count++ < lineLen))
            {
            *(shortFITS + index) = *strPtr;
            strPtr++;
            if (*(shortFITS + index) == ' ')
                *(shortFITS + index) = '.';
            index++;
            }

        if (count == lineLen)
            {
            errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, __FILE__, 
                   NULL, normalFITS);
            return FAILURE;
            }

        index2 = (index - 1);
        while (*(shortFITS + index2) == '.')
            *(shortFITS + index2--) = ' ';
        }

    /* Make the string pointer point to the 1. character of the value, and
     * copy at the same time some blanks over.
     */ 
    strPtr++;
    count = 0;
    while ((*strPtr == ' ') && (count++ < lineLen))
       {
       *(shortFITS + index++) = *strPtr;
       strPtr++;
       }
    if (count == lineLen)
        {
        errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, __FILE__, NULL,
               normalFITS);
        return FAILURE;
        }

    /* Convert the value. A string '<string>' -> "<string>", otherwise
     * straightforward.  Remember the ";" after the value.
     * A logical (T or F): T -> "T".
     */
    while (((index < lineLen) && ((*strPtr != '/') && !inAString)) ||
           ((index < lineLen) && inAString))
        {
        if (*strPtr == '\'')
            {
            if (!inAString)
                inAString = 1;
            else
                inAString = 0;
	    /* Make sure we don't 'glue' the quote directly onto the
	     * parameter name
	     */
	    if (inAString && (*(shortFITS + (index - 1)) != ' '))
		*(shortFITS + index++) = ' ';
            *(shortFITS + index++) = '"';
            }
        else if ((*strPtr == ' ') && !inAString && !semiColonPlaced)
	    {
            semiColonPlaced = 1;
            *(shortFITS + index++) = ';';
            }
        else if (((*strPtr == 'T') || (*strPtr == 'F')) && 
                 ((*(strPtr + 1) == ' ') || (*(strPtr + 1) == '/')) &&
                 !wasLogical && !inAString)
            {
            wasLogical = 1;
            /*index -= 2;
	    if (*(shortFITS + (index - 1)) != ' ')
	    *(shortFITS + index++) = ' ';*/
            *(shortFITS + index++) = '\"';     /* " */
            *(shortFITS + index++) = *strPtr;
            *(shortFITS + index++) = '\"';     /* " */ 
            }
        else
            *(shortFITS + index++) = *strPtr;

        strPtr++;
        }
    
    /* Convert a possible comment
     */
    if (*strPtr == '/')
        {
        *(shortFITS + index++) = ' ';
        *(shortFITS + index++) = '#';
        strPtr++;
        while ((index < lineLen) && (*strPtr != '\0'))
            {
            *(shortFITS + index++) = *strPtr;
            strPtr++;
            }
        }

    *(shortFITS + index) = '\0';

    /* If the Normal FITS line contains a '\n' and the 
     * length of the Short-FITS line is longer than sizeof(slxKEYW_LINE)
     * -> write a '\n' at the sizeof(slxKEYW_LINE)'th character of the 
     * Short-FITS line.
     */
    /*if ((strchr(normalFITS, '\n') != NULL) && 
      (strlen(shortFITS) > sizeof(slxKEYW_LINE)))
      *(shortFITS + sizeof(slxKEYW_LINE) - 1) = '\n';
      
      *(shortFITS + slxMAX_FITS_LI_LEN) = '\0';*/

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/
