/******************************************************************************
* E.S.O. - VLT project
*
* slxPrepImageFile.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  02/09/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxPrepImageFile - prepare an image file with a number of FITS blocks
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxPrepImageFile(const slxFILENAME  fileName, 
*                                  const int          noOfBlocks,
*                                  const vltLOGICAL   writeEND,
*                                  ccsERROR           *error)
*
* 
*   DESCRIPTION
*   The function creates a file with the given filename, and pads with blanks
*   to prepare a header. There is padded with blanks corresponding to the 
*   number of FITS blocks specified. After the operation the file is closed 
*   again.
*
* 
*   fileName    IN   Name of the file to be created - must be full path!
*
*   noOfBlocks  IN   Number of FITS blocks to reserve space for in the file.
*
*   writeEND    IN   If this value is TRUE there will be written an END
*                    keyword in the end of the header.
*
*   error       OUT  Indicates a possible error occurring.
*
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed. The following errors can be 
*             generated by the function:
*
*             slxERR_CREATE_FILE   Could not create the specified file.
*
*             slxERR_WRITE_FILE    Could not write in the specified file.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxPrepImageFile.c,v 1.145 2005/07/27 09:26:42 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxPrepImageFile(const slxFILENAME  fileName, 
                               const int          noOfBlocks,
                               const vltLOGICAL   writeEND,
                               ccsERROR           *error)
{
    char      fitsBlock[slxFITS_HD_BLOCK_LEN];
    FILE      *filePtr;
    int       n;  
    long int  endOffset;
    

    memset((char *)fitsBlock, ' ', (size_t)slxFITS_HD_BLOCK_LEN);

    if ((filePtr = fopen((char *)fileName, "w")) == NULL)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_CREATE_FILE, __FILE__, NULL,
                  modUNIX, fileName);
        return FAILURE;
        }

    for (n = 0; n < noOfBlocks; n++)
        {
        if (fwrite((char *)fitsBlock, sizeof(char),
                   (size_t)slxFITS_HD_BLOCK_LEN, 
		   filePtr) < slxFITS_HD_BLOCK_LEN)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, NULL,
                      modUNIX, fileName);
            return FAILURE;
            }
        }

    /* If it is requested to write the END keyword - just do it!
     */
    if (writeEND == TRUE)
        {
        if (fseek(filePtr, (long int)0, SEEK_END) == -1)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, NULL,
                      modUNIX, fileName);
            return FAILURE;
            }
        endOffset = ftell(filePtr);
        endOffset -= slxMAX_FITS_LI_LEN;
        if (fseek(filePtr, endOffset, SEEK_SET) == -1)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, NULL,
                      modUNIX, fileName);
            return FAILURE;
            }
        if (fwrite((char *)"END", (size_t)3, (size_t)1, filePtr) == -1)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                      NULL, modUNIX, fileName);
            return FAILURE;
            }
        }    /* end-if (writeEND == TRUE) */


    fclose(filePtr);

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/
