/******************************************************************************
* E.S.O. - VLT project
*
* slxSearchUtils.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  23/10/95  Created
* epozna    22/01/01  VLTSW20000555 slxGetDirInfo2 
*                     (added 'isDirErrReport' argument to report/not report 
*                     error when arg1 is not a directory)
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized
*                     is initialized to ccsFALSE, VLTSW20040034
*                     Removal of ghost code in slxSearchUserFile.
* - - - - - - - - - - APR2005 - - - - - - - - - - - - - - - - - - - 
* mnylund   12/07/04  VLTSW20040202, Coredumps using the function 
*                     slxSearchConfigFile. Not alloved to add NULL to a
*                     string before calling the function stat. Results in
*                     a coredump for both SUNOS_5.8 and Linux_Redhat9.0_i686
* - - - - - - - - - - JAN2006 - - - - - - - - - - - - - - - - - - - - - - - 
* mnylund   22/07/05  VLTSW20050173 Support for slx under LCU
*
*/

/************************************************************************
*   NAME
*   slxSearchUtils, slxSearchSetup, slxGetDirInfo - functions to perform a 
*   search for various files
* 
*   SYNOPSIS
*   #include "slx.h"
*  
*   ccsCOMPL_STAT slxSearchSetupFile(const slxFILENAME    baseName,
*				     vltINT32             *numberFound,
*				     slxSTRING_LIST       *fileList,
*				     const vltLOGICAL     globalSearch,
*                                    const vltLOGICAL     stopFirstOccur,
*				     const vltLOGICAL     skipInsUserDir,
*				     ccsERROR             *error)
* 
*   ccsCOMPL_STAT slxSearchDictionary(const slxDICTIONARY  dictionary,
* 			              slxFILENAME          pafDicFullName,
* 			              ccsERROR             *error)
*
*   ccsCOMPL_STAT slxSearchTemplateFile(const slxFILENAME       baseName,
*				        const slxTEMPLATE_FILES fileType,
*      				        vltINT32                *numberFound,
*   				        slxSTRING_LIST          *fileList,
*				        const vltLOGICAL        globalSearch,
*				        const vltLOGICAL        stopFirstOccur,
*				        const vltLOGICAL        skipInsUserDir,
*				        const char              *pathEnvVar,
*				        ccsERROR                *error)
*
*   ccsCOMPL_STAT slxGetDirInfo(slxFILENAME     dirName,
*   			        slxSTRING_LIST  *entryList,
*			        vltINT32        *noOfEntries,
*			        ccsERROR        *error)
*
*   DESCRIPTION
*   Common parameters for search routines:
*
*   baseName        IN  The basename is the name of the file to look for.
*                       The name may contain wildcards ("*"), e.g.:
*                       "myFile*". In this case no assumption is made about the
*                       extension. The basename can be given in as
*
*                           o A single filename, e.g. "myFile.det".
*                   
*                           o Including the complete path, e.g.
*                             "/home/users/user/INS_ROOT/SYSTEM/COMMON/\
*                               SETUPFILES/DET/myFile*"
*
*                           o Starting with tilde for user's home directory,
*                             e.g. "~/INS_ROOT/SYSTEM/COMMON/\
*                                   SETUPFILES/DET/myFile*"
*
*                           o Starting with an environment variable, e.g.
*                             "$MY_VAR/DIR/myFile.det".
* 
*                       In the latter three cases, a search will only be
*                       performed in one directory, even if "globalSearch" is
*                       TRUE.
*
*   numberFound     OUT Indicates the number of files found, matching the
*                       specified conditions.
*
*   fileList        IN  Pointer to slxSTRING_LIST element. The instance must
*                       be created and initialized by the calling application.
*
*   globalSearch    IN  Indicates if a global search in the INS Tree should be
*                       performed.
*
*   stopFirstOccur  IN  If true, the file search will stop at the first
*                       file found, matching the specified conditions.
*
*   skipInsUserDir  IN  Indicates if the search should skip the branch 
*                       currently referenced to by the INS_USER environment
*                       variable.
*
*   slxSearchSetupFile()
*   Used to search for Setup Files in the INS Tree. The search is performed
*   according to the scheme defined in the INS Common Software Specification
*   "VLT-SPE-ESO-17240-0385, 2.1".
*
*   slxGetDirInfo()
*   Used to get all directories starting from a certain start directory.
*   The slxSTRING_LIST element must be allocated by the calling application.
*   The number of directory entries found is indcated by "noOfEntries".
*
*   FILES
*
*   ENVIRONMENT
*
*
*   RETURN VALUES 
*
*   CAUTIONS 
*   The slxSTING_LIST pointer given in to the functions must point to an
*   element allocated by the calling application.
*
*   EXAMPLES
*
*   SEE ALSO
*   slxStringList
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxSearchUtils.c,v 1.145 2005/07/27 09:28:07 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */ 
#include <stdlib.h>
/* VLTSW20050173 */
#ifdef MAKE_LCU
    #include "vxWorks.h"
#endif
#include <dirent.h>
#include <sys/stat.h>
#include <regex.h>
#include <string.h>
#include <ctype.h>

/* Local Headers
 */
#include "slx.h"

ccsCOMPL_STAT slxSearchInsSetupPath(const slxFILENAME   setupFileName,
				    slxFILENAME         completeName,
				    ccsERROR            *error);

void slxYankWildCards(slxFILENAME   fileName)
{
    if (*fileName == '*')
	strcpy(fileName, (fileName + 1));
    if (slxGetLastChar(fileName) == '*')
	*(fileName + (strlen(fileName) - 1)) = '\0';
}

ccsCOMPL_STAT _slxMatchFiles(char            *extension,
			     slxFILENAME     baseNameLoc,
			     vltLOGICAL      stopFirstOccur,
			     slxSTRING_LIST  *targetFileList,
			     vltINT32        *numberFound,
			     ccsERROR        *error)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL      initialized=ccsFALSE;
    static slxSTRING_LIST  *tmpFileList;
    slxSTRING_LIST         *tmpFilePtr;
    vltINT32               noOfTmpFiles, i;
    slxFILENAME            basePath, compareKey, tmpFileName;

    if (!initialized)
	{
	if (slxInitStrList(&tmpFileList, "HEADER", error) == FAILURE)
	    return FAILURE;
	initialized = TRUE;
	}
    noOfTmpFiles = 0;

    if (slxGetPath(baseNameLoc, basePath, error) == FAILURE)
	return FAILURE;
    if (slxGetDirInfo(basePath, tmpFileList, &noOfTmpFiles, error) == FAILURE)
	return FAILURE;
    tmpFilePtr = tmpFileList->next;
    if (slxGetLastChar(basePath) != '/')
	strcat(basePath, "/");
    if (slxGetFileName(baseNameLoc, compareKey, error) == FAILURE)
	return FAILURE;
    if (slxGetLastChar(compareKey) == '*')
	*(compareKey + (strlen(compareKey) - 1)) = '\0';
    for (i = 0; (i < noOfTmpFiles) &&
	     ((!*numberFound || !stopFirstOccur)); i++)
	{
	/* Check that the extension is correct of the file
	 */
	if (slxCheckExtension2(tmpFilePtr->string, extension))
	    {
	    if (strstr(tmpFilePtr->string, compareKey) == tmpFilePtr->string)
		{
		sprintf(tmpFileName, "%s%s", basePath, tmpFilePtr->string);
		if (slxStrListMerge(tmpFileName, targetFileList, 
				    numberFound, error) == FAILURE)
		    return FAILURE;
		}
	    }
	tmpFilePtr = tmpFilePtr->next;
	}   /* end-for (i = 0; (i < noOfTmpFiles) && ... */
 
    return SUCCESS;
}

ccsCOMPL_STAT slxGetDirInfo(slxFILENAME     dirName,
			    slxSTRING_LIST  *entryList,
			    vltINT32        *noOfEntries,
			    ccsERROR        *error)
{
  return slxGetDirInfo2(dirName,entryList,noOfEntries,error,ccsTRUE);
}

ccsCOMPL_STAT slxGetDirInfo2(slxFILENAME     dirName,
			    slxSTRING_LIST  *entryList,
			    vltINT32        *noOfEntries,
			    ccsERROR        *error,
			    vltLOGICAL      isDirErrReport)
{ 
  char             *completePath;
  DIR              *dirPtr;
  struct dirent    *dirEnt;
  slxFILENAME      dirNameLoc;
  slxSTRING_LIST   *entryPtr;
  vltINT32         entryCount;
  struct stat stbuf;
  
  if (slxResolvePath(dirName, &completePath, error) == FAILURE)
    return FAILURE;
  slxCOPY_MAX_N_BYTES(dirNameLoc, completePath, sizeof(slxFILENAME), error);
  entryPtr = entryList->next;

  /*Check if given directory name is a directory*/  
  if(stat(dirNameLoc,&stbuf)==-1)
      {
      errSysAdd(error, slxMODULE_NAME, slxERR_STATUS_PATH, __FILE__, NULL,modUNIX,
		dirNameLoc);
      return FAILURE;
      }
  
  if( (stbuf.st_mode & S_IFMT)==S_IFDIR  )
    {      
      if ((dirPtr = opendir(dirNameLoc)) == NULL)
	{    
	  errSysAdd(error, slxMODULE_NAME,slxERR_OPEN_DIR, __FILE__, NULL,modUNIX,
		    dirName);
	    return FAILURE;
	}
      
      entryCount = 0;
      while ((dirEnt = readdir(dirPtr)) != NULL)
	{
	  if (*(dirEnt->d_name) != '.')
	    {
	      if (slxStrListMerge(dirEnt->d_name, entryList, &entryCount,
				  error) == FAILURE)
		{
		  closedir(dirPtr);
		  return FAILURE;
		}
	    }    /* end-if (*(dirEnt->d_name) != '.') */
	}    /* end-while ((dirEnt = readdir(dirPtr)) != NULL) */
      
      *noOfEntries = entryCount;
      closedir(dirPtr);
      
    } /*end of if dir*/
  else if (isDirErrReport==ccsTRUE)
    {

      errSysAdd(error, slxMODULE_NAME,slxERR_NOT_DIRECTORY , __FILE__, NULL,modUNIX,
		dirName);  
      return FAILURE;
    }

    return SUCCESS;
}


/* The function does not return FAILURE in case no files are found.
 */
ccsCOMPL_STAT slxSearchSetupFile(const slxFILENAME    baseName,
				 vltINT32             *numberFound,
				 slxSTRING_LIST       *fileList,
				 const vltLOGICAL     globalSearch,
                                 const vltLOGICAL     stopFirstOccur,
				 const vltLOGICAL     skipInsUserDir,
				 ccsERROR             *error)
{ 
    static slxSTRING_LIST  *modeDirList;  
    static slxSTRING_LIST  *userDirList;
    static slxSTRING_LIST  *dirFileList;
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL      initialized=ccsFALSE;
    static ccsERROR        tmpError;
    char                   *chrPtr;
    static char            *extBuf[] = {slxINS_EXTENSION, slxDET_EXTENSION,
			  	        slxTARG_EXTENSION, slxREF_EXTENSION};
    static char            *typeDirBuf[] = {slxINS_SETUP_FILE_DIR,
					    slxDET_SETUP_FILE_DIR,
					    slxTARG_SETUP_FILE_DIR,
					    slxREF_SETUP_FILE_DIR};
    struct stat            statBuf;
    slxFILENAME            baseNameLoc;
    slxFILENAME            fileName;  
    slxFILENAME            insUserDir;
    slxFILENAME            basePath;
    slxSETUP_TYPE          setupType;
    slxSTRING_LIST         *modeDirPtr;
    slxSTRING_LIST         *userDirPtr;
    vltINT32               i, j;
    vltINT32               userDir;
    vltINT32               noOfModeDirs;
    vltINT32               noOfUserDirs;
    vltLOGICAL             wildCards = FALSE;
    slxFILENAME            completeName;


    if (!initialized)
	{
	if (slxInitStrList(&modeDirList, "HEADER", error) == FAILURE)
	    return FAILURE;
	if (slxInitStrList(&userDirList, "HEADER", error) == FAILURE)
	    return FAILURE;
	if (slxInitStrList(&dirFileList, "HEADER", error) == FAILURE)
	    return FAILURE;
	errResetStack(&tmpError);
	initialized = TRUE;
	}

    *numberFound = 0;

    /* Try to open the file exactly as it is. If this succeeds we return
     * immedialtely
     */
    /* 
     * VLTSW20050173 
     * if (stat(baseName, &statBuf) == 0)
     */
    if (stat((char *)baseName, &statBuf) == 0)
	{
	if (slxStrListMerge((char *)baseName, fileList, numberFound,
			    error) == FAILURE)
	    return FAILURE;
	return SUCCESS;
	}
    
    /* Check if the filename contains "~/..." or "$<env var>", and resolved
     * this if this is the case
     */
    if ((*baseName == '~') || (*baseName == '$'))
	{
	if (slxResolvePath(baseName, &chrPtr, error) == FAILURE)
	    return FAILURE;
	slxCOPY_MAX_N_BYTES(baseNameLoc, chrPtr, slxFILENAME_LEN, error);
	}
    else
	slxCOPY_MAX_N_BYTES(baseNameLoc, baseName, slxFILENAME_LEN, error);


    /* If complete path just stat() the file:
     * 1. If known Setup File extension only once.
     * 2. If no extension is specified try with .ins, .det, .targ, .ref
     */
    if (*baseNameLoc == '/')
	{
/* VLTSW20050173 */
#if defined(SUN_COMP) || defined(LINUX) || defined(MAKE_LCU)
	if ((strstr(baseNameLoc, slxINS_EXTENSION) != NULL) || 
	    (strstr(baseNameLoc, slxDET_EXTENSION) != NULL) || 
	    (strstr(baseNameLoc, slxTARG_EXTENSION) != NULL) ||
	    (strstr(baseNameLoc, slxREF_EXTENSION) != NULL))
#else
	if ((strrstr(baseNameLoc, slxINS_EXTENSION) != NULL) || 
	    (strrstr(baseNameLoc, slxDET_EXTENSION) != NULL) || 
	    (strrstr(baseNameLoc, slxTARG_EXTENSION) != NULL) ||
	    (strrstr(baseNameLoc, slxREF_EXTENSION) != NULL))
#endif
	    {
	    if (!wildCards)
		{
		/* The filename includes the extension - just stat() 
		 * it as it is
		 */	      
		if (stat(baseNameLoc, &statBuf) == 0)
		    {
		    if (slxStrListMerge((char *)baseNameLoc, fileList,
					numberFound, error) == FAILURE)
			return FAILURE;
		    }
		}    /* end-if (!wildCards) */
	    }
	else
	    {
	    /* Check the various possibilities
	     */
	    for (i = 0; ((i < 4) && (!*numberFound || !stopFirstOccur)); i++)
		{
		strcpy(fileName, baseNameLoc);
		strcat(fileName, extBuf[i]);
		if (stat(fileName, &statBuf) == 0)
		    {
		    if (slxStrListMerge(fileName, fileList, numberFound,
					error) == FAILURE)
			return FAILURE;
		    }
		}
	    }
	return SUCCESS;
	}    /* end-if (*baseNameLoc == '/') */

    /* Temporary, this is a temporary path to catch if the INS_USER
     * is not properly set. Should be taken out. Later just return error if
     * not set.
     */
    if (slxGetValueEnvVar("INS_USER", &chrPtr, &tmpError) == FAILURE)
	{
	/* Try to locate it via the INS_SETUPPATH
	 */
	errResetStack(&tmpError);
	if (slxSearchInsSetupPath((char *)baseName, fileName, 
				  &tmpError) == FAILURE)
	    {
	    errResetStack(&tmpError);
	    return SUCCESS;
	    }
	if (*fileName != '\0')
	    { 
	    if (slxStrListMerge(fileName, fileList, numberFound,
				error) == FAILURE)
		return FAILURE;
	    }
	return SUCCESS;
	}

    /* Check if the name contains wildcards
     */
    if ((strchr(baseName, '*') != NULL) || (strchr(baseName, '?') != NULL))
	wildCards = TRUE;
    else
	wildCards = FALSE;

    /* The path is not given - check if the type of the file is given
     */
    if (strstr(baseNameLoc, slxINS_EXTENSION) != NULL)
	setupType = slxINSTRUMENT_SETUP;
    else if (strstr(baseNameLoc, slxDET_EXTENSION) != NULL)
	setupType = slxDETECTOR_SETUP;
    else if (strstr(baseNameLoc, slxTARG_EXTENSION) != NULL)
	setupType = slxTARGET_SETUP;
    else if (strstr(baseNameLoc, slxREF_EXTENSION) != NULL)
	setupType = slxREFERENCE_SETUP;
    else
	setupType = slxUNKNOWN_SETUP_TYPE;

    /* If we should not skip searching the INS_USER directory, we check now in 
     * the Observation Mode Directories in the current 
     * INS_USER directory, first get the various directories
     */
    if (!skipInsUserDir)
	{
	if (slxBuildInsUserDir(insUserDir, "", error) == FAILURE)
	    return FAILURE;
	if (slxGetDirInfo(insUserDir, modeDirList, &noOfModeDirs, 
			  error) == FAILURE)
	    return FAILURE;

	/* Look only in Observation Mode Directories in the INS_USER directory.
	 */
	modeDirPtr = modeDirList->next;
	for (i = 0; i < noOfModeDirs; i++)
	    {
	    if ((strstr(modeDirPtr->string, "Alias") == NULL) &&
		(strstr(modeDirPtr->string, "DETDATA") == NULL) &&
		(strstr(modeDirPtr->string, "Dictionary") == NULL) &&
		(strstr(modeDirPtr->string, "MISC") == NULL) &&
		(strstr(modeDirPtr->string, "CVS") == NULL))
		{
		strcpy(basePath, insUserDir);
		if (slxGetLastChar(basePath) != '/')
		    strcat(basePath, "/");
		strcat(basePath, modeDirPtr->string);
		if (slxGetLastChar(basePath) == '/')
		    slxSetLastChar(basePath, '\0');
		strcat(basePath, slxSETUP_FILE_DIR);

		if (setupType == slxUNKNOWN_SETUP_TYPE)
		    {
		    for (j = 0; ((j < 4) && 
				 (!*numberFound || !stopFirstOccur));j++)
			{
			strcpy(fileName, basePath);		    
			strcat(fileName, typeDirBuf[j]);
			strcat(fileName, "/");
			strcat(fileName, baseNameLoc);
			strcat(fileName, extBuf[j]);
			if (stat(fileName, &statBuf) == 0)
			    {
			    if (slxStrListMerge(fileName, fileList, 
						numberFound, error) == FAILURE)
				return FAILURE;
			    }
			}   /* end-for (j = 0; ((j < 4) && ... */
		    }   /* end-if (setupType == slxUNKNOWN_SETUP_TYPE) */
		else
		    {
		    strcpy(fileName, basePath);		    
		    switch (setupType)
			{
			case slxDETECTOR_SETUP:
			    strcat(fileName, slxDET_SETUP_FILE_DIR);
			    break;		   
			case slxINSTRUMENT_SETUP:
			    strcat(fileName, slxINS_SETUP_FILE_DIR);
			    break; 
			case slxREFERENCE_SETUP:
			    strcat(fileName, slxREF_SETUP_FILE_DIR);
			    break;
			default:
			    strcat(fileName, slxTARG_SETUP_FILE_DIR);
			    break;
			}    /* end-switch (setupType) */		  
		    if (slxGetLastChar(fileName) != '/')
			strcat(fileName, "/");
		    strcat(fileName, baseNameLoc);
		    if (stat(fileName, &statBuf) == 0)
			{
			if (slxStrListMerge(fileName, fileList, numberFound,
					    error) == FAILURE)
			    return FAILURE;
			}
		    }
		}   /* end-if ((strstr("Alias", modeDirPtr->string) ... */
	    modeDirPtr = modeDirPtr->next;
	    }   /* end-for (i = 0; i < noOfModeDirs; i++) */
	}   /* end-if (!skipInsUserDir) */
    
    /* If a global search is requested do it if a file was found and there
     * should be searched for more, or if no files were found so far.
     * Don't check the INS_USER directory - is already checked, or should not
     * be checked.
     */
    if (globalSearch && ((*numberFound && !stopFirstOccur) || !*numberFound))
	{
	if ((chrPtr = getenv(slxINS_ROOT)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_ROOT);
	    return FAILURE;
	    }
	if (slxGetDirInfo(chrPtr, userDirList, &noOfUserDirs,
			  error) == FAILURE)
	    return FAILURE;

	if ((chrPtr = getenv(slxINS_USER)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_USER);
	    return FAILURE;
	    }

	/* Loop over the User Directories
	 */
	userDirPtr = userDirList->next;
	for (userDir = 0; userDir < noOfUserDirs; userDir++)
	    {
	    if ((strstr(userDirPtr->string, chrPtr) == NULL) &&
		(strstr(userDirPtr->string, "CVS") == NULL))
		{
		/* Get the Observation Mode directories within this User Dir 
		 * and loop over that.
		 */
		if (slxBuildInsUserDir(insUserDir, userDirPtr->string, 
				       error) == FAILURE)
		    return FAILURE;
		if (slxGetDirInfo(insUserDir, modeDirList, &noOfModeDirs, 
				  error) == FAILURE)
		    return FAILURE;

		/* Assume here that there are only Observation Mode 
		 * Directories in the INS_USER directory.
		 */
		modeDirPtr = modeDirList->next;
		for (i = 0; i < noOfModeDirs; i++)
		    {
		    if ((strstr(modeDirPtr->string, "Alias") == NULL) &&
			(strstr(modeDirPtr->string, "DETDATA") == NULL) &&
			(strstr(modeDirPtr->string, "Dictionary") == NULL) &&
			(strstr(modeDirPtr->string, "MISC") == NULL) &&
			(strstr(modeDirPtr->string, "CVS") == NULL))
			{
			strcpy(basePath, insUserDir);
			if (slxGetLastChar(basePath) != '/')
			    strcat(basePath, "/");
			strcat(basePath, modeDirPtr->string);
			if (slxGetLastChar(basePath) == '/')
			    slxSetLastChar(basePath, '\0');
			strcat(basePath, slxSETUP_FILE_DIR);
		    
			if (setupType == slxUNKNOWN_SETUP_TYPE)
			    {
			    for (j = 0; ((j < 4) && (!*numberFound || 
						     !stopFirstOccur)); j++)
				{
				strcpy(fileName, basePath);		    
				strcat(fileName, typeDirBuf[j]);
				strcat(fileName, "/");
				strcat(fileName, baseNameLoc);
				strcat(fileName, extBuf[j]);
				if (stat(fileName, &statBuf) == 0)
				    {
				    if (slxStrListMerge(fileName, fileList, 
							numberFound,
							error) == FAILURE)
					return FAILURE;
				    }
				}	/* end-for (i = 0; ((i < 4) && ... */
			    }   /* end-if (setupType == slxUNKNOWN_SETUP ... */
			else
			    {
			    strcpy(fileName, basePath);		    
			    switch (setupType)
				{
				case slxDETECTOR_SETUP:
				    strcat(fileName, slxDET_SETUP_FILE_DIR);
				    break;		   
				case slxINSTRUMENT_SETUP:
				    strcat(fileName, slxINS_SETUP_FILE_DIR);
				    break; 
				case slxREFERENCE_SETUP:
				    strcat(fileName, slxREF_SETUP_FILE_DIR);
				    break;
				default:
				    strcat(fileName, slxTARG_SETUP_FILE_DIR);
				    break;
				}    /* end-switch (setupType) */
			    if (slxGetLastChar(fileName) != '/')
				strcat(fileName, "/");
			    strcat(fileName, baseNameLoc);	       	    
			    if (stat(fileName, &statBuf) == 0)
				{
				if (slxStrListMerge(fileName, fileList, 
						    numberFound,
						    error) == FAILURE)
				    return FAILURE;
				}
			    }
			}  /* end-if ((strrstr("Alias", modeDirPtr-> ... */
		    modeDirPtr = modeDirPtr->next;
		    }   /* end-for (i = 0; i < noOfModeDirs; i++) */
		}   /* end-if (strstr(chrPtr, userDirList->string) ... */
	    userDirPtr = userDirPtr->next;
	    }   /* end-for (userDir = 0; userDir < noOfUserDirs; userDir++) */
	}   /* end-if (globalSearch && ((*numberFound && ... */
  
    /* If still not found, try to locate it according to the INS_SETTUPATH
     */
    if (!*numberFound)
	{
	if (slxSearchInsSetupPath(baseNameLoc, completeName, 
				  &tmpError) == FAILURE)
	    {
	    errResetStack(&tmpError);
	    return SUCCESS;
	    }	
	if (*completeName != '\0')
	    {
	    if (slxStrListMerge(completeName, fileList, numberFound,
				error) == FAILURE)
		return FAILURE;
	    }
	}
    
    return SUCCESS;
}

ccsCOMPL_STAT slxSearchInsSetupPath(const slxFILENAME   setupFileName,
				    slxFILENAME         completeName,
				    ccsERROR            *error)
{
  static char            *extBuf[] = {slxINS_EXTENSION, slxDET_EXTENSION,
				      slxTARG_EXTENSION, slxREF_EXTENSION};
  char                   *nextPathPtr;
  char                   *prevPathPtr;
  char                   *insSetUpPathPtr; 
  struct stat            statBuf;
  slxFILENAME            currentPath;
  vltUINT8               fileFound = 0;
  vltUINT8               morePaths = 1;
  vltUINT8               pathLen = 0;
  vltUINT16              pathVarLen;
  vltUINT16              pathIndex = 0;
  vltUINT16              count;
  vltINT32               extCount;


  *completeName = '\0';
  if ((insSetUpPathPtr = getenv(slxINS_SETUPPATH_NAME)) == NULL)
    {
    errSysAdd(error, slxMODULE_NAME, slxERR_ENV_VAR, __FILE__, 
	      NULL, modUNIX, slxINS_SETUPPATH_NAME);
    return FAILURE;
    }

  nextPathPtr = insSetUpPathPtr;
  prevPathPtr = insSetUpPathPtr;
  pathVarLen = strlen(insSetUpPathPtr);
  
  while ((!fileFound) && morePaths)
    {
    /* Generate path. We assume that the path list in INS_SETUPPATH
     * can have the following structure:
     * [:]/-----/-----[/]:/-----/-----/-----[/]: ... :/-----/-----[/][:]
     */
    if (((pathIndex + 1) == pathVarLen) &&
	(strcmp(insSetUpPathPtr, slxINS_PATH_DEFAULT) != 0))
      morePaths = 0;
    else
      {
      /* If the first char in the path segnent is a ':', skip it */
      if (*prevPathPtr == ':')
	{
	prevPathPtr++;
	pathIndex++;
	}
      
      /* If this is the last ":", it is the last piece of path */
      if ((nextPathPtr = strstr(prevPathPtr, ":")) == NULL)
	{
	count = 0;
	nextPathPtr = prevPathPtr;
	memset((char *)currentPath, 0, (size_t)256);
	while ((*nextPathPtr != ':') && (pathIndex < pathVarLen))
	  {
	  if (!isalpha((int)*nextPathPtr) &&
	      !isdigit((int)*nextPathPtr) &&
	      (*nextPathPtr != '_') && 
	      (*nextPathPtr != '-') &&
	      (*nextPathPtr != '#') &&
	      (*nextPathPtr != '/') &&
	      (*nextPathPtr != '.'))
	    {
	    /* There is an illegal character in the string
	     */
	    errAdd(error, slxMODULE_NAME, slxERR_INS_SETUPPATH, 
		   __FILE__, NULL);
	    *completeName = '\0';
	    return FAILURE;
	    }
	  
	  *(currentPath + count++) = *nextPathPtr;
	  pathIndex++;
	  nextPathPtr++;
	  }
	if (pathIndex == pathVarLen)
	  morePaths = 0;
	}
      
      /* Otherwise there is another path component 
       */
      else
	{
	pathLen = (nextPathPtr - prevPathPtr);
	pathIndex += pathLen;
	memset((char *)currentPath, 0, (size_t)256);
	strncpy((char *)currentPath, prevPathPtr, (size_t)pathLen);
	}
      
      /* Check trying with the allowed file extensions 
       */
      extCount = -1;
      while ((extCount < 4) && !fileFound)
	{
	strcpy(completeName, currentPath);
	if (*(currentPath + strlen(currentPath) - 1) != '/')
	  strcat(completeName, "/");
	strcat(completeName, setupFileName);
	if (extCount >= 0)
	  strcat(completeName, extBuf[extCount]);
	extCount++;
	if (stat(completeName, &statBuf) == 0)
	  fileFound = 1;
	}   /* end-while ((extCount < 5) && !fileFound) */
      }
    prevPathPtr = nextPathPtr;
    }

  if (!fileFound)
    *completeName = '\0';

  return SUCCESS;
}

/* IMPL.: This function is not yet finished.
 * The function is used to search for a file in the $INS_ROOT/$INS_USER/MISC
 * directories. It does not return FAILURE in case no files are found.
 *
 * Search Strategy:
 * 
 * 1. Start to search in $INS_ROOT/$INS_USER/MISC ..., dir walking through
 *    all the dir.s in the user's dir if not "stopFirstOccur", and not
 *    "skipInsUserDir".
 *
 * 2. If "globalSearch" start to go through all the <ins_root>/<user/MISC
 *    dir.s. Stop either at first occurrence or go through the complete 
 *    tree.
 */
ccsCOMPL_STAT slxSearchUserFile(const slxFILENAME    baseName,
				vltINT32             *numberFound,
				slxSTRING_LIST       *fileList,
				const vltLOGICAL     globalSearch,
				const vltLOGICAL     stopFirstOccur,
				const vltLOGICAL     skipInsUserDir,
				ccsERROR             *error)
{ 
/* 
   MNY: VLTSW20040034Removal of ghost code
    static slxSTRING_LIST  *miscDirList1;
    static slxSTRING_LIST  *miscFileList2;
    static slxSTRING_LIST  *userDirList;
    static slxSTRING_LIST  *dirFileList;
*/

/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL      initialized=ccsFALSE;
    static ccsERROR        tmpError;
    ccsSTACK_ELEM          *errDescription;
    slxFILENAME            fileName;  
    vltLOGICAL             errFound;
/*
   MNY: VLTSW20040034 Removal of ghost code
    char                   *chrPtr;
    struct stat            statBuf;
    slxFILENAME            baseNameLoc;
    slxFILENAME            insUserDir;
    slxSTRING_LIST         *userDirPtr;
    slxSTRING_LIST         *miscDirPtr1;
    vltINT32               i, j;
    vltINT32               userDir;
    vltINT32               noOfUserDirs;
    vltINT32               noOfMiscDirs1;
    vltLOGICAL             wildCards;
    vltLOGICAL             errFound, errFound1, errFound2;
*/

    if (!initialized)
	{
	errResetStack(&tmpError);
	initialized = TRUE;
	}

  /* IMPL.: This function is not yet finished so we only search after 
     * user files according to the INS_USER_PATH variable
     */
    if (slxLocateFile("INS_USER_PATH", baseName, fileName, 
		      &tmpError) == FAILURE)
	{
	if (errIsReason(&tmpError, slxMODULE_NAME, slxERR_LOCATE_FILE,
			&errFound, &errDescription) == FAILURE)
	    {
	    errMergeStack(error, &tmpError, errBOTTOM);
	    errResetStack(&tmpError);
	    return FAILURE;
	    }
	if (errFound)
	    {
	    errResetStack(&tmpError);
	    *numberFound = 0;
	    return SUCCESS;
	    }
	else
	    {
	    errMergeStack(error, &tmpError, errBOTTOM);
	    errResetStack(&tmpError);
	    return FAILURE;
	    }
	}
    else
	{ 
	if (slxStrListMerge(fileName, fileList, numberFound, error) == FAILURE)
	    return FAILURE;
	*numberFound = 1;
	}
    return SUCCESS;


/*
MNY Removal of ghost code VLTSW20040034
    if (!initialized)
	{
	if (slxInitStrList(&miscDirList1, "HEADER", error) == FAILURE)
	    return FAILURE;
	if (slxInitStrList(&miscFileList2, "HEADER", error) == FAILURE)
	    return FAILURE;
	if (slxInitStrList(&userDirList, "HEADER", error) == FAILURE)
	    return FAILURE;
	if (slxInitStrList(&dirFileList, "HEADER", error) == FAILURE)
	    return FAILURE;
	initialized = TRUE;
	}

    *numberFound = 0;

    // Check if the name contains wildcards
    //

    if ((strchr(baseName, '*') != NULL) || (strchr(baseName, '?') != NULL))
	wildCards = TRUE;
    else
	wildCards = FALSE;
    // Check if the filename contains "~/..." or "$<env var>", and resolved
    // this if this is the case
    //
    if ((*baseName == '~') || (*baseName == '$'))
	{
	if (slxResolvePath((char *)baseName, &chrPtr, error) == FAILURE)
	    return FAILURE;
	slxCOPY_MAX_N_BYTES(baseNameLoc, chrPtr, slxFILENAME_LEN, error);
	}
    else
	slxCOPY_MAX_N_BYTES(baseNameLoc, baseName, slxFILENAME_LEN, error);

    // If complete path just stat() the file.
    //
    if (*baseNameLoc == '/')
	{
	if (!wildCards)
	    {
	    // The filename includes the extension - just stat() 
	    // it as it is
	    //	      
	    if (stat(baseNameLoc, &statBuf) == 0)
		{
		if (slxStrListMerge((char *)baseNameLoc, fileList,
				    numberFound, error) == FAILURE)
		    return FAILURE;
		}
	    else
		{
		// IMPL.:
		///
		}
	    }    // end-if (!wildCards) //
}    // end-if (*baseNameLoc == '/') //

    // If we should not skip searching the INS_USER directory, we check now in 
    // the MISC Directories in the current INS_USER directory, first get
    // the various directories.
    ///
    if (!skipInsUserDir)
	{
	if (slxBuildInsUserDir(insUserDir, "", error) == FAILURE)
	    return FAILURE;
	strcat(insUserDir, "MISC/");

	// Look only in Observation Mode Directories in the INS_USER directory.
	 //
	miscDirPtr1 = miscDirList1->next;
	for (i = 0; i < noOfMiscDirs1; i++)
	    {
	    // 1. Check $USER/MISC.
	    // 2. If no success|!stopFirstOccur, get check 
	    //    <dir n> in $USER/MISC.
	    //
	    if (!wildCards)
		{
		strcpy(fileName, insUserDir);
		slxSetLastChar(fileName, '/');
		strcat(fileName, baseName);
		if (stat(fileName, &statBuf) == 0)
		    {
		    if (slxStrListMerge(fileName, fileList, 
					numberFound, error) == FAILURE)
			return FAILURE;
		    if (stopFirstOccur)
			return SUCCESS;
		    }

		// We start to check the sub-directories in the MISC dir
		 // 
		// 1. Get all files/dir.s in MISC
		///
		if (slxGetDirInfo(insUserDir, miscDirList1, &noOfMiscDirs1, 
				  error) == FAILURE)
		    return FAILURE;
		for (j = 0; j < noOfMiscDirs1; j++)
		    {
		    // IMPL.
		     //
		    }
		}
	    else
		{
		// The name contains wild cards so we have to do matching
		 // with regular expressions.
		 //
		// IMPL.: //
		}
	    miscDirPtr1 = miscDirPtr1->next;
	    }   // end-for (i = 0; i < noOfMiscDirs1; i++) //
	}   // end-if (!skipInsUserDir) //
    
    // If a global search is requested do it if a file was found and there
     // should be searched for more, or if no files were found so far.
     // Don't check the INS_USER directory - is already checked, or should not
     // be checked.
     //
    if (globalSearch && ((*numberFound && !stopFirstOccur) || !*numberFound))
	{
	if ((chrPtr = getenv(slxINS_ROOT)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_ROOT);
	    return FAILURE;
	    }
	if (slxGetDirInfo(chrPtr, userDirList, &noOfUserDirs,
			  error) == FAILURE)
	    return FAILURE;

	if ((chrPtr = getenv(slxINS_USER)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_USER);
	    return FAILURE;
	    }

	// Loop over the User Directories
	 //
	userDirPtr = userDirList->next;
	for (userDir = 0; userDir < noOfUserDirs; userDir++)
	    {
	    if ((strstr(userDirPtr->string, chrPtr) == NULL) &&
		(strstr(userDirPtr->string, "CVS") == NULL))
		{
		// Get the Observation Mode directories within this User Dir 
		// and loop over that.
		 //
		if (slxBuildInsUserDir(insUserDir, userDirPtr->string, 
				       error) == FAILURE)
		    return FAILURE;
		//if (slxGetDirInfo(insUserDir, modeDirList, &noOfModeDirs, 
				  error) == FAILURE)
		    return FAILURE;//

		// Check all MISC dir.s in the user dir.s
		 //
		miscDirPtr1 = miscDirList1->next;
		for (i = 0; i < noOfMiscDirs1; i++)
		    {
		    miscDirPtr1 = miscDirPtr1->next;
		    }   // end-for (i = 0; i < noOfMiscDirs1; i++) //
		}   // end-if (strstr(chrPtr, userDirList->string) ... //
	    userDirPtr = userDirPtr->next;
	    }   // end-for (userDir = 0; userDir < noOfUserDirs; userDir++) //
}   // end-if (globalSearch && ((*numberFound && ... //
  
    // If still not found, try to locate it according to the INS_SETTUPATH
     //
    if (!*numberFound)
	{
	if (slxLocateSetupFile((char *)baseNameLoc, fileName, 
			       &tmpError) == FAILURE)
	    {
	    if ((errIsReason(&tmpError, slxMODULE_NAME, slxERR_LOCATE_FILE,
			     &errFound1, &errDescription) == FAILURE) ||
		(errIsReason(&tmpError, slxMODULE_NAME, slxERR_INS_SETUPPATH,
			     &errFound2, &errDescription) == FAILURE))
		{
		errMergeStack(error, &tmpError, errBOTTOM);
		errResetStack(&tmpError);
		return FAILURE;
		}
	    if (errFound1 || errFound2)
		errResetStack(&tmpError);
	    else
		{
		errMergeStack(error, &tmpError, errBOTTOM);
		errResetStack(&tmpError);
		return FAILURE;
		}
	    }
	else
	    { 
	    if (slxStrListMerge(fileName, fileList, numberFound,
				error) == FAILURE)
		return FAILURE;
	    }
	}

    return SUCCESS;
*/
}

ccsCOMPL_STAT slxSearchConfigFile(const slxFILENAME      baseName,
				  vltINT32               *numberFound,
				  slxSTRING_LIST         *fileList,
				  const vltLOGICAL       globalSearch,
				  const vltLOGICAL       stopFirstOccur,
				  const vltLOGICAL       skipInsUserDir,
				  const char             *pathEnvVar,
				  ccsERROR               *error)
{
    static slxSTRING_LIST  *modeDirList;  
    static slxSTRING_LIST  *userDirList;
    static slxSTRING_LIST  *tmpFileList;
  /* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL      initialized=ccsFALSE;
    static ccsERROR        tmpError;
    char                   *chrPtr;
    static char            *extBuf[] = {slxINS_CONFIG_FILE_EXT,
					slxREF_CONFIG_FILE_EXT, NULL}; 
    regex_t                preg;
    struct stat            statBuf;
    ccsSTACK_ELEM          *errDescription;
    slxFILENAME            baseNameLoc;
    slxFILENAME            fileName;  
    slxFILENAME            insUserDir;
    slxFILENAME            basePath;
    slxCONFIG_TYPE         configType;
    slxSTRING_LIST         *modeDirPtr;
    slxSTRING_LIST         *userDirPtr;
    slxSTRING_LIST         *tmpFilePtr;
    vltINT32               i, j;
    vltINT32               userDir;
    vltINT32               noOfModeDirs;
    vltINT32               noOfUserDirs;
    vltINT32               noOfTmpFiles;
    vltLOGICAL             run;
    vltLOGICAL             wildCards;
    vltLOGICAL             errFound;


    if (!initialized)
	{
	if (slxInitStrList(&modeDirList, "HEADER", error) == FAILURE)
	    return FAILURE;
	if (slxInitStrList(&userDirList, "HEADER", error) == FAILURE)
	    return FAILURE;
	if (slxInitStrList(&tmpFileList, "HEADER", error) == FAILURE)
	    return FAILURE;
	errResetStack(&tmpError);
	initialized = TRUE;
	}

    *numberFound = 0;

    /* Check if the name contains wildcards
     */
    if ((strchr(baseName, '*') != NULL) || (strchr(baseName, '?') != NULL))
	wildCards = TRUE;
    else
	wildCards = FALSE;

    /* Check if the filename contains "~/..." or "$<env var>", and resolved
     * this if this is the case
     */
    if ((*baseName == '~') || (*baseName == '$'))
	{
	if (slxResolvePath(baseName, &chrPtr, error) == FAILURE)
	    return FAILURE;
	slxCOPY_MAX_N_BYTES(baseNameLoc, chrPtr, slxFILENAME_LEN, error);
	}
    else
	slxCOPY_MAX_N_BYTES(baseNameLoc, baseName, slxFILENAME_LEN, error);


    /* If complete path just stat() the file:
     * 1. If known Config File extension only once.
     * 2. If no extension is specified try with .cfg, .refcfg, .auxcfg
     */
    if (*baseNameLoc == '/')
	{                                        
/* VLTSW20050173 */
#if defined(SUN_COMP) || defined(LINUX) || defined(MAKE_LCU)
	if ((strstr(baseNameLoc, slxINS_CONFIG_FILE_EXT) != NULL) || 
	    (strstr(baseNameLoc, slxREF_CONFIG_FILE_EXT) != NULL))
#else
	if ((strrstr(baseNameLoc, slxINS_CONFIG_FILE_EXT) != NULL) || 
	    (strrstr(baseNameLoc, slxREF_CONFIG_FILE_EXT) != NULL))
#endif
	    {
	    /* The filename includes the path
	     */	     
	    if (!wildCards)
		{
		if (stat(baseNameLoc, &statBuf) == 0)
		    {
		    if (slxStrListMerge(baseNameLoc, fileList, numberFound,
					error) == FAILURE)
			return FAILURE;
		    }
		}
	    else
		{
		/* Get all the files in the directory and make a regexp()
		 * for each of them
		 */
		slxGetPath(baseNameLoc, basePath, error);
		if (slxGetDirInfo(basePath, tmpFileList, &noOfTmpFiles, 
				  error) == FAILURE)
		    return FAILURE;
		tmpFilePtr = tmpFileList->next;
		if (regcomp(&preg, baseNameLoc, REG_NOSUB) != 0)
		    {
		    errSysAdd(error, slxMODULE_NAME, slxERR_SYSTEM_CALL, 
			      __FILE__, NULL, modUNIX, "regcomp()",
			      "Cannot filter on filenames");
		    return FAILURE;
		    }
		for (i = 0; (i < noOfTmpFiles) && 
		     ((!*numberFound || !stopFirstOccur)); i++)
		    {
		    if (regexec(&preg, tmpFileList->string, (size_t)0, 
				NULL, 0) == 0)
			{
			if (slxStrListMerge(tmpFileList->string, fileList, 
					    numberFound, error) == FAILURE)
			    {
			    regfree(&preg);
			    return FAILURE;
			    }
			}
		    regfree(&preg);
		    }   /* end-for (i = 0; (i < noOfTmpFiles) && ... */
		}
	    }
	else
	    {
	    /* Check the various possibilities
	     */
	    for (i = 0; ((i < 3) && (!*numberFound || !stopFirstOccur)); i++)
		{
		strcpy(fileName, baseNameLoc);
                /*
		  MNY VLTSW20040202 
		  strcat(fileName, extBuf[i]);
                  Do not add a NULL string to the filename 
		  Otherwise it will result in a core dump for LINUX and SUN
		*/
		if (i!=2)
		    strcat(fileName, extBuf[i]);
		
		if (stat(fileName, &statBuf) == 0)
		    {
		    if (slxStrListMerge(fileName, fileList, numberFound,
					error) == FAILURE)
			return FAILURE;
		    }
		}
	    }
	return SUCCESS;
	}    /* end-if (*baseNameLoc == '/') */
    
    
    /* The path is not given - check if the type of the file is given
     */
/* VLTSW20050173 */
#if defined(SUN_COMP) || defined(LINUX) || defined(MAKE_LCU)
    if (strstr(baseNameLoc, slxINS_CONFIG_FILE_EXT) != NULL)
	configType = slxINS_CONFIG_FILE;
    else if (strstr(baseNameLoc, slxREF_CONFIG_FILE_EXT) != NULL)
	configType = slxREF_CONFIG_FILE;
    else
	configType = slxANY_CONFIG_FILE;
#else
    if (strrstr(baseNameLoc, slxINS_CONFIG_FILE_EXT) != NULL)
	configType = slxINS_CONFIG_FILE;
    else if (strrstr(baseNameLoc, slxREF_CONFIG_FILE_EXT) != NULL)
	configType = slxREF_CONFIG_FILE;
    else
	configType = slxANY_CONFIG_FILE;
#endif
    
    /* If we should not skip searching the INS_USER directory, we check now in 
     * the Observation Mode Directories in the current 
     * INS_USER directory, first get the various directories
     */
    if (!skipInsUserDir)
	{
	if (slxBuildInsUserDir(insUserDir, "", error) == FAILURE)
	    return FAILURE;
	if (slxGetDirInfo(insUserDir, modeDirList, &noOfModeDirs, 
			  error) == FAILURE)
	    return FAILURE;
	
	/* Look only in Observation Mode Directories in the INS_USER directory.
	 */
	modeDirPtr = modeDirList->next;
	run = TRUE;
	for (i = 0; (i < noOfModeDirs) && run; i++)
	    {
	    if ((strstr(modeDirPtr->string, "Alias") == NULL) &&
		(strstr(modeDirPtr->string, "DETDATA") == NULL) &&
		(strstr(modeDirPtr->string, "Dictionary") == NULL) &&
		(strstr(modeDirPtr->string, "MISC") == NULL) &&
		(strstr(modeDirPtr->string, "CVS") == NULL))
		{
		strcpy(basePath, insUserDir);
		slxSetLastChar(basePath, '/');
		strcat(basePath, modeDirPtr->string);
		slxSetLastChar(basePath, '/');
		strcat(basePath, slxCONFIG_FILE_DIR);
		
		if (configType == slxANY_CONFIG_FILE)
		    {
		    for (j = 0; ((j < 3) && 
				 (!*numberFound || !stopFirstOccur)); j++)
			{
			strcpy(fileName, basePath);		    
			slxSetLastChar(fileName, '/');
			strcat(fileName, baseNameLoc);
			strcat(fileName, extBuf[j]);
			if (stat(fileName, &statBuf) == 0)
			    {
			    if (slxStrListMerge(fileName, fileList, 
						numberFound, error) == FAILURE)
				return FAILURE;
			    }
			}   /* end-for (j = 0; ((j < 4) && ... */
		    }   /* end-if (configType == slxINS_UNKNOWN_CONFIG) */
		else
		    {
		    slxSetLastChar(basePath, '/');
		    strcat(basePath, baseNameLoc);
		    if (stat(basePath, &statBuf) == 0)
			{
			if (slxStrListMerge(basePath, fileList, numberFound,
					    error) == FAILURE)
			    return FAILURE;
			}
		    }
		}   /* end-if ((strstr("Alias", modeDirPtr->string) ... */
	    if (*numberFound && stopFirstOccur)
		run = FALSE;
	    else
		modeDirPtr = modeDirPtr->next;
	    }   /* end-for (i = 0; i < noOfModeDirs; i++) */
	}   /* end-if (!skipInsUserDir) */
    
    /* If a global search is requested do it if a file was found and there
     * should be searched for more, or if no files were found so far.
     * Don't check the INS_USER directory - is already checked, or should not
     * be checked.
     */
    if (globalSearch && ((*numberFound && !stopFirstOccur) || !*numberFound))
	{
	if ((chrPtr = getenv(slxINS_ROOT)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_ROOT);
	    return FAILURE;
	    }
	if (slxGetDirInfo(chrPtr, userDirList, &noOfUserDirs,
			  error) == FAILURE)
	    return FAILURE;

	if ((chrPtr = getenv(slxINS_USER)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_USER);
	    return FAILURE;
	    }

	/* Loop over the User Directories
	 */
	userDirPtr = userDirList->next;
	run = TRUE;
	for (userDir = 0; (userDir < noOfUserDirs) && run; userDir++)
	    {
	    if ((strstr(userDirPtr->string, chrPtr) == NULL) &&
		(strstr(userDirPtr->string, "CVS") == NULL))
		{
		/* Get the Observation Mode directories within this User Dir 
		 * and loop over that.
		 */
		if (slxBuildInsUserDir(insUserDir, userDirPtr->string, 
				       error) == FAILURE)
		    return FAILURE;
		if (slxGetDirInfo(insUserDir, modeDirList, &noOfModeDirs, 
				  error) == FAILURE)
		    return FAILURE;
		
		/* Assume here that there are only Observation Mode 
		 * Directories in the INS_USER directory.
		 */
		modeDirPtr = modeDirList->next;
		for (i = 0; (i < noOfModeDirs) && run; i++)
		    {
		    if ((strstr(modeDirPtr->string, "Alias") == NULL) &&
			(strstr(modeDirPtr->string, "DETDATA") == NULL) &&
			(strstr(modeDirPtr->string, "Dictionary") == NULL) &&
			(strstr(modeDirPtr->string, "MISC") == NULL) &&
			(strstr(modeDirPtr->string, "CVS") == NULL))
			{
			strcpy(basePath, insUserDir);
			slxSetLastChar(basePath, '/');
			strcat(basePath, modeDirPtr->string);
                        slxSetLastChar(basePath, '/');
			strcat(basePath, slxCONFIG_FILE_DIR);
			
			if (configType == slxANY_CONFIG_FILE)
			    {
			    for (j = 0; ((j < 3) && (!*numberFound || 
						     !stopFirstOccur)); j++)
				{
				strcpy(fileName, basePath);	
				slxSetLastChar(fileName, '/');
				strcat(fileName, baseNameLoc);
				strcat(fileName, extBuf[j]);
				if (stat(fileName, &statBuf) == 0)
				    {
				    if (slxStrListMerge(fileName, fileList, 
							numberFound,
							error) == FAILURE)
					return FAILURE;
				    }
				}	/* end-for (i = 0; ((i < 4) && ... */
			    }  /* end-if (configType == slxINS_UNKNOWN_ ... */
			else
			    {
			    slxSetLastChar(basePath, '/');
			    strcat(basePath, baseNameLoc);	    
			    if (stat(basePath, &statBuf) == 0)
				{
				if (slxStrListMerge(basePath, fileList, 
						    numberFound,
						    error) == FAILURE)
				    return FAILURE;
				}
			    }
			}  /* end-if ((strrstr("Alias", modeDirPtr-> ... */
		    if (*numberFound && stopFirstOccur)
			run = FALSE;
		    else
			modeDirPtr = modeDirPtr->next;
		    }   /* end-for (i = 0; (i < noOfModeDirs) && run; i++) */
		}   /* end-if (strstr(chrPtr, userDirList->string) ... */
	    if (run)
		userDirPtr = userDirPtr->next;
	    }   /* end-for (userDir = 0; (userDir < noOfUserDirs) && run;... */
	}   /* end-if (globalSearch && ((*numberFound && ... */
  
    /* If still not found, try to locate it according to the a possible path
     * given by the possible  env. variable given as input parameter
     */
    if ((!*numberFound) && (pathEnvVar != NULL))
	{
	if (slxLocateFile(baseNameLoc, pathEnvVar, fileName, 
			  &tmpError) == FAILURE)
	    {
	    if (errIsReason(&tmpError, slxMODULE_NAME, slxERR_LOCATE_FILE_GEN,
			    &errFound, &errDescription) == FAILURE)
		{
		errMergeStack(error, &tmpError, errBOTTOM);
		errResetStack(&tmpError);
		return FAILURE;
		}
	    if (errFound)
		errResetStack(&tmpError);
	    else 
		{
		errMergeStack(error, &tmpError, errBOTTOM);
		errResetStack(&tmpError);
		return FAILURE;
		}
	    }
	else
	    { 
	    if (slxStrListMerge(fileName, fileList, numberFound,
				error) == FAILURE)
		return FAILURE;
	    }
	}

    return SUCCESS;
}

ccsCOMPL_STAT slxSearchTemplateFile(const slxFILENAME         baseName,
				    const slxTEMPLATE_FILES   fileType,
				    vltINT32                  *numberFound,
				    slxSTRING_LIST            *fileList,
				    const vltLOGICAL          globalSearch,
				    const vltLOGICAL          stopFirstOccur,
				    const vltLOGICAL          skipInsUserDir,
				    const char                *pathEnvVar,
				    ccsERROR                  *error)
{
    static slxSTRING_LIST  *modeDirList;  
    static slxSTRING_LIST  *userDirList;
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL      initialized=ccsFALSE;
    static ccsERROR        tmpError;
    char                   *chrPtr;
    struct stat            statBuf;
    ccsSTACK_ELEM          *errDescription;
    slxFILENAME            baseNameLoc;
    slxFILENAME            fileName;  
    slxFILENAME            insUserDir;
    slxFILENAME            basePath;
    slxFILENAME            extension;
    slxSTRING_LIST         *modeDirPtr;
    slxSTRING_LIST         *userDirPtr;
    vltINT32               i;
    vltINT32               userDir;
    vltINT32               noOfModeDirs;
    vltINT32               noOfUserDirs;
    vltLOGICAL             run;
    vltLOGICAL             wildCards;
    vltLOGICAL             errFound;


    if (!initialized)
	{
	if (slxInitStrList(&modeDirList, "HEADER", error) == FAILURE)
	    return FAILURE;
	if (slxInitStrList(&userDirList, "HEADER", error) == FAILURE)
	    return FAILURE;
	errResetStack(&tmpError);
	initialized = TRUE;
	}

    *numberFound = 0;

    /* Check if the name contains wildcards
     */
    if ((strchr(baseName, '*') != NULL) || (strchr(baseName, '?') != NULL))
	wildCards = TRUE;
    else
	wildCards = FALSE;

    /* Resolve path
     */
    if (slxResolvePath(baseName, &chrPtr, error) == FAILURE)
	return FAILURE;
    slxCOPY_MAX_N_BYTES(baseNameLoc, chrPtr, slxFILENAME_LEN, error);
    
    if (fileType == slxTEMPLATE_TSF)
	strcpy(extension, "tsf");
    else if (fileType == slxEXT_TEMPLATE_TSF)
	strcpy(extension, "tsfx");
    else if (fileType == slxTEMPLATE_SEQ)
	strcpy(extension, "seq");
    else
	strcpy(extension, "obd");

    /* If complete path just stat() the file
     */
    if (*baseNameLoc == '/')
	{ 
	if (!wildCards)
	    {
	    if (fileType == slxTEMPLATE_TSF)
		slxCheckExtension(baseNameLoc, ".tsf");
	    else if (fileType == slxEXT_TEMPLATE_TSF)
		slxCheckExtension(baseNameLoc, ".tsfx");
	    else if (fileType == slxTEMPLATE_SEQ)
		slxCheckExtension(baseNameLoc, ".seq");
	    else
		slxCheckExtension(baseNameLoc, ".obd");
	    }
	if (!wildCards)
	    {
	    if (stat(baseNameLoc, &statBuf) == 0)
		{
		if (slxStrListMerge(baseNameLoc, fileList, numberFound,
				    error) == FAILURE)
		    return FAILURE;
		}
	    }
	else
	    {
	    if (_slxMatchFiles(extension, baseNameLoc, stopFirstOccur,
			       fileList, numberFound, error) == FAILURE)
		return FAILURE;
	    }
	return SUCCESS;
	}    /* end-if (*baseNameLoc == '/') */

    /* Search in the $INS_ROOT/<user>/<mode>/TEMPLATES/OBD|SEQ|TSF
     * directories
     */

    /* If we should not skip searching the INS_USER directory, we check now in 
     * the Observation Mode Directories in the current 
     * INS_USER directory, first get the various directories
     */
    if (!skipInsUserDir)
	{
	if (slxBuildInsUserDir(insUserDir, "", error) == FAILURE)
	    return FAILURE;
	if (slxGetDirInfo(insUserDir, modeDirList, &noOfModeDirs, 
			  error) == FAILURE)
	    return FAILURE;
	
	/* Look only in Observation Mode Directories in the INS_USER directory.
	 */
	modeDirPtr = modeDirList->next;
	run = TRUE;
	for (i = 0; (i < noOfModeDirs) && run; i++)
	    {
	    if ((strstr(modeDirPtr->string, "Alias") == NULL) &&
		(strstr(modeDirPtr->string, "DETDATA") == NULL) &&
		(strstr(modeDirPtr->string, "Dictionary") == NULL) &&
		(strstr(modeDirPtr->string, "MISC") == NULL) &&
		(strstr(modeDirPtr->string, "CVS") == NULL))
		{
		strcpy(basePath, insUserDir);
		slxSetLastChar(basePath, '/');
		strcat(basePath, modeDirPtr->string);
		slxSetLastChar(basePath, '/');
		strcat(basePath, slxSEQ_FILE_DIR);
		strcpy(fileName, basePath);		    
		slxSetLastChar(fileName, '/');
		if (fileType == slxTEMPLATE_TSF)
		    strcat(fileName, slxTSF_DIR);
		else if (fileType == slxEXT_TEMPLATE_TSF)
		    strcat(fileName, slxTSFX_DIR);
		else if (fileType == slxTEMPLATE_SEQ)
		    strcat(fileName, slxSEQ_DIR);
		else
		    strcat(fileName, slxOBD_DIR);
		slxSetLastChar(fileName, '/');
		strcat(fileName, baseNameLoc);
		if (fileType == slxTEMPLATE_TSF)
		    slxCheckExtension(fileName, ".tsf");
		else if (fileType == slxEXT_TEMPLATE_TSF)
		    slxCheckExtension(fileName, ".tsfx");
		else if (fileType == slxTEMPLATE_SEQ)
		    slxCheckExtension(fileName, ".seq");
		else
		    slxCheckExtension(fileName, ".obd");
		if (stat(fileName, &statBuf) == 0)
		    {
		    if (slxStrListMerge(fileName, fileList, 
					numberFound, error) == FAILURE)
			return FAILURE;
		    }
		}   /* end-if ((strstr("Alias", modeDirPtr->string) ... */
	    if (*numberFound && stopFirstOccur)
		run = FALSE;
	    else
		modeDirPtr = modeDirPtr->next;
	    }   /* end-for (i = 0; i < noOfModeDirs; i++) */
	}   /* end-if (!skipInsUserDir) */
    
    /* If a global search is requested do it if a file was found and there
     * should be searched for more, or if no files were found so far.
     * Don't check the INS_USER directory - is already checked, or should not
     * be checked.
     */
    if (globalSearch && ((*numberFound && !stopFirstOccur) || !*numberFound))
	{
	if ((chrPtr = getenv(slxINS_ROOT)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_ROOT);
	    return FAILURE;
	    }
	if (slxGetDirInfo(chrPtr, userDirList, &noOfUserDirs,
			  error) == FAILURE)
	    return FAILURE;

	if ((chrPtr = getenv(slxINS_USER)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_USER);
	    return FAILURE;
	    }

	/* Loop over the User Directories
	 */
	userDirPtr = userDirList->next;
	run = TRUE;
	for (userDir = 0; (userDir < noOfUserDirs) && run; userDir++)
	    {
	    if ((strstr(userDirPtr->string, chrPtr) == NULL) &&
		(strstr(userDirPtr->string, "CVS") == NULL))
		{
		/* Get the Observation Mode directories within this User Dir 
		 * and loop over that.
		 */
		if (slxBuildInsUserDir(insUserDir, userDirPtr->string, 
				       error) == FAILURE)
		    return FAILURE;
		if (slxGetDirInfo2(insUserDir, modeDirList, &noOfModeDirs, 
				  error,ccsFALSE) == FAILURE)
		    return FAILURE;
		
		/* Assume here that there are only Observation Mode 
		 * Directories in the INS_USER directory.
		 */
		modeDirPtr = modeDirList->next;
		for (i = 0; (i < noOfModeDirs) && run; i++)
		    {
		    if ((strstr(modeDirPtr->string, "Alias") == NULL) &&
			(strstr(modeDirPtr->string, "DETDATA") == NULL) &&
			(strstr(modeDirPtr->string, "Dictionary") == NULL) &&
			(strstr(modeDirPtr->string, "MISC") == NULL) &&
			(strstr(modeDirPtr->string, "CVS") == NULL))
			{
			strcpy(basePath, insUserDir);
			slxSetLastChar(basePath, '/');
			strcat(basePath, modeDirPtr->string);
                        slxSetLastChar(basePath, '/');
			strcat(basePath, slxSEQ_FILE_DIR);
			strcpy(fileName, basePath);	
			slxSetLastChar(fileName, '/');
			if (fileType == slxTEMPLATE_TSF)
			    strcat(fileName, slxTSF_DIR);
			else if (fileType == slxEXT_TEMPLATE_TSF)
			    strcat(fileName, slxTSFX_DIR);
			else if (fileType == slxTEMPLATE_SEQ)
			    strcat(fileName, slxSEQ_DIR);
			else
			    strcat(fileName, slxOBD_DIR);
			slxSetLastChar(fileName, '/');
			strcat(fileName, baseNameLoc);
			if (fileType == slxTEMPLATE_TSF)
			    slxCheckExtension(fileName, ".tsf");
			else if (fileType == slxEXT_TEMPLATE_TSF)
			    slxCheckExtension(fileName, ".tsfx");
			else if (fileType == slxTEMPLATE_SEQ)
			    slxCheckExtension(fileName, ".seq");
			else
			    slxCheckExtension(fileName, ".obd");
			if (stat(fileName, &statBuf) == 0)
			    {
			    if (slxStrListMerge(fileName, fileList, 
						numberFound,
						error) == FAILURE)
				return FAILURE;
			    }
			}  /* end-if ((strrstr("Alias", modeDirPtr-> ... */
		    if (*numberFound && stopFirstOccur)
			run = FALSE;
		    else
			modeDirPtr = modeDirPtr->next;
		    }   /* end-for (i = 0; (i < noOfModeDirs) && run; i++) */
		}   /* end-if (strstr(chrPtr, userDirList->string) ... */
	    if (run)
		userDirPtr = userDirPtr->next;
	    }   /* end-for (userDir = 0; (userDir < noOfUserDirs) && run;... */
	}   /* end-if (globalSearch && ((*numberFound && ... */
    

    /********************************************************************* 
     * This part searches according to the old scheme where these files
     * where stored in the $INS_ROOT/<user>/<mode>/SEQUENCES
     *********************************************************************
     */
    /* If we should not skip searching the INS_USER directory, we check now in 
     * the Observation Mode Directories in the current 
     * INS_USER directory, first get the various directories
     */
    if (!skipInsUserDir && 
	((*numberFound && !stopFirstOccur) || !*numberFound))
	{
	if (slxBuildInsUserDir(insUserDir, "", error) == FAILURE)
	    return FAILURE;
	if (slxGetDirInfo(insUserDir, modeDirList, &noOfModeDirs, 
			  error) == FAILURE)
	    return FAILURE;
	
	/* Look only in Observation Mode Directories in the INS_USER directory.
	 */
	modeDirPtr = modeDirList->next;
	run = TRUE;
	for (i = 0; (i < noOfModeDirs) && run; i++)
	    {
	    if ((strstr(modeDirPtr->string, "Alias") == NULL) &&
		(strstr(modeDirPtr->string, "DETDATA") == NULL) &&
		(strstr(modeDirPtr->string, "Dictionary") == NULL) &&
		(strstr(modeDirPtr->string, "MISC") == NULL) &&
		(strstr(modeDirPtr->string, "CVS") == NULL))
		{
		strcpy(basePath, insUserDir);
		slxSetLastChar(basePath, '/');
		strcat(basePath, modeDirPtr->string);
		slxSetLastChar(basePath, '/');
		strcat(basePath, "SEQUENCES");
		strcpy(fileName, basePath);		    
		slxSetLastChar(fileName, '/');
		strcat(fileName, baseNameLoc);
		if (fileType == slxTEMPLATE_TSF)
		    slxCheckExtension(fileName, ".tsf");
		else if (fileType == slxEXT_TEMPLATE_TSF)
		    slxCheckExtension(fileName, ".tsfx");
		else if (fileType == slxTEMPLATE_SEQ)
		    slxCheckExtension(fileName, ".seq");
		else
		    slxCheckExtension(fileName, ".obd");
		if (stat(fileName, &statBuf) == 0)
		    {
		    if (slxStrListMerge(fileName, fileList, 
					numberFound, error) == FAILURE)
			return FAILURE;
		    }
		}   /* end-if ((strstr("Alias", modeDirPtr->string) ... */
	    if (*numberFound && stopFirstOccur)
		run = FALSE;
	    else
		modeDirPtr = modeDirPtr->next;
	    }   /* end-for (i = 0; i < noOfModeDirs; i++) */
	}   /* end-if (!skipInsUserDir) */
    
    /* If a global search is requested do it if a file was found and there
     * should be searched for more, or if no files were found so far.
     * Don't check the INS_USER directory - is already checked, or should not
     * be checked.
     */
    if (globalSearch && ((*numberFound && !stopFirstOccur) || !*numberFound))
	{
	if ((chrPtr = getenv(slxINS_ROOT)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_ROOT);
	    return FAILURE;
	    }
	if (slxGetDirInfo(chrPtr, userDirList, &noOfUserDirs,
			  error) == FAILURE)
	    return FAILURE;

	if ((chrPtr = getenv(slxINS_USER)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_USER);
	    return FAILURE;
	    }

	/* Loop over the User Directories
	 */
	userDirPtr = userDirList->next;
	run = TRUE;
	for (userDir = 0; (userDir < noOfUserDirs) && run; userDir++)
	    {
	    if ((strstr(userDirPtr->string, chrPtr) == NULL) &&
		(strstr(userDirPtr->string, "CVS") == NULL))
		{
		/* Get the Observation Mode directories within this User Dir 
		 * and loop over that.
		 */
		if (slxBuildInsUserDir(insUserDir, userDirPtr->string, 
				       error) == FAILURE)
		    return FAILURE;
		if (slxGetDirInfo2(insUserDir, modeDirList, &noOfModeDirs, 
				  error,ccsFALSE) == FAILURE)
		    return FAILURE;
		
		/* Assume here that there are only Observation Mode 
		 * Directories in the INS_USER directory.
		 */
		modeDirPtr = modeDirList->next;
		for (i = 0; (i < noOfModeDirs) && run; i++)
		    {
		    if ((strstr(modeDirPtr->string, "Alias") == NULL) &&
			(strstr(modeDirPtr->string, "DETDATA") == NULL) &&
			(strstr(modeDirPtr->string, "Dictionary") == NULL) &&
			(strstr(modeDirPtr->string, "MISC") == NULL) &&
			(strstr(modeDirPtr->string, "CVS") == NULL))
			{
			strcpy(basePath, insUserDir);
			slxSetLastChar(basePath, '/');
			strcat(basePath, modeDirPtr->string);
                        slxSetLastChar(basePath, '/');
			strcat(basePath, "SEQUENCES");
			strcpy(fileName, basePath);	
			slxSetLastChar(fileName, '/');
			strcat(fileName, baseNameLoc);
			if (fileType == slxTEMPLATE_TSF)
			    slxCheckExtension(fileName, ".tsf");
			else if (fileType == slxEXT_TEMPLATE_TSF)
			    slxCheckExtension(fileName, ".tsfx");
			else if (fileType == slxTEMPLATE_SEQ)
			    slxCheckExtension(fileName, ".seq");
			else
			    slxCheckExtension(fileName, ".obd");
			if (stat(fileName, &statBuf) == 0)
			    {
			    if (slxStrListMerge(fileName, fileList, 
						numberFound,
						error) == FAILURE)
				return FAILURE;
			    }
			}  /* end-if ((strrstr("Alias", modeDirPtr-> ... */
		    if (*numberFound && stopFirstOccur)
			run = FALSE;
		    else
			modeDirPtr = modeDirPtr->next;
		    }   /* end-for (i = 0; (i < noOfModeDirs) && run; i++) */
		}   /* end-if (strstr(chrPtr, userDirList->string) ... */
	    if (run)
		userDirPtr = userDirPtr->next;
	    }   /* end-for (userDir = 0; (userDir < noOfUserDirs) && run;... */
	}   /* end-if (globalSearch && ((*numberFound && ... */
    /********************************************************************* 
     * End of part searching according to old scheme with SEQUENCES 
     * directory
     *********************************************************************
     */
  
    /* If still not found, try to locate it according to the a possible path
     * given by the possible env. variable given as input parameter
     */
    if ((!*numberFound) && (pathEnvVar != NULL))
	{
	if (slxLocateFile(pathEnvVar, baseNameLoc, fileName, 
			  &tmpError) == FAILURE)
	    {
	    if (errIsReason(&tmpError, slxMODULE_NAME, slxERR_LOCATE_FILE_GEN,
			    &errFound, &errDescription) == FAILURE)
		{
		errMergeStack(error, &tmpError, errBOTTOM);
		errResetStack(&tmpError);
		return FAILURE;
		}
	    if (errFound)
		errResetStack(&tmpError);
	    else 
		{
		errMergeStack(error, &tmpError, errBOTTOM);
		errResetStack(&tmpError);
		return FAILURE;
		}
	    }
	else
	    { 
	    if (slxStrListMerge(fileName, fileList, numberFound,
				error) == FAILURE)
		return FAILURE;
	    }
	}

    return SUCCESS;
}


ccsCOMPL_STAT slxSearchFitsFile(const slxFILENAME      baseName,
				vltINT32               *numberFound,
				slxSTRING_LIST         *fileList,
				const vltLOGICAL       globalSearch,
				const vltLOGICAL       stopFirstOccur,
				const vltLOGICAL       skipInsUserDir,
				const char             *pathEnvVar,
				ccsERROR               *error)
{
    static slxSTRING_LIST  *userDirList;
    static slxSTRING_LIST  *tmpFileList;  
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL      initialized=ccsFALSE;
    static ccsERROR        tmpError;
    char                   *chrPtr;
    /*regex_t                preg;*/
    struct stat            statBuf;
    ccsSTACK_ELEM          *errDescription;
    slxFILENAME            baseNameLoc;
    slxFILENAME            fileName;  
    slxFILENAME            tmpFileName;  
    slxFILENAME            basePath;
    slxFILENAME            detDataDir;
    slxFILENAME            detDataFileName;
    slxKEYW_LINE           curUser;
    slxKEYW_LINE           tmpName;
    slxKEYW_LINE           compareKey;
    slxSTRING_LIST         *userDirPtr;
    slxSTRING_LIST         *tmpFilePtr = NULL;
    vltINT32               i;
    vltINT32               userDir;
    vltINT32               noOfUserDirs;
    vltINT32               noOfTmpFiles;
    vltLOGICAL             run;
    vltLOGICAL             wildCards;
    vltLOGICAL             takeAll;
    vltLOGICAL             errFound;


    /* IMPL.: Note that the usage of regexp() is temporarily disabled due
     * to problems on a NOCCS machine. The usage of regexp() and this
     * problem should be investigated and refined!
     */

    if (!initialized)
	{
	if (slxInitStrList(&userDirList, "HEADER", error) == FAILURE)
	    return FAILURE;
	if (slxInitStrList(&tmpFileList, "HEADER", error) == FAILURE)
	    return FAILURE;
	errResetStack(&tmpError);
	initialized = TRUE;
	}

    *numberFound = 0;
    takeAll = FALSE;

    /* Check if the name contains wildcards
     */
    if ((strchr(baseName, '*') != NULL) || (strchr(baseName, '?') != NULL))
	wildCards = TRUE;
    else
	wildCards = FALSE;

    /* Check if the filename contains "~/..." or "$<env var>", and resolved
     * this if this is the case
     */
    if ((*baseName == '~') || (*baseName == '$') || (*baseName == '/'))
	{
	if (slxResolvePath(baseName, &chrPtr, error) == FAILURE)
	    return FAILURE;
	slxCOPY_MAX_N_BYTES(baseNameLoc, chrPtr, slxFILENAME_LEN, error);
	}
    else
	{
	/* If the baseName is only an asterix we have to put an extra
	 */
	if (wildCards)
	    {	    
	    if (slxGetFileName(baseName, tmpName, error) == FAILURE)
		return FAILURE;
	    slxYankExtension(tmpName, ".fits");
	    if (strcmp(tmpName, "*") == 0)
		{
		takeAll = TRUE;
		strcat(tmpName, "*");
		}
	    slxCOPY_MAX_N_BYTES(baseNameLoc,tmpName,sizeof(slxFILENAME),error);
	    }
	else
	    slxCOPY_MAX_N_BYTES(baseNameLoc, baseName, sizeof(slxFILENAME),
				error);
	}

    if (!wildCards)
	slxCheckExtension(baseNameLoc, ".fits");

    /* If complete path just stat() the file:
     */
    if (*baseNameLoc == '/')
	{                                        
	/* The filename includes the path
	 */	     
	if (!wildCards)
	    {
	    if (stat(baseNameLoc, &statBuf) == 0)
		{
		if (slxStrListMerge(baseNameLoc, fileList, numberFound,
				    error) == FAILURE)
		    return FAILURE;
		}
	    }
	else
	    {
	    /* Get all the files in the directory and make a regexp()
	     * for each of them
	     */
	    slxGetPath(baseNameLoc, basePath, error);
	    if (slxGetDirInfo(basePath, tmpFileList, &noOfTmpFiles, 
			      error) == FAILURE)
		return FAILURE;
	    tmpFilePtr = tmpFileList->next;
	    if (slxGetLastChar(basePath) != '/')
		strcat(basePath, "/");
	    if (slxGetFileName(baseNameLoc, compareKey, error) == FAILURE)
		return FAILURE;
	    if (slxGetLastChar(compareKey) == '*')
		*(compareKey + (strlen(compareKey) - 1)) = '\0';
	    for (i = 0; (i < noOfTmpFiles) && 
		     ((!*numberFound || !stopFirstOccur)); i++)
		{
		/* Check that the extension is correct of the file
		 */
		if (slxCheckExtension2(tmpFilePtr->string, "fits"))
		    {
		    if (((strstr(tmpFilePtr->string, compareKey) == 
			  tmpFilePtr->string) || takeAll))
			{
			sprintf(tmpFileName, "%s%s", 
				basePath,tmpFilePtr->string);
			if (slxStrListMerge(tmpFileName, fileList, 
					    numberFound, error) == FAILURE)
			    return FAILURE;
			}
		    }
		tmpFilePtr = tmpFilePtr->next;
		}   /* end-for (i = 0; (i < noOfTmpFiles) && ... */
	    }	
	return SUCCESS;
	}    /* end-if (*baseNameLoc == '/') */
    
    
    /* If we should not skip searching the INS_USER directory, we check now in 
     * the DETDATA directory in the current INS_USER directory
     */
    if (!skipInsUserDir)
	{
	/* If no wildcards are specified we stat the file in the user directory
	 */
	if (!wildCards)
	    {
	    if (slxBuildInsFileName("$INS_USER", "DETDATA", "", 
				    baseNameLoc, detDataFileName, 
				    error) == FAILURE)
		return FAILURE;
	    if (strstr(detDataFileName, ".fits") == NULL)
		strcat(detDataFileName, ".fits");
	    if (stat(detDataFileName, &statBuf) == 0)
		{
		if (slxStrListMerge(detDataFileName, fileList, numberFound,
				    error) == FAILURE)
		    return FAILURE;
		}
	    }
	else
	    {
	    /* We generate a list of all files in the USER/DETDATA dir and
	     * run a regexp() matching on these
	     */
	    if (slxBuildInsFileName("$INS_USER", "DETDATA", "", "", detDataDir,
				    error) == FAILURE)
		return FAILURE;

	    noOfTmpFiles = 0;
	    if (slxGetDirInfo(detDataDir, tmpFileList, &noOfTmpFiles, 
			      error) == FAILURE)
		return FAILURE;	
	    tmpFilePtr = tmpFileList->next; 
	    /* IMPL.: regexp() disabled
	     * if (regcomp(&preg, baseNameLoc, REG_NOSUB) != 0)
	     * {
	     * errSysAdd(error, slxMODULE_NAME, slxERR_SYSTEM_CALL, 
	     * __FILE__, NULL, modUNIX, "regcomp()",
	     * "Cannot filter on filenames");
	     * return FAILURE;
	     * }
	     */
	    slxCOPY_MAX_N_BYTES(compareKey, baseNameLoc, sizeof(slxKEYW_LINE),
				error);
	    *(compareKey + (strlen(compareKey) - 1)) = '\0';
	    for (i = 0; (i < noOfTmpFiles) && 
		     ((!*numberFound || !stopFirstOccur)); i++)
		{
		/* IMPL.: regexp() disabled
		 * if (regexec(&preg, tmpFilePtr->string, (size_t)0, 
		 * NULL, 0) == 0)
		 */
		if (slxCheckExtension2(tmpFilePtr->string, "fits"))
		    {
		    if ((strstr(tmpFilePtr->string, compareKey) == 
			 tmpFilePtr->string) || takeAll)
			{
			sprintf(tmpFileName, "%s/%s", detDataDir, 
				tmpFilePtr->string);
			if (slxStrListMerge(tmpFileName, fileList, 
					    numberFound, error) == FAILURE)
			    {
			    /* IMPL.: regexp() disabled
			     * regfree(&preg);
			     */
			    return FAILURE;
			    }
			}
		    }
		tmpFilePtr = tmpFilePtr->next;
		}   /* end-for (i = 0; (i < noOfTmpFiles) && ... */
	    /* IMPL.: regexp() disabled
	     * regfree(&preg);
	     */
	    }
	}   /* end-if (!skipInsUserDir) */
    
    /* If a global search is requested do it if a file was found and there
     * should be searched for more, or if no files were found so far.
     * Don't check the INS_USER directory - is already checked, or should not
     * be checked.
     */
    if (globalSearch && ((*numberFound && !stopFirstOccur) || !*numberFound))
	{
	if ((chrPtr = getenv(slxINS_ROOT)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_ROOT);
	    return FAILURE;
	    }
	if (slxGetDirInfo(chrPtr, userDirList, &noOfUserDirs,
			  error) == FAILURE)
	    return FAILURE;
	if ((chrPtr = getenv(slxINS_USER)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ENV_VAR_NOT_DEF, __FILE__, 
		   NULL, slxINS_USER);
	    return FAILURE;
	    }
	slxCOPY_MAX_N_BYTES(curUser, chrPtr, sizeof(slxKEYW_LINE), error);

	/* Loop over the User Directories
	 */
	userDirPtr = userDirList->next;
	run = TRUE;
	for (userDir = 0; (userDir < noOfUserDirs) && run; userDir++)
	    {
	    /* We don't search the Current INS User Dir since this is
	     * already done
	     */
	    if ((strstr(userDirPtr->string, curUser) == NULL) &&
		(strstr(userDirPtr->string, "CVS") == NULL))
		{
		/* If no wildcards are specified we stat the file in the 
		 * user directory
		 */
		if (!wildCards)
		    {
		    if (slxBuildInsFileName(userDirPtr->string, "DETDATA",
					    "", "baseNameLoc",detDataFileName, 
					    error) == FAILURE)
			return FAILURE;
		    if (strstr(detDataFileName, ".fits") == NULL)
			strcat(detDataFileName, ".fits");
		    if (stat(detDataFileName, &statBuf) == 0)
			{
			if (slxStrListMerge(detDataFileName, fileList, 
					    numberFound, error) == FAILURE)
			    return FAILURE;
			}
		    }
		else
		    {
		    /* We generate a list of all files in the 
		     * USER/DETDATA dir and run a regexp() matching on these
		     */
		    if (slxBuildInsFileName(userDirPtr->string, "DETDATA", "",
					    "", detDataDir, error) == FAILURE)
			return FAILURE;
		    noOfTmpFiles = 0;
		    if (slxGetDirInfo(detDataDir, tmpFileList, &noOfTmpFiles, 
				      error) == FAILURE)
			return FAILURE;	
		    tmpFilePtr = tmpFileList->next;
		    /* IMPL.: regexp() disabled
		     * if (regcomp(&preg, baseNameLoc, REG_NOSUB) != 0)
		     * {
		     * errSysAdd(error, slxMODULE_NAME,
		     * slxERR_SYSTEM_CALL, 
		     * __FILE__, NULL, modUNIX, "regcomp()",
		     * "Cannot filter on filenames");
		     * return FAILURE;
		     * }
		     */
		    slxCOPY_MAX_N_BYTES(compareKey, baseNameLoc, 
					sizeof(slxKEYW_LINE), error);
		    *(compareKey + (strlen(compareKey) - 1)) = '\0';
		    for (i = 0; (i < noOfTmpFiles) && 
			     ((!*numberFound || !stopFirstOccur)); i++)
			{
			/* IMPL.: regexp() disabled
			 * if (regexec(&preg, tmpFilePtr->string, (size_t)0, 
			 *     NULL, 0) == 0)
			 */
			if (slxCheckExtension2(tmpFilePtr->string, "fits"))
			    {
			    if ((strstr(tmpFilePtr->string, compareKey) ==
				 tmpFilePtr->string) || takeAll)
				{
				sprintf(tmpFileName, "%s/%s", detDataDir, 
					tmpFilePtr->string);
				if (slxStrListMerge(tmpFileName, fileList, 
						    numberFound, 
						    error) == FAILURE)
				    {
				    /* IMPL.: regexp() disabled
				     * regfree(&preg);
				     */
				    return FAILURE;
				    }
				}
			    }
			tmpFilePtr = tmpFilePtr->next;
			}   /* end-for (i = 0; (i < noOfTmpFiles) && ... */
		    /* IMPL.: regexp() disabled
		     * regfree(&preg);
		     */
		    }
		}	
	    if (run)
		userDirPtr = userDirPtr->next;
	    }   /* end-for (userDir = 0; (userDir < noOfUserDirs) ... */
	}   /* end-if (globalSearch && ((*numberFound && ... */
  
    /* If still not found, try to locate it according to the a possible path
     * given by the possible  env. variable given as input parameter
     */
    if ((!*numberFound) && (pathEnvVar != NULL))
	{
	if (slxLocateFile(baseNameLoc, pathEnvVar, fileName, 
			  &tmpError) == FAILURE)
	    {
	    if (errIsReason(&tmpError, slxMODULE_NAME, slxERR_LOCATE_FILE_GEN,
			    &errFound, &errDescription) == FAILURE)
		{
		errMergeStack(error, &tmpError, errBOTTOM);
		errResetStack(&tmpError);
		return FAILURE;
		}
	    if (errFound)
		errResetStack(&tmpError);
	    else 
		{
		errMergeStack(error, &tmpError, errBOTTOM);
		errResetStack(&tmpError);
		return FAILURE;
		}
	    }
	else
	    {
	    if (slxCheckExtension2(tmpFilePtr->string, "fits")) 
		{ 
		if (slxStrListMerge(fileName, fileList, numberFound,
				    error) == FAILURE)
		    return FAILURE;
		}
	    }
	}

    return SUCCESS;
}


ccsCOMPL_STAT slxSearchDictionary(const slxDICTIONARY  dictionary,
				  slxFILENAME          pafDicFullName,
				  ccsERROR             *error)
{
    static ccsERROR   tmpError;
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL initialized=ccsFALSE;
    ccsCOMPL_STAT     stat = SUCCESS;
    slxFILENAME       dicName, dicFileName;
    vltLOGICAL        errorOccurred;    

    if (!initialized)
	{
	errResetStack(&tmpError);
	initialized = TRUE;
	}

    if (strstr(dictionary, "-DIC.") != NULL)
	sprintf(dicName, "%s", dictionary);
    else
	sprintf(dicName, "ESO-VLT-DIC.%s", dictionary);
    errorOccurred = FALSE;

    /* Try in:
     *
     *    o INS_ROOT
     *    o INTROOT/include
     *    o INTROOT/config
     *    o VLTROOT/include
     *    o VLTROOT/config
     */
    sprintf(dicFileName, "$INS_ROOT/SYSTEM/Dictionary/%s", dicName);
    stat = slxLocateFile("", dicFileName, pafDicFullName, &tmpError);
    if (stat == FAILURE)
	{
	errorOccurred = TRUE;
	sprintf(dicFileName, "$INTROOT/include/%s", dicName);
	stat = slxLocateFile("", dicFileName, pafDicFullName, &tmpError);    
	}
    if (stat == FAILURE)
	{
	errorOccurred = TRUE;
	sprintf(dicFileName, "$INTROOT/config/%s", dicName);
	stat = slxLocateFile("", dicFileName, pafDicFullName, &tmpError);    
	}
    if (stat == FAILURE)
	{
	errorOccurred = TRUE;
	sprintf(dicFileName, "$VLTROOT/include/%s", dicName);
	stat = slxLocateFile("", dicFileName, pafDicFullName, &tmpError);
	}    
    if (stat == FAILURE)
	{
	errorOccurred = TRUE;
	sprintf(dicFileName, "$VLTROOT/config/%s", dicName);
	stat = slxLocateFile("", dicFileName, pafDicFullName, &tmpError);
	}    
    if (stat == FAILURE)
	{
	errorOccurred = TRUE;
	sprintf(dicFileName, "%s", dicName);
	stat = slxLocateFile(slxINS_SETUPPATH_NAME, dicFileName,
			     pafDicFullName, &tmpError);
	}
    if (stat == FAILURE)
	{	
	*pafDicFullName = '\0';
	errMergeStack(error, &tmpError, errBOTTOM);
	errAdd(error, slxMODULE_NAME, slxERR_LOCATE_DIC, __FILE__, NULL,
	       dictionary);
	errResetStack(&tmpError);
	}
    else if (errorOccurred)
	errResetStack(&tmpError);

    return stat;
}

/*
 *****************************************************************************
 */

/*___oOo___*/

