/******************************************************************************
* E.S.O. - VLT project
*
* slxShortFITSToAlias.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  18/09/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxShortFITSToAlias - converts Short-FITS keyword to its alias
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxShortFITSToAlias(const slxKEYW_LINE  shortFITS,
*                                     const vltLOGICAL    loadTable,
*                                     const slxFILENAME   tableFileName,
*                                     slxKEYW_LINE        alias,
*                                     ccsERROR            *error)
*
* 
*   DESCRIPTION
*   The function converts a Short-FITS keyword into the alias representation.
*   If the alias was not defined, the first character of the buffer pointed
*   to be "alias" will be a NULL character.
*
*   The first time the function is used, and whenever the Alias Conversion
*   Table is changed, the function must be called with the "loadTable" 
*   parameter set to TRUE, and the name of the table to load must be 
*   specified.
*
* 
*   shortFITS      IN   Points to buffer containing the Short-FITS keyword.
*
*   loadTable      IN   Indicates if the function should reinitialize its 
*                       Alias Conversion Table. If this is the case, TRUE
*                       must be the value, otherwise FALSE.
*
*   tableFileName  IN   Filename of Alias Conversion Table to be loaded. If
*                       "loadTable" is FALSE, this parameter is ignored.
*
*   alias          OUT  The alias corresponding to the Short-FITS keyword.
*
*   error          OUT  Indicates a possible error that occurred.
*
*
*
*   RETURN VALUES 
*   SUCCESS  If the operation was successfull.
*   FAILURE  If the operation failed. The following error codes can be 
*            returned:
*
*            slxERR_OPEN_ALIAS_TABLE      Couldn't open the alias conversion
*                                         table.
*
*            slxERR_CRE_ALIAS_HASH_TABLE  Couldn't create the alias conversion
*                                         table.
*
*            slxERR_ALIAS_NOT_DEF         The alias referenced is not defined
*                                         in the table.
*
*
*   CAUTIONS 
*   The function keeps track of a local hash table for the conversions. It is
*   important that it is initialized with the correct alias definitions.
*
*   The function is private to each application, which then has to initialize
*   it itself.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxShortFITSToAlias.c,v 1.145 2005/07/27 09:28:47 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/*  System Headers
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Hash table used for the conversion
 */
static slxALIAS_HASH    *hashTable[slxMAX_NO_OF_ALIASES];

/* Function prototyping
 */
unsigned slxHash2(const char *string);
slxALIAS_HASH *slxLookUp2(const char *string);
slxALIAS_HASH *slxInstall2(const  char *shortFITS, const char *alias);

/* Function definition
 */
ccsCOMPL_STAT slxShortFITSToAlias(const slxKEYW_LINE  shortFITS,
                                  const vltLOGICAL    loadTable,
                                  const slxFILENAME   tableFileName,
                                  slxKEYW_LINE        alias,
                                  ccsERROR            *error)
/* The function is based on the algorithm described in 
 * "The C (ANSI C) Programming Language", Brian W. Kernighan, and
 * Dennis M. Ritchie.
 */
{
    char                    inBuf[256];
    char                    *comPathPtr;
    slxFILENAME             aliasTableFileName;
    slxKEYW_LINE            inShortFITS;
    slxKEYW_LINE            inAlias;
    FILE                    *filePtr;    
    static vltUINT8         calledBefore;
    slxALIAS_HASH           *hashTablePtr;
    slxALIAS_HASH           *hashTableDel;
    vltUINT16               n;


    if (!calledBefore || (loadTable == TRUE))
        {        
        /* Delete a possible existing table and build a new one
         */        
        for (n = 0; n < slxMAX_NO_OF_ALIASES; n++)
            {
            if (hashTable[n] != NULL)
                {
                hashTablePtr = hashTable[n];
                do
                    {
                    hashTableDel = hashTablePtr;
                    hashTablePtr = hashTablePtr->next;
                    free(hashTableDel->alias); 
                    free(hashTableDel->shortFITS); 
                    free(hashTableDel); 
                    }
                while (hashTablePtr != NULL);
                hashTable[n] = NULL;
                }
            }    /* end-for (n = 0; n < slxMAX_NO_OF_ALIASES; n++) */

	/* Build the new hash table. Alias Conversion tables are located in
	 * $INS_ROOT/SYSTEM/Alias
         */
	if (slxResolvePath(tableFileName, &comPathPtr, error) == FAILURE)
	    return FAILURE;
	if ((*comPathPtr != '/') && (*comPathPtr != '.'))
	    {
	    if (slxBuildInsFileName("SYSTEM", "Alias", "", tableFileName,
				    aliasTableFileName, error) == FAILURE)
		return FAILURE;
	    }
	else
	    {
	    slxCOPY_MAX_N_BYTES(aliasTableFileName, comPathPtr, 
				slxFILENAME_LEN, error);
	    }
	if (strstr(aliasTableFileName, slxALIAS_FILE_EXT) == NULL)
	    strcat(aliasTableFileName, slxALIAS_FILE_EXT);
        if ((filePtr = fopen(aliasTableFileName, "r")) == NULL)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_ALIAS_TABLE, 
                      __FILE__, NULL, modUNIX, aliasTableFileName);
            return FAILURE;
            }

        while (fgets(inBuf, (int)256, filePtr) != NULL)
	    {
            if ((inBuf[0] != '#') && (strlen(inBuf) > 5) &&
                (inBuf[0] != ' ') && (inBuf[0] != '\t'))
                {
                if (sscanf(inBuf, "%s %s", inShortFITS, inAlias) == 2)
                    {
                    if (slxInstall2(inShortFITS, inAlias) == NULL)
                        {
                        errAdd(error, slxMODULE_NAME, 
                               slxERR_CRE_ALIAS_HASH_TABLE, __FILE__, NULL,
                               aliasTableFileName); 
                        fclose(filePtr);
                        return FAILURE;
                        }
                    }
                }
            }    /* end-while (fgets(inBuf(inBuf, (int)256, ... */
        fclose(filePtr);
        calledBefore = 1;
        }    /* end-if (!calledBefore || (loadTable == TRUE)) */

    /* Look up the alias - if the function did not just create the hash table
     */
    if (loadTable == FALSE)
        {
        if ((hashTablePtr = slxLookUp2(shortFITS)) == NULL)
            {
            errAdd(error, slxMODULE_NAME, slxERR_ALIAS_NOT_DEF, __FILE__, 
                   NULL, shortFITS);  
            return FAILURE;
            }
        strcpy(alias, hashTablePtr->alias);
        }

    return SUCCESS;
}

/* Generate hash value for the string
 */
unsigned slxHash2(const char *string)
{
    unsigned hashVal;

    for (hashVal = 0; *string != '\0'; string++)
        hashVal = *string + 31 * hashVal;
 
    return (hashVal % slxMAX_NO_OF_ALIASES);
}

/* Searchs for string in the hash table
 */
slxALIAS_HASH *slxLookUp2(const char *string)
{
    slxALIAS_HASH  *np;

    for (np = hashTable[slxHash2(string)]; np != NULL; np = np->next)
        if (strcmp(string, np->shortFITS) == 0)
            return np;    /* Found */

    return NULL;    /* Not found */
}

/* Install write new name to the hash table
 */
slxALIAS_HASH *slxInstall2(const char *shortFITS, 
			   const char *alias)
{
    slxALIAS_HASH   *np;
    unsigned        hashVal;

    if ((np = slxLookUp2(shortFITS)) == NULL)
        {
        np = (slxALIAS_HASH *)malloc(sizeof(slxALIAS_HASH));
        if (np == NULL || (np->shortFITS = slxStrDup(shortFITS)) == NULL)
            return NULL;
        hashVal = slxHash2(shortFITS);
        np->next = hashTable[hashVal];
        hashTable[hashVal] = np;
        }
    else
        free((void *)np->alias);

    if ((np->alias = (char *)slxStrDup(alias)) == NULL)
        return NULL;

    return np;
}

/*
 *****************************************************************************
 */




/*___oOo___*/
