/******************************************************************************
* E.S.O. - VLT project
*
* slxSplitFITS.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  07/09/94  created
*/

/************************************************************************
*   NAME
*   slxSplitFITS - splits a FITS line in its components
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxSplitFITS(const slxKEYW_LINE  fitsLine, 
*                              slxCATEGORY         category,
*                              slxSUBSYSTEM        subSystem,
*                              slxPARAMETER        parameter,
*                              slxVALUE            value,
*                              slxCOMMENT          comment,
*                              ccsERROR            *error)
* 
* 
*   DESCRIPTION
*   NOTE: A new function slxSplitFits2() is now supported providing
*   providing two sub-system keywords. This new function should be used.
*
*   The function splits a FITS line in its components and return these in the
*   output parameters. The "HIERACH ESO " is not returned.
*
*   All output buffers must be allocated by the calling application. They 
*   should be at least one byte longer than the maximum length of the string to
*   fill in, i.e. for the category 4 bytes.
*
*   If an output buffer is not defined, the first element in this will be 
*   a null charcter ('\0'). E.g. if a standard FITS line is parted, the 
*   "category" and "subSystem" buffers will start with a '\0'.
* 
*   
*   fitsLine    IN   Pointer to buffer containing the FITS line. MUST be
*                    terminated with a null character.
*
*   category    OUT  Pointer to buffer which will contain the possible
*                    category.
*
*   subSystem   OUT  Pointer to buffer which will contain the possible 
*                    subsystem.
*
*   parameter   OUT  Pointer to buffer which will contain the parameter.
*
*   value       OUT  Pointer to buffer which will contain the value 
*                    (as a string). If the value is a string, the single
*                    quotes are returned. A logical value is not returned with
*                    single quotes.
*
*   comment     OUT  Pointer to buffer which will contain the possible 
*                    commment.
*
*
*   RETURN VALUES
*   SUCCESS   In case the operation was succcessful.
*   FAILURE   In case the operation failed. The function can return the 
*             following error code:
*
*             slxERR_FORMAT_KEYWORD   The format of the keyword line is wrong.
*
*
*   CAUTIONS 
*   The function does not check for the syntax of the line, so if it is not 
*   correctly formatted, the result in the output buffers may be unpredictable.
*
*
*   SEE ALSO
*   slxCheckSyntaxLine()
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxSplitFITS.c,v 1.145 2005/07/27 09:30:40 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxSplitFITS(const slxKEYW_LINE   fitsLine, 
                           slxCATEGORY          category,
                           slxSUBSYSTEM         subSystem,
                           slxPARAMETER         parameter,
                           slxVALUE             value,
                           slxCOMMENT           comment,
                           ccsERROR             *error)
{
    char       tempBuf[16];
    char       *chrPtr;
    vltUINT8   inAString;
    vltUINT16  strLen;
    vltUINT16  index;
    vltUINT16  index2;
    vltUINT16  fitsLen;


    /*
     * Check if just one comment line
     * ("COMMENT / <comment>\0")
     */

    if (strstr(fitsLine, "COMMENT ") != NULL)
        {
        *category = (char)'\0';
        *subSystem = (char)'\0';
        *parameter = (char)'\0';
        *value = (char)'\0';
        index = strlen("COMMENT ");
        strncpy(comment, (fitsLine + index), (size_t)(strlen(fitsLine) - 2));
        return SUCCESS;
        }

    fitsLen = strlen(fitsLine);

    /*
     * Check if HIERARCHical keyword or standard FITS
     */

    if (strstr(fitsLine, "HIERARCH") == NULL)
        {
 
        /*
         * It's standard FITS: "<parameter>[ ]= <value> [/ <commment>]\0"
         */        

        *category = (char)'\0';
        *subSystem = (char)'\0';

        index = 0;
  
        /*
         * Copy out parameter - we do all this manipulation manually
         */

        while ((index < fitsLen) && (*(fitsLine + index) != '=') &&
               (*(fitsLine + index) != ' '))
            {
            *(parameter + index) = *(fitsLine + index);
            index++;
            }
        *(parameter + index) = '\0';
        }    /* end-if (strstr(fitLine, "HIERARCH") == NULL) */
    else
        {

         /*
          * It's hierarchical FITS - first skip "HIERARCH ESO "
          */

        index = (vltUINT16)strlen("HIERARCH ESO ");

        /*
         * Copy out category - after this the index indexes the first 
         * character of the subsystem or parameter
         */

        if ((chrPtr = strchr((char *)(fitsLine + index), ' ')) == NULL)
            {
            errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, __FILE__, "",
                   fitsLine);
            return FAILURE;
            }
        strLen = (chrPtr - (fitsLine + index));
        strncpy(category, (fitsLine + index), (size_t)strLen);
        *(category + strLen) = (char)'\0';
        index += (strLen + 1);

        /*
         * The next string might be the subsystem or the parameter -
         * determine which
         */

        memset(tempBuf, 0, (size_t)16);
        index2 = 0;
        while ((index < fitsLen) && (*(fitsLine + index) != ' ') &&
               (*(fitsLine + index) != '='))
            {
            tempBuf[index2] = *(fitsLine + index);
            index++; index2++;
            }

        /*
         * Check if there is something fundamentally wrong with the 
         * FITS line given in
         */

        if (index >= fitsLen)
            {
            errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, __FILE__, "",
                   fitsLine);
            return FAILURE;
            }

        index++;
        if ((*(fitsLine + index) != ' ') && (*(fitsLine + index) != '='))
            {
            
            /*
             * It's a subsystem copy into "subSystem" buffer + get the 
             * parameter
             */

            strcpy(subSystem, tempBuf);
            index2 = 0;
            while ((index < fitsLen) && (*(fitsLine + index) != '=') &&
                   (*(fitsLine + index) != ' '))
                {
                *(parameter + index2) = *(fitsLine + index);
                index++; index2++;
                }
            *(parameter + index2) = '\0';
            }
        else
            {
            strcpy(parameter, tempBuf);
            *subSystem = '\0';
            }
        }    /* end-else (hierarchical FITS) */

    /*
     * Skip some possible blanks and the equal sign
     */

    while ((index < fitsLen) && ((*(fitsLine + index) == '=') ||
           (*(fitsLine + index) == ' ')))
        index++;

    if (index >= fitsLen)
        {
        errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, __FILE__, NULL,
               fitsLine);
        return FAILURE;
        }

    index2 = 0;
    inAString = 0;
    if (*(fitsLine + index) == '\'')
        inAString = 1;
    *(value + index2++) = *(fitsLine + index++);
    while (((*(fitsLine + index) != ' ') ||
           ((*(fitsLine + index) == ' ') && inAString)) && 
           (index < fitsLen))
        {
        if (*(fitsLine + index) == '\'')
            inAString = 0;

        *(value + index2++) = *(fitsLine + index++);
        }
     *(value + index2) = '\0';

    if (index >= fitsLen)
        {
        errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, __FILE__, NULL,
               fitsLine);
        return FAILURE;
        }

    /*
     * Skip blanks and a '/' for the possible comment
     */

    while ((index < fitsLen) && ((*(fitsLine + index) == ' ') ||
           (*(fitsLine + index) == '/')))
        index++;
        
    /*
     * Copy out a possible comment
     */

    if (index < (fitsLen - 1))
        strncpy(comment, (fitsLine + index), (size_t)(fitsLen - index + 1));
    else
        *comment = '\0';
          
    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/

