/******************************************************************************
* E.S.O. - VLT project
*
* slxStoreLine.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  27/07/94  Created 1. version
* jknudstr  30/05/95  Fixed bug when a keyword was a subset of another
*                     -> it overwrote the other if placed before
*/

/************************************************************************
*   NAME
*   slxStoreLine - stores a set-up keyword line in a file.
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxStoreLine(const slxKEYW_LINE  keywordLine, 
*                              const slxFILENAME   fileName,
*                              ccsERROR            *error)
*
*
*   DESCRIPTION
*   The function searches in the specified file for a set-up keyword
*   identical to the one in the buffer pointed to by "keywordLine". If it 
*   finds this line, it replaces the old with the new. Note that this 
*   includes overwriting the old comment which might be there with the new one.
*
*   If the keyword (combination) was not found, the new line is appended in the
*   file.
*
*   The function works on kyewords in the Short-FITS format.
*
* 
*   keywordLine	IN	The keyword command line which shall be stored in the
*			set-up file. Must be in the Short-FITS format.
* 
*   fileName	IN	Pointer to the set-up file where the new keyword line
*                       shall be stored.
*
*   error       OUT     CCS error structure. Indicates a possible error 
*                       occurring.
*    
*
*   RETURN VALUES 
*   SUCCESS	If the operation was successful.
*   FAILURE	If something went wrong. The following error codes can be
*               returned:
*
*               slxERR_INS_SETUPPATH  Could get the value of the environment
*                                     variable INS_SETUPPATH.
*
*               slxERR_OPEN_FILE      Could not open file.
*
*               slxERR_FORMAT_KEYWORD Format of keyword is wrong.
*
*               slxERR_WRITE_FILE     Error in writing in file.
*
*
*               The function makes calls to slxLocateSetupFile() and
*               slxGetPath() and may return errors generated by these functions.
*
*
*   CAUTIONS
*   The subroutine assumes that the set-up file is syntactically correct.
*   To ensure this, run possibly "slxCheckSyntaxSetUp" on the set-up first.
*
*
*   SEE ALSO
*   slxDetectFormat()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxStoreLine.c,v 1.145 2005/07/27 09:32:39 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>
#include <unistd.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxStoreLine(const slxKEYW_LINE  keywordLine, 
                           const slxFILENAME   fileName, 
                           ccsERROR            *error)
{
    FILE         *filePtr;
    FILE         *tempFilePtr;
    char         refFitsLine[81];
    char         newLine = '\n';
    char         *readStatus;   
    char         *tempFile = "slxStoreLineTempFile";
    char         completeFileName[256];
    char         completeTempFileName[256];
    char         path[256];
    char         tempBuf[256];
    char         *bufPtr;
    slxKEYW_LINE fitsLine;
    vltUINT16    lineCount = 0;
    vltUINT16    refLength;
    vltUINT16    n;
    vltLOGICAL   lineFound = FALSE;


    /* Open file (search according to INS_SETUPPATH - slxOpenFile())
     */
    if (slxLocateSetupFile(fileName, completeFileName, error) != SUCCESS)
        return FAILURE;
        
    if (slxGetPath(completeFileName, path, error) != SUCCESS)
        return FAILURE;

    strcpy(completeTempFileName, path);
    strcat(completeTempFileName, tempFile);

    if ((filePtr = fopen(completeFileName, "r+")) == NULL)
        {   
        errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, NULL,
                  modUNIX, completeFileName);  
        return FAILURE;
        }

    /* Check format of set-up file - this is only a superficial check
     */
    if ((strncmp((char *)keywordLine, slxFITS_ESO_PREFIX, 
                strlen(slxFITS_ESO_PREFIX)) == 0) ||
        (strncmp((char *)keywordLine, slxFITS_ESO_PREFIX, 
                strlen("COMMENT")) == 0))
        {
        errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, __FILE__, NULL,
               keywordLine);  
        return FAILURE;
        }

    /* Get the keyword part of the new FITS line ("<cat>.<sub>.<par>")
     */
    bufPtr = (char *)keywordLine;
    if (slxGetNextField(bufPtr, slxKEYW_LINE_LEN, refFitsLine, &bufPtr,
			error) == FAILURE)
	return FAILURE;    
    strcat(refFitsLine, " ");
    refLength = strlen(refFitsLine);
    
    /* Search after FITS line in the set-up file.
     *
     * There is searched only according to the following keywords:
     * "<cat>.[<sub>.]<par>".
     */  
    do
        {
        readStatus = fgets(fitsLine,
			   (slxMAX_FITS_LI_LEN + slxEXTRA_FITS_LI_LEN),
                           filePtr);        
        lineCount++;        

        if (strncmp(fitsLine, refFitsLine, (size_t)refLength) == 0)
            lineFound = TRUE;
        }
    while ((readStatus != NULL) && (lineFound == FALSE));

    /* If the line was found:
     * 
     * 1. Create a temporary file.
     * 2. Copy over the lines until the line to be replaced.
     * 3. Write the new line.
     * 4. Copy the rest of the lines.
     * 5. Rename the temporary file to the original file name.
     *
     * If the line wasn't found, just append it in the original set-up file.
     */
    if (lineFound == TRUE)
        {    
        if ((tempFilePtr = fopen(completeTempFileName, "w")) == NULL)
            {
            fclose(filePtr);
            errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, NULL, 
                      modUNIX, completeTempFileName);  
            return FAILURE;
            }
     
        rewind(filePtr);
        lineCount--;
        for (n = 0; n < lineCount; n++)
            { 
            fgets(fitsLine, (slxMAX_FITS_LI_LEN + slxEXTRA_FITS_LI_LEN),
                  filePtr);
               
            if (fputs(fitsLine, tempFilePtr) == EOF)
	        {
                errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                          NULL, modUNIX, completeTempFileName); 
                fclose(filePtr);
                fclose(tempFilePtr);
                return FAILURE;
                }
            }     /* for n < lineCount */

        strcpy(tempBuf, keywordLine);
        if (*(tempBuf + strlen(tempBuf) - 1) != '\n')
            strcat(tempBuf, "\n");
        
        if (fputs(tempBuf, tempFilePtr) == EOF)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                      NULL, modUNIX, completeTempFileName);
            fclose(filePtr);
            fclose(tempFilePtr);
            return FAILURE;
            }
     
        fgets(fitsLine, (slxMAX_FITS_LI_LEN + slxEXTRA_FITS_LI_LEN), filePtr);
        
        while (fgets(fitsLine, (slxMAX_FITS_LI_LEN + slxEXTRA_FITS_LI_LEN), 
               filePtr) != NULL)
            {
            if (fputs(fitsLine, tempFilePtr) == EOF)
                {
                errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                          NULL, modUNIX, completeTempFileName);
                fclose(filePtr);
                fclose(tempFilePtr);
                return FAILURE;
                }            
            }   /* end-while (more in original file) */

        fclose(filePtr);
        fclose(tempFilePtr);

        if (rename(completeTempFileName, completeFileName) != 0)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                      NULL, modUNIX, completeTempFileName);
            return FAILURE;
            }
        }    /* end-if line found */
    else
        {
        /* Write the new line in the file; append a new-line character
         * - if there is not already one!!
         */
        if (fputs(keywordLine, filePtr) == EOF)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                      NULL, modUNIX, completeFileName);
            fclose(filePtr);
            return FAILURE;
            }  
        if (*(keywordLine + strlen(keywordLine) - 1) != '\n')
            {
            if (fwrite(&newLine, sizeof(char), (size_t)1, 
                       filePtr) != (size_t)1)
                {
                errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                          NULL, modUNIX, completeFileName);
                fclose(filePtr);
                return FAILURE;
                }  
            }
        fclose(filePtr);
        }    /* else-append line */

    return SUCCESS;
}


/*
 *****************************************************************************
 */



/*___oOo___*/

