#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxLocateSetupFile.test.tcl,v 1.145 2005/07/27 11:08:07 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxLocateSetupFile.test.prog - test program for slxLocateSetupFile()
#
#   SYNOPSIS
#   slxLocateSetupFile.test.prog
#
#   DESCRIPTION
#   This script tests the slxLocateSetupFile program, as follows:
#	1.1 not existing FITS header file, INS_SETUPPATH not set - SKIPPED!!!
#	1.2 not existing FITS header file, INS_SETUPPATH set
#	1.3 INS_SETUPPATH has wrong format
#	1.4 INS_SETUPPATH contains non-existing directories
#	1.5 non-existing file, INS_SETUPPATH contains ".."
#	1.6 Second argument has ".det" extension
#	1.7 Fourth argument refers to not existing dictionary
#	1.8 First argument refers to not existing image file
#	1.9 First argument refers to read-protected file
#	1.10 Second argument describes already existing file
#	1.11 Setup file template is not in $INS_SETUPPATH
#	1.15 First argument describes non-FITS file
#	1.16 Second argument refers to write-protected directory
#
#	2.1 Normal execution of slxLocateSetupFile - file is in 1st directory
#	    of $INS_SETUPPATH
#	2.2 Normal execution of slxLocateSetupFile - file is in 2nd directory
#	    of $INS_SETUPPATH
#	2.3 Normal execution of slxLocateSetupFile - complete filepath is given
#
#   To run these tests, the program slxSetupToShortFITS is used.  The tests
#   for the latter program should therefore be run before this one.
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   FILES
#   The following files are required and accessed under ~slx/test:
#	- slxFitsHeader.test.bs (tests 2.1, 2.2, 2.3)
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------
source ../test/defs
source ../test/slxTestCommon.tcl

# This test program will use the services of the program "slxSetupToShortFITS".
# It is therefore expected that the test suite for the latter was run 
# successfully before.

# For more info, see also the test suite for "slxSetupToShortFITS".
set slxSetupToShortFITS slxSetupToShortFITS


# PART 1: produce errors
# ======================
catch {unset env(INS_SETUPPATH)} msg

# IMPL.: Temporary skipped - how is it possible to unset INS_SETUPPATH
#        when "exec" is used?
if {1 == 0} {
# ----------------------------------------------------------------------------
# Not existing file, and INS_SETUPPATH is not set
test slxLocateSetupFile-1.1 {not existing FITS header file, INS_SETUPPATH not set} {
	catch "exec $slxSetupToShortFITS sureUcantFindThis1 -t" msg
	set msg
}   "\nCouldn't convert the file - check if filename is correct!\n
slxERR_INS_SETUPPATH : Environment variable INS_SETUPPATH is not set"}

# ----------------------------------------------------------------------------
# Not existing file, and INS_SETUPPATH is set
test slxLocateSetupFile-1.2 {not existing FITS header file, INS_SETUPPATH set} {
	set env(INS_SETUPPATH) [pwd]
	catch "exec $slxSetupToShortFITS sureUcantFindThis1 -t" msg
	unset env(INS_SETUPPATH)
	set msg
}   "\nCouldn't convert the file - check if filename is correct!\n
slxERR_LOCATE_FILE : Could not find \"sureUcantFindThis1\" according to\
INS_SETUPPATH or path given with filename"

# ----------------------------------------------------------------------------
# Not existing file, and INS_SETUPPATH is not correctly set
test slxLocateSetupFile-1.3 {INS_SETUPPATH has wrong format} {
	set env(INS_SETUPPATH) "[pwd]\;.."
	catch "exec $slxSetupToShortFITS sureUcantFindThis1 -t" msg
	unset env(INS_SETUPPATH)
	set msg
}   "\nCouldn't convert the file - check if filename is correct!\n
slxERR_LOCATE_FILE : Could not find \"sureUcantFindThis1\" according to INS_SETUPPATH or path given with filename"

# ----------------------------------------------------------------------------
# Not existing file, and INS_SETUPPATH is set
test slxLocateSetupFile-1.4 {INS_SETUPPATH contains non-existing directories} {
	set env(INS_SETUPPATH) "[pwd]:/ThisDirectory/DoesNot/Exist"
	catch "exec $slxSetupToShortFITS sureUcantFindThis1 -t" msg
	unset env(INS_SETUPPATH)
	set msg
}   "\nCouldn't convert the file - check if filename is correct!\n
slxERR_LOCATE_FILE : Could not find \"sureUcantFindThis1\" according to\
INS_SETUPPATH or path given with filename"

# ----------------------------------------------------------------------------
# Not existing file, and INS_SETUPPATH contains ..
test slxLocateSetupFile-1.5 {non-existing file, INS_SETUPPATH contains ".."} {
	set env(INS_SETUPPATH) $env(PWD)/../test
	catch "exec $slxSetupToShortFITS sureUcantFindThis1 -t" msg
	unset env(INS_SETUPPATH)
	set msg
}   "\nCouldn't convert the file - check if filename is correct!\n
slxERR_LOCATE_FILE : Could not find \"sureUcantFindThis1\" according to\
INS_SETUPPATH or path given with filename"

# PART 2: test functionality
# ==========================

# ---------------------------------------------------------------------------
# file is to be found in $INS_SETUPPATH
test slxLocateSetupFile-2.1 {file is in 1st directory of $INS_SETUPPATH} {
    set tf slxFitsHeader.test
    set env(INS_SETUPPATH) "[pwd]:..:../.."
    catch "exec chmod ugo=+rwx $tf.temp"
    exec cp $tf.bs $tf.temp
    exec chmod ugo=+rwx $tf.temp
    if {[catch "exec $slxSetupToShortFITS $tf.temp -t" msg] == 0} {
	set msg [exec diff $tf.temp $tf.rf]
    } 
    unset env(INS_SETUPPATH)
    catch "exec chmod ugo=+rwx $tf.temp"
    exec rm $tf.temp
    set msg
}   ""

# ---------------------------------------------------------------------------
# file is to be found in $INS_SETUPPATH
test slxLocateSetupFile-2.2 {file is in 2nd directory of $INS_SETUPPATH} {
    set tf slxFitsHeader.test
    set env(INS_SETUPPATH) "..:[pwd]:../.."
    catch "exec chmod ugo=+rwx $tf.temp"
    exec cp $tf.bs $tf.temp
    exec chmod ugo=+rwx $tf.temp
    if {[catch "exec $slxSetupToShortFITS $tf.temp -t" msg] == 0} {
	set msg [exec diff $tf.temp $tf.rf]
    } 
    unset env(INS_SETUPPATH)
    catch "exec chmod ugo=+rwx $tf.temp"
    exec rm $tf.temp
    set msg
}   ""

# ---------------------------------------------------------------------------
# file path is given, $INS_SETUPPATH not set
test slxLocateSetupFile-2.3 {file path is given} {
    set tf slxFitsHeader.test
    catch "exec chmod ugo=+rwx $tf.temp"
    exec cp $tf.bs $tf.temp
    exec chmod ugo=+rwx $tf.temp
    if {[catch "exec $slxSetupToShortFITS [pwd]/$tf.temp -t" msg] == 0} {
	set msg [exec diff $tf.temp $tf.rf]
    } 
    catch "exec chmod ugo=+rwx $tf.temp"
    exec rm $tf.temp
    set msg
}   ""


# --- oOo ---

