#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxMergeSetupFiles.test.tcl,v 1.145 2005/07/27 11:17:46 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxMergeSetupFiles.test.tcl - test program for slxMergeSetupFiles
#
#   SYNOPSIS
#   slxMergeSetupFiles.test.tcl
#
#   DESCRIPTION
#   This script tests the slxMergeSetupFiles program, as follows:
#	1.1 wrong number of arguments: 3
#	1.2 wrong number of arguments: 3, plus "-t" option
#	1.3 1st argument does not have valid extension
#	1.4 1st argument must be absolute path
#	1.5 3rd argument describes non-existing file - SKIPPED!!!
#	1.6 3rd argument describes read-protected file
#	1.7 3rd argument describes file not in short-FITS format
#	1.8 1st argument describes already existing file
#	1.9 cannot find setup file template
#	1.13 format of 1st setupfile is wrong
#
#	2.1 Normal execution of slxMergeSetupFiles - merge a setup file with 
#	    itself
#	2.2 Normal execution of slxMergeSetupFiles - merge two setup files
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   FILES
#   The following files are required and accessed under ~slx/test:
#	- slxFitsHeader.test.bs (tests 1.6)
#	- slxMergeSetupFiles.test.tcl (test 1.7)
#	- slxMergeSetupFiles.test1.8.bs (tests 1.8, 1.9, 1.14)
#	- slxMergeSetupFiles.test1.13.bs (test 1.13)
#	- slxMergeSetupFiles.test2.1.bs (tests 2.1, 2.2)
#	- slxMergeSetupFiles.test2.1.rf (test 2.1)
#	- slxMergeSetupFiles.test2.2.bs (test 2.2)
#	- slxMergeSetupFiles.test2.2.rf (test 2.2)
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------

# Take over Ousterhout's test procedure 
source ../test/defs
cd ../test

set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM

catch {unset env(INS_SETUPPATH)} msg
set slxMergeSetupFiles slxMergeSetupFiles



# In case of errors, slxMergeSetupFiles does an exit(1), so from Tcl
# we have to catch this error

# PART 1: argument testing
# ========================

# ---------------------------------------------------------------------------
# Produce error for wrong number of parameters (at least 4 needed, not counting
# the verbose option)
if {1} {
test slxMergeSetupFiles-1.1 {number of arguments: 3} {
	catch "exec $slxMergeSetupFiles A B C" msg
	set msg
}   "
Correct syntax is:

slxMergeSetupFiles <new setup.<type>> <setup ID> <dictionary>\\
                   <setup file 1> <setup file 2>\\
                   ... <setup file n> \[-t\]
"
}


# ---------------------------------------------------------------------------
# Produce error for wrong number of parameters (at least 4 needed)
test slxMergeSetupFiles-1.2 {number of arguments: 3, plus -t option} {
	catch "exec $slxMergeSetupFiles A B GEN C -t" msg
	set msg
}   "
Correct syntax is:

slxMergeSetupFiles <new setup.<type>> <setup ID> <dictionary>\\
                   <setup file 1> <setup file 2>\\
                   ... <setup file n> \[-t\]
"

# ---------------------------------------------------------------------------
# Produce error for 1st parameter not having valid extension
test slxMergeSetupFiles-1.3 {1st argument does not have valid extension} {
	catch "exec $slxMergeSetupFiles A.xyz B GEN C D" msg
	set msg
}   "\nError in output setup filename; extension must be:\n
.ref, .ins, .det or .targ\n"


# ---------------------------------------------------------------------------
# Produce error for 1st parameter not absolute path
test slxMergeSetupFiles-1.4 {1st argument must be absolute path} {
	catch "exec -keepnewline $slxMergeSetupFiles A.ref B GEN C D E -t" msg
	set msg
}   "\nError, name of new setup file must be complete name!!\n\n"

# ---------------------------------------------------------------------------
# Produce error for first setupfile (par 3) not found
cd ../templates
set tempDir [pwd]
cd ../test
set env(INS_SETUPPATH) [pwd]:$tempDir
test slxMergeSetupFiles-1.5 {4th argument describes non-existing file} {
	set p1 [pwd]/A.ref
	catch "exec $slxMergeSetupFiles $p1 ID GEN C D -t" msg
	set msg
}   "\nContents of CCS Error Structure!!

   ModuleId   : slx
   LocationId : slxLocateSetupFile.c
   ErrorNumber: 37
   Parameters : slxERR_LOCATE_FILE : Could not find \"C\" according to INS_SETUPPATH or path given with filename"

# ---------------------------------------------------------------------------
# Produce error for first setupfile (par 3) read protected
set tf slxFitsHeader.test
cd ../test
set pwdPath [pwd]
test slxMergeSetupFiles-1.6 {3rd argument describes read-protected file} {
    set p1 $pwdPath/A.ref
    set env(INS_SETUPPATH) $pwdPath
    catch "exec chmod ugo=+rwx $tf.temp"
    exec cp $tf.bs $tf.temp
    chmod 0222 $tf.temp
    catch "exec $slxMergeSetupFiles $p1 ID GEN $tf.temp D -t" msg
    catch "exec chmod ugo=+rwx $tf.temp"
    catch "exec rm $tf.temp"
    set msg
}   "\nContents of CCS Error Structure!!

   ModuleId   : slx
   LocationId : slxDetectFormat.c
   ErrorNumber: 1
   Parameters : slxERR_OPEN_FILE : Could not open file \"slxFitsHeader.test.temp\""


# ---------------------------------------------------------------------------
# Produce error for first setupfile (par 3) not in short-FITS format
set tf slxMergeSetupFiles.test.tcl
cd ../test
set pwdPath [pwd]
test slxMergeSetupFiles-1.7 {3rd argument describes file not in short-FITS format} {
    set p1 $pwdPath/A.ref
    set env(INS_SETUPPATH) $pwdPath
    exec cp $tf $tf.temp
    exec chmod ugo=+rwx $tf.temp
    catch "exec $slxMergeSetupFiles $p1 ID GEN $tf.temp D -t" msg
    catch "exec rm $tf.temp"
    set msg
}   "\nContents of CCS Error Structure!!

   ModuleId   : slx
   LocationId : slxMergeSetupBuf.c
   ErrorNumber: 3
   Parameters : slxERR_FILE_FORMAT : File \"slxMergeSetupFiles.test.tcl.temp\" does not have the correct format"


# ---------------------------------------------------------------------------
# Produce error for resulting setupfile (par 1) already existing
set tf slxMergeSetupFiles.test1.8
cd ../test
set pwdPath [pwd]
test slxMergeSetupFiles-1.8 {1st argument describes already existing file} {
    set env(INS_SETUPPATH) $pwdPath
    exec cp $tf.bs $tf.ins
    exec chmod ugo=+rwx $tf.ins
    exec cp $tf.bs $tf.temp
    exec chmod ugo=+rwx $tf.temp
    set p1 $pwdPath/$tf.ins
    catch "exec $slxMergeSetupFiles $p1 ID GEN $tf.temp $tf.temp -t" msg
    catch "exec rm $tf.ins $tf.temp"
    set msg
}   "\nError: \"$pwdPath/$tf.ins\" exists already!\n"


# ---------------------------------------------------------------------------
# Produce error for copying setupfile template => code does not do this!!

# ---------------------------------------------------------------------------
# Produce error for creating short fits lines -> How????

# ---------------------------------------------------------------------------
# Produce error for storing line -> How????

# ---------------------------------------------------------------------------
# Produce error in format of 1st setupfile (par 4)
set tf slxMergeSetupFiles.test1.13
test slxMergeSetupFiles-1.13 {format of 1st setupfile is wrong} {
    set wd [pwd]
    cd $wd/../templates
    set env(INS_SETUPPATH) $wd:[pwd]
    cd $wd
    exec cp $tf.bs $tf.temp
    exec chmod ugo=+rwx $tf.temp 
    set p1 $wd/$tf.ins
    catch "exec -keepnewline $slxMergeSetupFiles \
	    $p1 ID GEN \
	    slxMergeSetupFiles.test1.13.temp \
	    slxMergeSetupFiles.test1.13.temp -t" msg  
    set msg
}   "
Contents of CCS Error Structure!!

   ModuleId   : misc
   LocationId : miscHash.c
   ErrorNumber: 100
   Parameters : miscERR_HASH_KEY_NOT_FOUND : The key \"IMPLE\" was not found in the Hash Table\n"
catch "exec rm $tf.temp"


# PART 2: functionality test
# ==========================
# ---------------------------------------------------------------------------
# merge a setup file with itself;
# if all is well, slxMergeSetupFiles should take an exit(0) without 
# additional messages.
# Keep the output of "diff", to restore after removing temporary files
set tf slxMergeSetupFiles.test2.1
cd ../test
set testDir [pwd]
catch "exec rm $testDir/slxMergeSetupFiles.test2.1.temp.ins \
	$testDir/slxMergeSetupFiles.test2.1.temp.ins2"
test slxMergeSetupFiles-2.1 {merge a setup file with itself} {
    set wd [pwd]
    cd $wd/../templates
    set env(INS_SETUPPATH) $wd:[pwd]
    if {![catch "exec $slxMergeSetupFiles \
	    $testDir/slxMergeSetupFiles.test2.1.temp.ins \
	    ID GEN \
	    $testDir/slxMergeSetupFiles.test2.1.bs \
	    $testDir/slxMergeSetupFiles.test2.1.bs -t" msg]} {

	exec egrep -v "SUP.CRTE.DATE|SUP.CRTE.UT|SUP.LCHG.DATE|SUP.LCHG.UT" \
		$testDir/slxMergeSetupFiles.test2.1.temp.ins \
		> $testDir/slxMergeSetupFiles.test2.1.temp.ins2
	catch "exec diff \
		$testDir/slxMergeSetupFiles.test2.1.rf \
		$testDir/slxMergeSetupFiles.test2.1.temp.ins2" msg
    }	
    set msg
} {}
catch "exec rm $testDir/slxMergeSetupFiles.test2.1.temp.ins \
	$testDir/slxMergeSetupFiles.test2.1.temp.ins2"


# ---------------------------------------------------------------------------
# Merge two different setupfiles
cd ../test
set testDir [pwd]
catch "exec rm $testDir/slxMergeSetupFiles.test2.2.temp.ins \
	$testDir/slxMergeSetupFiles.test2.2.temp.ins2"
test slxMergeSetupFiles-2.2 {merge two setup files} {
	set env(INS_SETUPPATH) $testDir
	if {![catch "exec $slxMergeSetupFiles \
		$testDir/slxMergeSetupFiles.test2.2.temp.ins \
		ID GEN \
		$testDir/slxMergeSetupFiles.test2.1.bs \
		$testDir/slxMergeSetupFiles.test2.2.bs -t" msg]} {

	 exec egrep -v "SUP.CRTE.DATE|SUP.CRTE.UT|SUP.LCHG.DATE|SUP.LCHG.UT" \
		 $testDir/slxMergeSetupFiles.test2.2.temp.ins \
		 > $testDir/slxMergeSetupFiles.test2.2.temp.ins2
	 catch "exec diff \
		 $testDir/slxMergeSetupFiles.test2.2.rf \
		 $testDir/slxMergeSetupFiles.test2.2.temp.ins2" msg
	}
	
	set msg
} {}
catch "exec rm $testDir/slxMergeSetupFiles.test2.2.temp.ins \
	$testDir/slxMergeSetupFiles.test2.2.temp.ins2"
